/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CDeviceDStd.c  :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CDeviceDStd.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CDeviceDStd_Init(CDeviceDStd *new)
{
    new->ClassID = (uint32)FmDEVICE << 16 | GnDATA << 8 | (uint8)CDEVDSTD_SpSTANDARD;
    new->DirtyFlag = 0;

    memset(&new->m_DateTime, 0, sizeof(new->m_DateTime));
    new->m_RunTime = 0;
    memset(&new->m_ModelName, 0, sizeof(new->m_ModelName));
    memset(&new->m_SerialNumber, 0, sizeof(new->m_SerialNumber));
    memset(&new->m_DeviceGUID, 0, sizeof(new->m_DeviceGUID));
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CDeviceDStd_GetPayloadSize(CDeviceDStd *pObject)
{
    uint32 Size = SIZEOF(uint16)                    /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the static struct*/
            + CDEVDSTD_STATIC_DATA_SIZE;            /*Static data size*/
    Size += SIZEOF(uint16);            /* CRC bytes */

    TOUCH(pObject);

    return Size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CDeviceDStd_SizeAll ( CDeviceDStd *pObject )
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CDeviceDStd_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CDeviceDStd object into the buffer passed in.
*********************************************************************************/
boolean CDeviceDStd_Serialize (CDeviceDStd *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    uint8 *pTemp = pBuf;
    
    if (pTemp != NULL)
    {
        uint32 CRCSize;
        uint16 i;
        boolean ByteAligned = TRUE;

        /* Set the ClassID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the Payload size */
        WriteInt16(&pTemp, (uint16)CDeviceDStd_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the object version */
        WriteInt16(&pTemp, (uint16)CDEVDSTD_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CDEVDSTD_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the m_SDateTime structure */
        WriteInt16(&pTemp, pObject->m_DateTime.year, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_DateTime.month, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_DateTime.day, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_DateTime.hour, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_DateTime.minute, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_DateTime.second, &ByteAligned);
        WriteInt8(&pTemp, pObject->m_DateTime.format, &ByteAligned);

        /* Set the Runtime */
        WriteInt32(&pTemp, pObject->m_RunTime, &ByteAligned);

        /* Write the packed data buffer based on the number of 8 bit octets in the
         * buffer divided by 2 since this is a 16 bit char platform 
         */             
        for(i=0; i< (CDEVDSTD_MODELNAME_SIZE/2); i++)
        {
            WriteInt16(&pTemp, pObject->m_ModelName[i], &ByteAligned);
        }       
        WriteInt8(&pTemp, (pObject->m_ModelName[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);

        for(i=0; i< (CDEVDSTD_SERIALNUMBER_SIZE/2); i++)
        {
            WriteInt16(&pTemp, pObject->m_SerialNumber[i], &ByteAligned);
        }       
        WriteInt8(&pTemp, (pObject->m_SerialNumber[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);

        for(i=0; i< (CDEVDSTD_DEVICEGUID_SIZE/2); i++)
        {
            WriteInt16(&pTemp, pObject->m_DeviceGUID[i], &ByteAligned);
        }       

        /* Set the CRC */        
        CRCSize = CDeviceDStd_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CDeviceDStd_Deserialize(CDeviceDStd *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;

    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;
        uint16  i;
        boolean ByteAligned = TRUE;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Deserialize the CDeviceDStd static members */
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_SDateTime */
        pObject->m_DateTime.year = ReadInt16(&pTemp, &ByteAligned);
        pObject->m_DateTime.month = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_DateTime.day = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_DateTime.hour = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_DateTime.minute = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_DateTime.second = ReadInt8(&pTemp, &ByteAligned);
        pObject->m_DateTime.format = ReadInt8(&pTemp, &ByteAligned);

        /* Set the Runtime */
        pObject->m_RunTime = ReadInt32(&pTemp, &ByteAligned);

        for(i=0; i<(CDEVDSTD_MODELNAME_SIZE/2); i++)
        {
            pObject->m_ModelName[i] = ReadInt16(&pTemp, &ByteAligned);
        }
        pObject->m_ModelName[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;

        for(i=0; i<(CDEVDSTD_SERIALNUMBER_SIZE/2); i++)
        {
            pObject->m_SerialNumber[i] = ReadInt16(&pTemp, &ByteAligned);
        }
        pObject->m_SerialNumber[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;

        for(i=0; i<(CDEVDSTD_DEVICEGUID_SIZE/2); i++)
        {
            pObject->m_DeviceGUID[i] = ReadInt16(&pTemp, &ByteAligned);
        }

        /* Calculate the CRC */
        CRCSize = CDeviceDStd_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}
 
#if defined(COOA_OBJECT_TEST)
boolean Test_CDeviceDStd(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint32 Result;
    uint8 *pTemp;
	boolean ByteAligned = TRUE;
    CDeviceDStd SrcObj;
    CDeviceDStd DestObj;

    CDeviceDStd_Init(&SrcObj);
    CDeviceDStd_Init(&DestObj);

    SrcObj.DirtyFlag = TRUE;
    SrcObj.m_DateTime.year = 2002;
    SrcObj.m_DateTime.month = 12;
    SrcObj.m_DateTime.day = 31;
    SrcObj.m_DateTime.hour = 2;
    SrcObj.m_DateTime.minute = 06;
    SrcObj.m_DateTime.second = 00;
    SrcObj.m_DateTime.format = 00;
    SrcObj.m_RunTime = 65555;

    //memcpy(&SrcObj.m_ModelName, "Model_6", SIZEOF("Model_6"));
    pTemp = (uint8*)SrcObj.m_ModelName;
    WriteInt8(&pTemp, 'M', &ByteAligned);
    WriteInt8(&pTemp, 'o', &ByteAligned);
    WriteInt8(&pTemp, 'd', &ByteAligned);
    WriteInt8(&pTemp, 'e', &ByteAligned);
    WriteInt8(&pTemp, 'l', &ByteAligned);
    WriteInt8(&pTemp, '6', &ByteAligned);

    //memcpy(&SrcObj.m_SerialNumber, "123456789", SIZEOF("123456789"));
    pTemp = (uint8*)SrcObj.m_ModelName;
    WriteInt8(&pTemp, '1', &ByteAligned);
    WriteInt8(&pTemp, '2', &ByteAligned);
    WriteInt8(&pTemp, '3', &ByteAligned);
    WriteInt8(&pTemp, '4', &ByteAligned);
    WriteInt8(&pTemp, '5', &ByteAligned);
    WriteInt8(&pTemp, '6', &ByteAligned);
    WriteInt8(&pTemp, '7', &ByteAligned);
    WriteInt8(&pTemp, '8', &ByteAligned);
    WriteInt8(&pTemp, '9', &ByteAligned);

    //memcpy(&SrcObj.m_DeviceGUID, "123456789012345", SIZEOF("123456789012345"));
    pTemp = (uint8*)SrcObj.m_ModelName;
    WriteInt8(&pTemp, '1', &ByteAligned);
    WriteInt8(&pTemp, '2', &ByteAligned);
    WriteInt8(&pTemp, '3', &ByteAligned);
    WriteInt8(&pTemp, '4', &ByteAligned);
    WriteInt8(&pTemp, '5', &ByteAligned);
    WriteInt8(&pTemp, '6', &ByteAligned);
    WriteInt8(&pTemp, '7', &ByteAligned);
    WriteInt8(&pTemp, '8', &ByteAligned);
    WriteInt8(&pTemp, '9', &ByteAligned);
    WriteInt8(&pTemp, '0', &ByteAligned);
    WriteInt8(&pTemp, '1', &ByteAligned);
    WriteInt8(&pTemp, '2', &ByteAligned);
    WriteInt8(&pTemp, '3', &ByteAligned);
    WriteInt8(&pTemp, '4', &ByteAligned);
    WriteInt8(&pTemp, '5', &ByteAligned);
    
    Result = CDeviceDStd_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CDeviceDStd_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif
/* ---------------------------End of DataClass CDeviceDStd -------------------------------- */
