/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CErrSvr.c      :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CErrSvr.h"

void CErrSvr_Init(CErrSvr *new, uint8 Genus)
{
    new->ClassID = (uint32)FmTRAP << 16 | Genus << 8 | (uint8)CES_SpNONE;
    new->ErrorClassID = 0;
    new->DataSize = 0;
    memset(new->Data, 0, MAX_DATA_MEMBER_SIZE);
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CErrSvr_GetPayloadSize(CErrSvr *pObject)
{
    uint32 size  = SIZEOF(uint16)         /*Size of the data array*/
                 + CES_STATIC_STRUCT_SIZE /* Size of static struct */
                 + SIZEOF(uint32)         /*Size of the data array*/
                 + pObject->DataSize      /*data array size*/
                 + SIZEOF(uint16);        /* CRC bytes */

    return size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CErrSvr_SizeAll(CErrSvr *pObject)
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CErrSvr_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CErrSvr object into the buffer passed in.
*********************************************************************************/
boolean CErrSvr_Serialize(CErrSvr *pObject, uint8 *pBuf)
{
    uint8   *pTemp = pBuf;
    //byte    EncryptByte = 0;
    boolean Result = TRUE;
    boolean ByteAligned = TRUE; 
    
    if (pTemp != NULL)
    {
        uint32 CRCSize;

        /* Serialize the DtData members */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        WriteInt16(&pTemp, (uint16)CErrSvr_GetPayloadSize(pObject), &ByteAligned);
        
        /* Serialize the CErrSvr members */
        /* Set the Encryption byte */
        //*pTemp++ =  EncryptByte;

        /* Set the static struct size */
        WriteInt16(&pTemp, (uint16)CES_STATIC_STRUCT_SIZE, &ByteAligned);

        /* Set the Error ClassID*/
        WriteInt32(&pTemp, pObject->ErrorClassID, &ByteAligned);

        /* Set the Data size */
        WriteInt32(&pTemp, pObject->DataSize, &ByteAligned);
        
        /* Set the Data */
        if (pObject->DataSize > 0)
        {
            uint16 i;

            /* Write the packed data buffer based on the number of 8 bit octets in the
             * buffer divided by 2 since this is a 16 bit char platform 
             */             
            for(i=0; i< (pObject->DataSize/2); i++)
            {
                WriteInt16(&pTemp, pObject->Data[i], &ByteAligned);
            }
            
            /* Check for remaining byte from above division */
            if (pObject->DataSize % 2 == 1)             
                WriteInt8(&pTemp, (pObject->Data[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);
        }

        /* Set the CRC */
        CRCSize = CErrSvr_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}
    
/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CErrSvr_Deserialize( CErrSvr *pObject, uint8 *pBuf)
{   
    uint8   *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;

    if (pTemp != NULL)
    {
        uint32  Size; 
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        Size = ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the Encrypt/Compression Byte */
        //EncryptByte = *pTemp++;
        
        /* Get the static struct size */
        Size = ReadInt16(&pTemp, &ByteAligned);

        /* Get the Error ClassID */
        pObject->ErrorClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Deserialize the CErrSvr dynamic members */
        /* Get the DataSize */
        pObject->DataSize = ReadInt32(&pTemp, &ByteAligned);
        
        /* Set the Data */
        if (pObject->DataSize > 0)
        {
            uint16 i;
            
            for(i=0; i<(pObject->DataSize/2); i++)
                pObject->Data[i] = ReadInt16(&pTemp, &ByteAligned);

            if (pObject->DataSize % 2 == 1)
                pObject->Data[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;
        }
        else
            memset(pObject->Data, 0, MAX_DATA_MEMBER_SIZE);

        /* Calculate the CRC */
        CRCSize = CErrSvr_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
            
        TOUCH(Size);
    }

    return Result;
}


