/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CModuleDStd.c  :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CModuleDStd.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CModuleDStd_Init(CModuleDStd *new, uint8 Species)
{
    new->ClassID = (uint32)FmMODULE << 16 | GnDATA << 8 | (uint8)Species;
    new->DirtyFlag = 0;
    memset(new->m_Version, 0, VERSION_SIZE);      
    memset(new->m_Description, 0, DESCRIPTION_SIZE);  
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CModuleDStd_GetPayloadSize(CModuleDStd *pObject)
{
    uint32 Size = SIZEOF(uint16)                    /*Object Version*/
            + SIZEOF(boolean)                       /*is dirty flag*/
            + SIZEOF(uint16)                        /*Size of the static struct*/
            + CMODDSTD_STATIC_DATA_SIZE;            /*Static data size*/

    Size += SIZEOF(uint16);            /* CRC bytes */

    return Size;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CModuleDStd_SizeAll ( CModuleDStd *pObject )
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CModuleDStd_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CModuleDStd object into the buffer passed in.
*********************************************************************************/
boolean CModuleDStd_Serialize (CModuleDStd *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;
    
    uint8 *pTemp = pBuf;

    if (pTemp != NULL)
    {
        uint32 CRCSize;
        uint16 i;

        /* Set the ClassID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the Payload size */
        WriteInt16(&pTemp, (uint16)CModuleDStd_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the object version */
        WriteInt16(&pTemp, (uint16)CMODDSTD_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CMODDSTD_STATIC_DATA_SIZE, &ByteAligned);

        /* Write the packed data buffer based on the number of 8 bit octets in the
         * buffer divided by 2 since this is a 16 bit char platform 
         */             
        for(i=0; i< (VERSION_SIZE/2); i++)
        {
            WriteInt16(&pTemp, pObject->m_Version[i], &ByteAligned);
        }
        
        /* Check for remaining byte from above division */
        if (VERSION_SIZE % 2 == 1)             
            WriteInt8(&pTemp, (pObject->m_Version[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);

        /* Write the packed data buffer based on the number of 8 bit octets in the
         * buffer divided by 2 since this is a 16 bit char platform 
         */             
        for(i=0; i< (DESCRIPTION_SIZE/2); i++)
        {
            WriteInt16(&pTemp, pObject->m_Description[i], &ByteAligned);
        }
        
        /* Check for remaining byte from above division */
        if (DESCRIPTION_SIZE % 2 == 1)             
            WriteInt8(&pTemp, (pObject->m_Description[i] & HIGH_BYTE_MASK) >> 8, &ByteAligned);

        /* Set the CRC */
        CRCSize = CModuleDStd_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CModuleDStd_Deserialize(CModuleDStd *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;

    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;
        uint16  i;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Deserialize the CModuleDStd static members */
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_Version */
        for(i=0; i<(VERSION_SIZE/2); i++)
            pObject->m_Version[i] = ReadInt16(&pTemp, &ByteAligned);

        if (VERSION_SIZE % 2 == 1)
            pObject->m_Version[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;

        /* Get the m_Description */
        for(i=0; i<(DESCRIPTION_SIZE/2); i++)
            pObject->m_Description[i] = ReadInt16(&pTemp, &ByteAligned);

        if (DESCRIPTION_SIZE % 2 == 1)
            pObject->m_Description[i] = (ReadInt8(&pTemp, &ByteAligned) << 8) & HIGH_BYTE_MASK;

        /* Calculate the CRC */
        CRCSize = CModuleDStd_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}


#if defined(COOA_OBJECT_TEST)
boolean Test_CModuleDStd(void)
{
    uint8   Buffer[MAX_DATA_MEMBER_SIZE];
    uint32  Result = 0;
    uint8   *pTemp;
    boolean ByteAligned = TRUE;

    CModuleDStd SrcObj;
    CModuleDStd DestObj;

    CModuleDStd_Init(&SrcObj, CMODDSTD_SpBOOTLOADER);
    CModuleDStd_Init(&DestObj, CMODDSTD_SpBOOTLOADER);
    
    SrcObj.DirtyFlag = TRUE;

    pTemp = (uint8*)SrcObj.m_Version;
    WriteInt8(&pTemp, '1', &ByteAligned);
    WriteInt8(&pTemp, '.', &ByteAligned);
    WriteInt8(&pTemp, '2', &ByteAligned);
    WriteInt8(&pTemp, '.', &ByteAligned);
    WriteInt8(&pTemp, '3', &ByteAligned);

    pTemp = (uint8*)SrcObj.m_Description;
    ByteAligned = TRUE;
    WriteInt8(&pTemp, 'T', &ByteAligned);
    WriteInt8(&pTemp, 'e', &ByteAligned);
    WriteInt8(&pTemp, 's', &ByteAligned);
    WriteInt8(&pTemp, 't', &ByteAligned);
    WriteInt8(&pTemp, '1', &ByteAligned);

    Result = CModuleDStd_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CModuleDStd_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif
/* ---------------------------End of DataClass CModuleDStd -------------------------------- */
