/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CErrSvr.c      :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CTrapObjSvr.h"

static uint8 Outbuf[MAX_DATA_MEMBER_SIZE];

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
    CTrapObjSvr Start up: 

*********************************************************************************/
void CTrapObjSvr_Init(CTrapObjSvr *new)
{
    new->TxOptions = CTS_SpNONE;

    /* Initialize the general error servers and error objects */
    CError_Init(&new->TrapErrObj, CERR_SpWACPSTACK);
    CErrSvr_Init(&new->TrapErrSvr, GnERROR);
    CErrSvr_Init(&new->TrapErrCfgSvr, GnCONFIG);
    
    /* Clear the Outbuf */
    memset(Outbuf, 0, sizeof(Outbuf));
}

/*================================================================================
CTrapObjSvr TxErrMsg : Helper routine that transmits the contents of a CErrSvr object
================================================================================*/
boolean CTrapObjSvr_TxErrMsg(CErrSvr *pCErrSvr)
{
    uint32 OutBufSz;
    boolean Result = FALSE;

    /* Get the Size of the serialized byte Stream */
    OutBufSz = CErrSvr_SizeAll(pCErrSvr);

    /* Serialize the CErrSvr object */
    CErrSvr_Serialize(pCErrSvr, Outbuf);

    /* Send the message */
    Result = InterChange_SendPacket(Outbuf, OutBufSz);

    return(Result);
}

/*================================================================================
CTrapObjSvr SetConfig : Sends a configuration message to sync the type of information
                     that is sent/received for acknowledgements and errors. 
================================================================================*/
boolean CTrapObjSvr_SetConfig(CTrapObjSvr *this, uint16 NewOption)
{
    boolean Result = FALSE;

    /* Update the current configuration */
    this->TxOptions = NewOption;
    
    this->TrapErrCfgSvr.ClassID = SET_SPECIES(this->TrapErrCfgSvr.ClassID, (uint8)NewOption);

    Result = CTrapObjSvr_TxErrMsg(&this->TrapErrCfgSvr);

    return (Result);
}

/*================================================================================
CTrapObjSvr TxError : Sends a error message ACK/NACK with or without a textual 
                   description based on the TxOptions.  
================================================================================*/
boolean CTrapObjSvr_TxError(CTrapObjSvr *this, uint32 ClassID, uint8 ErrorType, uint8* pBuf, uint32 Size)
{
    boolean Result = FALSE;

    /* Set the ClassID */
    this->TrapErrSvr.ErrorClassID = ClassID;

    /* Set the Species to the right error type */
    this->TrapErrSvr.ClassID = SET_SPECIES(this->TrapErrSvr.ClassID, ErrorType);
    
    if (pBuf)
    {
        /* Copy the data into the dynamic data buffer */
        memcpy(this->TrapErrSvr.Data, pBuf, (Size/2) + (Size %2));
        this->TrapErrSvr.DataSize = Size;
    }
    else
    {
        memcpy(this->TrapErrSvr.Data, 0, MAX_DATA_MEMBER_SIZE);
        this->TrapErrSvr.DataSize = 0;
    }

    /* Send the message */
    Result = CTrapObjSvr_TxErrMsg(&this->TrapErrSvr);

    return Result;
}

/*================================================================================
TRAP SendError: Internal call for sending a CERR_SpWACPSTACK error. It takes the
    ClassID and ErrorCode to create a CError object byte stream and sends it via
    TxError(). If the ErrorCode is CES_SpNONE, then just send a simple ACK.
================================================================================*/
boolean CTrapObjSvr_SendError(CTrapObjSvr *this, uint32 ClassID, uint8 ErrorCode)
{
    boolean Result=FALSE;
    uint32  MsgSize;

    switch(this->TxOptions)
    {
        case CTS_SpNONE:
            /* Do not allow internal TRAP messages for CTS_SpNONE */
            break;
        case CTS_SpACK:
        {
            Result = CTrapObjSvr_TxError(this, ClassID, ErrorCode, NULL, 0);
            break;
        }
        case CTS_SpDESC:
        {
            if (ErrorCode == CES_SpNONE)
            {
                Result = CTrapObjSvr_TxError(this, ClassID, ErrorCode, NULL, 0);
            }
            else
            {
                /* Create a new instance of the error object */
                CError_Init(&this->TrapErrObj, CERR_SpWACPSTACK);

                /* Set the ErrorCode */
                this->TrapErrObj.ErrorCode = ErrorCode;

                /* Set the ClassID */
                this->TrapErrObj.ErrorClassID = ClassID;

                /* Set the Error Description */
                // N/A for DSP baseline.

                /* Get the Size of the error object */
                MsgSize = CError_SizeAll(&this->TrapErrObj);

                /* Serialize the error object */
                CError_Serialize(&this->TrapErrObj, Outbuf);

                /* Send the message */
                Result = CTrapObjSvr_TxError(this, ClassID, CES_SpERROR, Outbuf, MsgSize);
            }
        }
    }
    return Result;
}

/*================================================================================
CTrapObjSvr RecvMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
     the messages with the appropriate object.
================================================================================*/
boolean CTrapObjSvr_RecvMsgHandler (CTrapObjSvr *this,  uint8* pBuf)
{
    uint16  Family;
    uint8   Genus;
    uint8   Species;
    boolean RetVal=TRUE;

    /* Extract the Family */
    Family = GET_FAMILY(pBuf);

    /* Extract the Genus */
    Genus = GET_GENUS(pBuf);

    /* Extract the Species */
    Species = GET_SPECIES(pBuf);
    
    /* Genus Router */
    switch (Genus)
    {           
        case GnERROR:
        {           
            /* Set the Species */
            this->TrapErrSvr.ClassID = SET_SPECIES(this->TrapErrSvr.ClassID, Species);

            /* Deserialize the data and check for errors */
            if (CErrSvr_Deserialize(&this->TrapErrSvr, pBuf))
            {
                uint8   *pErrBuf = this->TrapErrSvr.Data;

                /* Setting pErrBuf to NULL means this is a simple ACK/NACK message */
                if ( this->TrapErrSvr.DataSize == 0 )
                    pErrBuf = NULL;

                WACP_RecvMsg(Family, Genus, Species, pErrBuf);
            }
            
            break;
        }/* case GnERROR */

        case GnCONFIG:
        {
            /* Set the Species */
            this->TrapErrCfgSvr.ClassID = SET_SPECIES(this->TrapErrCfgSvr.ClassID, Species);

            if (CErrSvr_Deserialize(&this->TrapErrCfgSvr,pBuf))
            {
                this->TxOptions = Species;
            }
            break;
        }

        default:
            break;

    }/* switch (Genus)*/

    return RetVal;
}
    




