/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWaveSampleCRPDetect.c :                                          //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CWaveSampleCRPDetect.h"

/*********************************************************************************
Initializes the structure and creates a default class ID
*********************************************************************************/
void CWaveSampleCRPDetect_Init(CWaveSampleCRPDetect *new)
{
    new->ClassID = (uint32)FmWAVESAMPLE << 16 | GnCONFIG << 8 | (uint8)CWVSCRPD_SpRWAVDETCFG;
    new->DirtyFlag = 0;
    new->m_IsActive = 0;
    new->m_DetectChannel = 0;
}

/*********************************************************************************
Returns the number of bytes that would be used by the Serialize and
Deserialize methods.
*********************************************************************************/
uint32 CWaveSampleCRPDetect_GetPayloadSize(CWaveSampleCRPDetect *pObject )
{
    uint32 Size = SIZEOF(uint16)                        /*Object Version*/
            + SIZEOF(boolean)                           /*is dirty flag*/
            + SIZEOF(uint16)                            /*Size of the struct*/
            + CWVSCRPD_STATIC_DATA_SIZE;                /*Static data size*/

    Size += SIZEOF(uint16);            /* CRC bytes */

    TOUCH(pObject);

    return Size;

}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CWaveSampleCRPDetect_SizeAll (CWaveSampleCRPDetect *pObject)
{
    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return SIZEOF(uint32) + SIZEOF(uint16) + CWaveSampleCRPDetect_GetPayloadSize(pObject);
}

/*********************************************************************************
Serializes the contents of the CWaveSampleCRPDetect object into the buffer passed in.
*********************************************************************************/
boolean CWaveSampleCRPDetect_Serialize(CWaveSampleCRPDetect *pObject, uint8 *pBuf)
{
    boolean Result = TRUE;
    boolean ByteAligned = TRUE;    
    uint8 *pTemp = pBuf;

    if (pTemp != NULL)
    {
        uint32 CRCSize;

        /* Set the ClassID */
        WriteInt32(&pTemp, pObject->ClassID, &ByteAligned);

        /* Set the Payload size */
        WriteInt16(&pTemp, (uint16)CWaveSampleCRPDetect_GetPayloadSize(pObject), &ByteAligned);
        
        /* Set the object version */
        WriteInt16(&pTemp, (uint16)CWVSCRPD_OBJECT_VERSION, &ByteAligned);
        
        /* Set the dirty byte */
        WriteInt8(&pTemp, pObject->DirtyFlag, &ByteAligned);
        
        /* Set the static data size */
        WriteInt16(&pTemp, (uint16)CWVSCRPD_STATIC_DATA_SIZE, &ByteAligned);

        /* Set the m_IsActive */
        WriteInt8(&pTemp, pObject->m_IsActive, &ByteAligned);

        /* Set the m_DetectChannel */
        WriteInt8(&pTemp, pObject->m_DetectChannel, &ByteAligned);

        /* Set the CRC */
        CRCSize = CWaveSampleCRPDetect_SizeAll(pObject) - SIZEOF(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE), &ByteAligned);
    }
    else
        Result = FALSE;

    return Result; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CWaveSampleCRPDetect_Deserialize(CWaveSampleCRPDetect *pObject, uint8* pBuf)
{
    uint8 *pTemp = pBuf;
    boolean Result = FALSE;
    boolean ByteAligned = TRUE;
    
    if (pTemp != NULL)
    {
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;

        /* Serialize the DtData members */
        pObject->ClassID = ReadInt32(&pTemp, &ByteAligned);

        /* Get the size of the payload */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Deserialize the CWaveSampleCRPDetect static members */
        /* Get the Version */
        ReadInt16(&pTemp, &ByteAligned);
        
        /* Get the DirtyFlag */
        pObject->DirtyFlag = ReadInt8(&pTemp, &ByteAligned);
        
        /* Get the size of the static data */
        ReadInt16(&pTemp, &ByteAligned);

        /* Get the m_IsActive */
        pObject->m_IsActive = ReadInt8(&pTemp, &ByteAligned);

        /* Get the m_DetectChannel */
        pObject->m_DetectChannel = ReadInt8(&pTemp, &ByteAligned);

        /* Calculate the CRC */
        CRCSize = CWaveSampleCRPDetect_SizeAll(pObject) - SIZEOF(uint16);
        CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize, TRUE);

        /* Get the stored CRC */
        StoredCRC = ReadInt16(&pTemp, &ByteAligned);

        /* Check the CRC */
        if (CalcCRC == StoredCRC) 
            Result = TRUE;
    }

    return Result;
}

#if defined(COOA_OBJECT_TEST)
boolean Test_CWaveSampleCRPDetect(void)
{
    uint8 Buffer[MAX_DATA_MEMBER_SIZE];
    uint32 Result = 0;

    CWaveSampleCRPDetect SrcObj;
    CWaveSampleCRPDetect DestObj;

    CWaveSampleCRPDetect_Init(&SrcObj);
    CWaveSampleCRPDetect_Init(&DestObj);

    SrcObj.m_IsActive = TRUE;
    SrcObj.m_DetectChannel = 3;

    Result = CWaveSampleCRPDetect_Serialize(&SrcObj, Buffer);
    if (Result)
        Result = CWaveSampleCRPDetect_Deserialize(&DestObj, Buffer);

    return((boolean)Result);
}
#endif

/* ---------------------------End of DataClass CWaveSampleCRPDetect -------------------------------- */
