/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Wacp.c         :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "wacp.h"

/* Static buffer used to store the contents of a message to be sent */
static uint8 MsgBuffer[MAX_DATA_MEMBER_SIZE];

CTrapObjSvr     TrapServer;
CMsgSvr 		MsgSvr;

/********************************************************************************************
DataReadyEvent: Callback for messages received from the exchange layer.
********************************************************************************************/
void DataReadyEvent(uint8 *pBuf, uint32 Size)
{
    if (Size > 0)
    {
        uint32 Result = TRUE;
        uint16 Family = GET_FAMILY(pBuf);
        uint8  Genus = GET_GENUS(pBuf);
        uint8  Species = GET_SPECIES(pBuf);
     
        switch(Family)
        {        
            case FmTRAP:
            {
                /* CTrapObjSvr_RecvMsgHandler(&TrapServer, pBuf); */
                break;
            }
            default:
            {        
                uint8 ErrorType;
                uint8 *pData = pBuf;

                if (Genus != GnDATA)
                {
                    /* Initialize the MsgSvr structure */
                    CMsgSvr_Init(&MsgSvr, Family, Genus, Species);

                    /* Serialize the data */
                    Result  = CMsgSvr_Deserialize(&MsgSvr, pBuf);
                                    
                    /* Set the Data pointer to the Message server data */
                    pData = MsgSvr.Data;
	            }

                if (Result)
                {	
	                /* Route the message to the Wacp application */
	                Result = WACP_RecvMsg(Family, Genus, Species, pData);
    	            
	                /* Check the message status */
	                ErrorType = (uint8)(Result ? CES_SpNONE: CERR_UNIMPL);
    	            
                }
			    else
	        	    ErrorType = CERR_CRC_MISMATCH;
    			

                if (Genus != GnREQUEST)
                {
                    uint32 ClassID = (uint32)Family << 16 | (uint16)Genus << 8 | (uint8)Species;

                    /* Send ACK/NACK message */
                    CTrapObjSvr_SendError(&TrapServer, ClassID, ErrorType);
                }            
                break;
            }
        };
    }/* if (Size) */
    else
    {
	    uint8 ErrString[] = "CRC Mismatch";
	    uint8 *pTemp;
	    boolean ByteAligned=TRUE;
	    uint8 i;
		void *memoryslot ;
		
	    /* Create the CError Object */
		memoryslot = GetMem(sizeof(CError));

	    /* Initialize the CError Object. CERR_SpWACPSTACK is used for stack specific errors */
	    CError_Init((CError *)memoryslot, CERR_SpWACPSTACK);
	
	    /* Set the ErrorCode */
	    ((CError *)memoryslot)->ErrorCode = CERR_CRC_MISMATCH;
	    
	    /* Set the ClassID to 0 since we cannot be sure the ClassID of the message was not corrupted */ 
	    ((CError *)memoryslot)->ErrorClassID = 0;
	    	    
	    /* Set the Description of the Error */
	    pTemp = (uint8*)((CError *)memoryslot)->Description;
	    for (i=0; i<sizeof(ErrString); i++)
	        WriteInt8(&pTemp, ErrString[i] & 0xFF, &ByteAligned);
    
		outbox.ID = NP_OperationalError;
		outbox.Argument.ptr = memoryslot;
		NP_Post(&outbox);                          
    }
}

/********************************************************************************************
WACP_RecvMsg: Main interface method for receiving a WACP message.
********************************************************************************************/
boolean WACP_RecvMsg (uint16 Family, uint8 Genus, uint8 Species, 
                      uint8* Buffer)
{
    boolean Result = FALSE;

    TOUCH(Buffer);

    switch(Family)
    {
        case FmRENDEZVOUS:
        {
            /* Process the RNDZ message */
            Result = CRNDZApp_RxMsg(Genus, Species, MsgSvr.Data);
            break;
        }
        case FmWAVESAMPLE:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case WAVESAMPLE_CCMD_SpSTART_ACQUIRE:
                            break;
                        case WAVESAMPLE_CCMD_SpSTOP_ACQUIRE:
                            break;
                        case WAVESAMPLE_CCMD_SpWRITE_CONFIG:
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case WAVESAMPLE_CRST_SpGET_WAVE:
                            break;
                        case WAVESAMPLE_CRST_SpGET_CONFIG:
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case WAVESAMPLE_CRSP_SpPUT_WAVE:
                            break;
                        case WAVESAMPLE_CRSP_SpPUT_CONFIG:
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CWSPDSTD_SpSTANDARD:
                        case CWSPDSTD_SpECGWAVSMP:
                        case CWSPDSTD_SpAUDIOWAVSMP:
                        case CWSPDSTD_SpECGIMPEDWAVSMP:
                        case CWSPDSTD_SpRWAVDETECT:
                        case CWSPDSTD_SpPACEWAVDETECT:
                        {
                            break;
                        }
                    }
                    break;
                }
            }
            break;
        }
        case FmFLASH:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case FLASH_CCMD_SpWRITE_SEGMENT:
                            break;
                        case FLASH_CCMD_SpERASE_SECTOR:
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case FLASH_CRST_SpGET_CRC:
                            break;
                        case FLASH_CRST_SpGET_SEGMENT:
                            break;
                        case FLASH_CRST_SpGET_FLASH_CONFIG:
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        
                        case FLASH_CRSP_SpPUT_CRC:
                            break;
                        case FLASH_CRSP_SpPUT_SEGMENT:
                            break;
                        case FLASH_CRSP_SpPUT_FLASH_CONFIG:
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    break;
                }
            }
            break;
        }
        case FmMODULE:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case MODULE_CCMD_SpWRITE_CONFIG:
                            break;
                        case MODULE_CCMD_SpWRITE_MODULE_DESC:
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case MODULE_CRST_SpGET_MODULE_DESC:
                            break;
                        case MODULE_CRST_SpGET_CONFIG:
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        
                        case MODULE_CRSP_SpPUT_MODULE_DESC:
                            break;
                        case MODULE_CRSP_SpPUT_CONFIG:
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    break;
                }
            }
            break;
        }/*case FmMODULE:*/
        case FmTOC:
        {
            switch(Genus)
            {
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case TOC_CRST_SpGET_TOC:
                        {
                        }
                    }
                    break;
                }/*case GnREQUEST:*/
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case TOC_CRSP_SpPUT_TOC:
                        {
                        }
                    }
                    break;
                }/*case GnRESPONSE:*/
                case GnDATA:
                {

                    break;
                }

            }/*switch(Genus)*/
            break;
        }/*case FmTOC:*/
        case FmDEVICE:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case DEVICE_CCMD_SpWRITE_CONFIG:
                        {
                            break;
                        }
                        case DEVICE_CCMD_SpRESET:
                        {
                            break;
                        }
                        case DEVICE_CCMD_SpPOWER:
                        {
                            break;
                        }
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case DEVICE_CRST_SpGET_DEVICEDESCRIPTION:
                        {
                            break;
                        }
                        case DEVICE_CRST_SpGET_CONFIG:
                        {
                            break;
                        }
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION:
                        {
                            break;
                        }
                        case DEVICE_CRSP_SpPUT_CONFIG:
                        {
                            break;
                        }
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CDEVDSTD_SpSTANDARD:
                        {
                            break;
                        }
                    }
                    break;
                }
            }
            break;
        }/*FmDEVICE*/
        default:
            break;
    };
    return(Result);
}

/********************************************************************************************
WACP_SendMsg: Main interface method for sending a WACP message.
********************************************************************************************/
boolean WACP_SendMsg (uint16 Family, uint8 Genus, uint8 Species, 
                      uint8* Buffer, uint32 Size)
{
    uint32 MsgSize = 0;
    boolean Result = TRUE;

    switch(Genus)
    {
        case GnDATA:
        {
            /* Pass the data straight through */
            MsgSize = Size;
            memcpy(MsgBuffer, Buffer, MsgSize);
            break;
        }
        default:
        {
            /* Initialize the MsgSvr structure */
            CMsgSvr_Init(&MsgSvr, Family, Genus, Species);

            /* Copy the payload into the Data buffer */
            memcpy(MsgSvr.Data, Buffer, Size);

            /* Update the DataSize member */
            MsgSvr.DataSize = Size;

            /* Update the size */
            MsgSize = CMsgSvr_SizeAll(&MsgSvr);

            /* Serialize the data */
            Result = (boolean)CMsgSvr_Serialize(&MsgSvr, MsgBuffer);
            
            break;
        }
    }

    if (Result == TRUE)
    {
        /* Send the message */
        Result = InterChange_SendPacket(MsgBuffer, MsgSize);
    }
    
    return(Result);
                 
}


/********************************************************************************************
WACP_AppStart: Main interface starting the WACP connection.
Returns either TRUE if the stack was properly connected or FALSE if an error occurred.
********************************************************************************************/
boolean WACP_AppStart(void)
{
    boolean Result = FALSE;

    /* Initialize the Trap Object Server */
    CTrapObjSvr_Init(&TrapServer);

    /* Initialize the Interchange layer */
    InterChange_Init(DataReadyEvent);

    Result = CRNDZApp_AppStart();

    return(Result);
}

int test_main(void)
{      
    /* Start the WACP Stack. This starts the RNDZ connection sequence and sets up the stack
     * to start sending/receiving data... this should be moved into it's own task.
     */
    WACP_AppStart();
    
#if defined(COOA_OBJECT_TEST)
    /* Start the test example */
    Test();
#endif 

    return(TRUE);
    
}

#if defined(COOA_OBJECT_TEST)
/* Simulate the sending of a wavesample object and run through a short serialize/deserialize 
 * test of each cooa object
 */
void Test()
{
    char SampleBuf[] = "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    uint8  ObjBuffer[MAX_DATA_MEMBER_SIZE];
    CWaveSampleDStd Sample;

    uint32 ObjSize;
    int16  MsgCount = 5;
    uint16 Delay;
    int16  i;
    uint8 *pTemp;
    boolean ByteAligned = TRUE;
    
#if defined(COOA_OBJECT_TEST)
    boolean Result;

    Result = Test_CWaveSampleDStd();
    Result = Test_CWaveSampleCStd();
    Result = Test_CWaveSampleCRPDetect();
    Result = Test_CError();
    Result = Test_CFlashCfgDevice();
    Result = Test_CFlashCfgSector();
    Result = Test_CFlashDSectorCRC();
    Result = Test_CTOCDStd();
    Result = Test_CFlashDSegment();
    
    TOUCH(Result);
#endif
    
    /* Initialize the Sample object */
    CWaveSampleDStd_Init(&Sample, CWSPDSTD_SpSTANDARD);
    
    /* Set a temporary pointer to the data buffer */
    pTemp = (uint8*)&Sample.m_Sample;

    /* Byte pack the data into the buffer */
    for(i=0; i<73; i++)
    {
        WriteInt8(&pTemp, SampleBuf[i], &ByteAligned);      
    }   
    
    /* Set the size to number of 8-bit octets in the buffer */
    Sample.m_SampleSize = sizeof(SampleBuf);
    
    for(;;)
    {
        for (i=0; i < MsgCount; i++)
        {
            /* Set the sequence number */
            Sample.m_SequenceNumber = i;
            
            /* Get the number of 8-bit octets in the serialized stream */
            ObjSize = CWaveSampleDStd_SizeAll(&Sample);

            /* Serialize the data */
            CWaveSampleDStd_Serialize(&Sample, ObjBuffer);

            /* Send the data */
            WACP_SendMsg(FmWAVESAMPLE, GnDATA, 0, ObjBuffer, ObjSize);

            /* Wait the specified amount of time in ms */
            SYS_SLEEP(Delay);               
        }
    }
}


/* Example function of how to send a Trap Message */
void SendTrapExample()
{
    uint8 TempString[] = "Testing AppInternal Error";
    uint8 TempBuf[128];
    uint8 TempDescBuf[] = "CRCError";
    uint8 *pTemp;
    boolean ByteAligned=TRUE;
    uint8 i;
    /* Create the CError Object */
    CError TestErrObj;

    /* Initialize the CError Object. CERR_SpAPPINTERNAL is used for application specific errors */
    CError_Init(&TestErrObj, CERR_SpAPPINTERNAL);

    /* Set the ErrorCode */
    TestErrObj.ErrorCode = CERR_CRC_MISMATCH;
    
    /* Set the ClassID of the message that caused the error */
    TestErrObj.ErrorClassID = (uint32)FmWAVESAMPLE << 16 | (uint16)GnCOMMAND << 8 | 0;
    
    /* Set the Dynamic data buffer with additional data about the error */
	pTemp = TestErrObj.Data;
    for (i=0; i<sizeof(TempString); i++)
        WriteInt8(&pTemp, TempString[i] & 0xFF, &ByteAligned);
    
    /* Set the size of the Dynamic data */
    TestErrObj.DataSize = sizeof(TempString);
    
    /* Set the Description of the Error */
    pTemp = (uint8*)TestErrObj.Description;
    for (i=0; i<sizeof(TempDescBuf); i++)
        WriteInt8(&pTemp, TempDescBuf[i] & 0xFF, &ByteAligned);
        
    /* Serialize the object */
    CError_Serialize(&TestErrObj, TempBuf);    

    /* Send the Trap message */
    SEND_TRAP(TestErrObj.ErrorClassID, CES_SpERROR, (uint8*)&TempBuf, CError_SizeAll(&TestErrObj));
}
#endif




