/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CConObjSvr.h   :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#if !defined(CCONOBJSVR_H)
    #define  CCONOBJSVR_H

#include "DtSupport.h"
#include "DtData.h"
#include "Common.h"
#include "CWACPMutex.h"
#include "CWACPThread.h"
#include "CWACPSemaphore.h"
#include "CMsgSvr.h"
#include "Interchange.h"
#include <time.h>

typedef enum KEEPALIVESTATUS{
    KEEP_ALIVE_RUNNING = 0,
    KEEP_ALIVE_STOPPED,
    KEEP_ALIVE_TIMEOUT,
    KEEP_ALIVE_END
}KEEP_ALIVE_STATUS;

typedef  enum CONNCEVNTtypSPECIES{
    CONN_CEVNT_SpNONE = 0,
    CONN_CEVNT_SpCONNECTED,
    CONN_CEVNT_SpFAILED,
    CONN_CEVNT_SpMAX
}CONN_CEVNT_SPECIES;

typedef  enum CONNCRSTtypSPECIES{
    CONN_CRST_SpKEEPALIVE = 0,
    CONN_CRST_SpMAX
}CONN_CRST_SPECIES;

typedef  enum CONNCRSPtypSPECIES{
    CONN_CRSP_SpKEEPALIVE = 0,
    CONN_CRSP_SpMAX
}CONN_CRSP_SPECIES;

typedef  enum CONNCMDtypSPECIES{
    CONN_CMD_SpSHUTDOWN = 0,
    CONN_CMD_SpMAX
}CONN_CMD_SPECIES;

typedef  enum CONNCFGtypSPECIES{
    CONN_CFG_SpKEEPALIVEON = 0,
    CONN_CFG_SpKEEPALIVEOFF,
    CONN_CFG_SpKEEPALIVECHECK,
    CONN_CFG_SpMAX
}CONN_CFG_SPECIES;

typedef struct ConnObj
{
    uint8               TxOptions;
    KEEP_ALIVE_STATUS   KeepAliveStatus;
    time_t              TimeoutInterval;
    time_t              KeepAliveInterval;
    time_t              KeepAliveTimer;
    time_t              TimeoutTimer;
    char*               pSourceID;
}ConnObj;

typedef struct CConObjSvr
{
    boolean             ThreadStatus;
    CLinkTable*         pLinkTable;
    CMsgSvr             MsgSvr;           
    uint16              Count;
    CWACPMutex          ConnMutex;
    CWACPMutex          ExitMutex;
    CWACPThread         ConnObjThread;
    CWACPSemaphore      TimeOutSem;
}CConObjSvr;

uint32 WACPAPI CConObjSvr_KeepAliveThread(void* pData);
void           CConObjSvr_Init(CConObjSvr* pObject, CLinkTable *pLinkTable, int32 ThreadPriority);
void           CConObjSvr_Destroy(CConObjSvr* pObject);
ConnObj*       CConObjSvr_GetConnObj(const CConObjSvr* pObject, const char* pSourceID);

/*================================================================================
CConObjSvr  Connection methods
================================================================================*/
boolean     CConObjSvr_AddConnection(const CConObjSvr* pObject, const char* pSourceID);
boolean     CConObjSvr_RemoveConnection(const CConObjSvr* pObject, const char* pSourceID);
boolean     CConObjSvr_Disconnect(const CConObjSvr* pObject, const char* pSourceID);

/*================================================================================
CConObjSvr  Set Connection Transmission Options
================================================================================*/
boolean     CConObjSvr_SetTxOptions(const CConObjSvr* pObject, const char* pSourceID, uint8 Options);
uint8       CConObjSvr_GetTxOptions(const CConObjSvr* pObject, const char* pSourceID);
boolean     CConObjSvr_SetKeepAliveInterval(const CConObjSvr* pObject, const char* pSourceID, time_t Interval);
time_t      CConObjSvr_GetKeepAliveInterval(const CConObjSvr* pObject, const char* pSourceID);
boolean     CConObjSvr_SetTimeoutInterval(const CConObjSvr* pObject, const char* pSourceID, time_t TimeOut);
time_t      CConObjSvr_GetTimeoutInterval(const CConObjSvr* pObject, const char* pSourceID);

/*================================================================================
CConObjSvr  KeepAlive methods
================================================================================*/
boolean     CConObjSvr_EvaluateKeepAlive(const CConObjSvr* pObject, char* pSourceID); //Called once per DataThread evaluation
boolean     CConObjSvr_StartKeepAlive(const CConObjSvr* pObject, const char* pSourceID);    //Starts Keepalives and initializes the state
boolean     CConObjSvr_StopKeepAlive(const CConObjSvr* pObject, const char* pSourceID);     //Stops Keepalives and initializes the state

/*================================================================================
CConObjSvr  Callback registration
================================================================================*/
boolean     CConObjSvr_RecvMsgHandler (CConObjSvr* pObject, char* pSourceID, uint8* Buffer);

#endif /* #if !defined (CCONOBJSVR_H) */




