/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CEncryption.c  :                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CEncryption.h"
#include "AESCrypt.h"

#if defined(ENCRYPTION_ON)
/*Encryption Specific Functions*/
void ChunkAESEncrypt(uint8* Key, const uint8* Plaintext, uint8* Ciphertext, uint32 MsgLen)
{
    AESCrypt Enc;

    uint32 i, j, NumChunks;
    uint32 ChunkIndex = 0;
    uint8 ThisChunk[16];
    uint8 ThisEncChunk[16];

    /*setup the key*/
    AESCrypt_Init(&Enc);
    
    AESCrypt_key(&Enc, Key, AES_KEY_BITS);

    /* Determine number of 16 byte chunks in the message */
    NumChunks = MsgLen / (uint32)16;

    /*Encrypt data from Plaintext in 16 byte chunks*/
    for(i = 0; i < NumChunks; i++)
    { 
        for(j = 0; j < 16; j++)
            ThisChunk[j] = Plaintext[ChunkIndex + j];
        AESCrypt_encrypt(&Enc, ThisChunk, ThisEncChunk);
        for(j = 0; j < 16; j++)
            Ciphertext[ChunkIndex+j] = ThisEncChunk[j];
        ChunkIndex = ChunkIndex + 16;
    }

    AESCrypt_Destroy(&Enc);
}

void ChunkAESDecrypt(uint8* Key, const uint8* CipherText, uint8* PlainText, uint32 MsgLen)
{
    AESCrypt Dec;

    uint32 i, j, NumChunks;
    uint32 ChunkIndex = 0;
    uint8 ThisChunk[16];
    uint8 ThisDecChunk[16];

    AESCrypt_Init(&Dec);

    /*setup the key*/
    AESCrypt_key(&Dec, Key, AES_KEY_BITS);

    /* Determine number of 16 byte chunks in the message */
    NumChunks = MsgLen / (uint32)16;

    /*Decrypt data from msg in 16 byte chunks*/
    for(i = 0; i < NumChunks; i++)
    {
        for(j = 0; j < 16; j++)
            ThisChunk[j] = CipherText[ChunkIndex + j];
        AESCrypt_decrypt(&Dec, ThisChunk, ThisDecChunk);
        for(j = 0; j < 16; j++)
            PlainText[ChunkIndex+j] = ThisDecChunk[j];
        ChunkIndex = ChunkIndex + 16;
    }

    AESCrypt_Destroy(&Dec);
}

#endif /* #if defined(ENCRYPTION_ON) */


