/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CErrSvr.c      :                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CErrSvr.h"
#include "DtData.h"

void CErrSvr_Init(CErrSvr *pObject, uint8 Genus)
{
    /* Set the ClassID */
    pObject->ClassID = MakeClassID(FmTRAP, Genus, CES_SpNONE);

    /* Initialize the ErrorClassID */
    pObject->ErrorClassID = 0;

    /* Initialize the Data */
    pObject->Data = NULL;

    /* Initialize the Datasize */
    pObject->DataSize = 0;
}

/*********************************************************************************
Returns the total number of bytes required to serialize the data.
*********************************************************************************/
uint32 CErrSvr_SizeAll(const CErrSvr *pObject)
{
    uint32 size  = sizeof(uint16)         /*Size of the data array*/
                 + CES_STATIC_STRUCT_SIZE /* Size of static struct */
                 + sizeof(uint32)         /*Size of the data array*/
                 + pObject->DataSize      /*data array size*/
                 + sizeof(uint16);        /* CRC bytes */


    /* ClassID + Payload Length(uint16 non-aggregate) + Payload */
    return sizeof(uint32) + sizeof(uint16) + size;
}

/*********************************************************************************
Serializes the contents of the CErrSvr object into the buffer passed in.
*********************************************************************************/
boolean CErrSvr_Serialize(const CErrSvr *pObject, uint8 *pBuf)
{
    uint8   *pTemp = pBuf;
    boolean Result = TRUE;

    if (pTemp != NULL)
    {
        uint32 CRCSize;

        uint16 PayloadSize = (uint16)CErrSvr_SizeAll(pObject) - CES_HEADER_SIZE;

        /* Serialize the DtData members */
        WriteInt32(&pTemp, pObject->ClassID);

        /* Set the payload size */
        WriteInt16(&pTemp, (uint16)PayloadSize);
        
        /* Set the static struct size */
        WriteInt16(&pTemp, (uint16)CES_STATIC_STRUCT_SIZE);

        /* Set the Error ClassID*/
        WriteInt32(&pTemp, pObject->ErrorClassID);

        /* Set the Data size */
        WriteInt32(&pTemp, pObject->DataSize);
        
        if (pObject->DataSize > 0)
        {
            /* Set the Data */
            memcpy(pTemp, pObject->Data, pObject->DataSize);
            pTemp += pObject->DataSize;
        }

        /* Set the CRC */
        CRCSize = CErrSvr_SizeAll(pObject) - sizeof(uint16);
        WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pBuf, CRCSize));
    }
    else
        Result = FALSE;

    return Result; 
}
    
/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CErrSvr_Deserialize(CErrSvr *pObject, uint8 *pBuf)
{   
    uint8   *pTemp = pBuf;
    boolean Result = FALSE;

    if (pTemp != NULL)
    {
        uint32  Size; 
        uint16  PayloadSize;
        uint16  StoredCRC;
        uint16  CalcCRC;
        uint32  CRCSize;
        uint32  ClassID;

        /* Verify the Family and Genus match */
        ClassID = ReadInt32(&pTemp);
        if ((ClassID & 0xFFF0) == (pObject->ClassID & 0xFFF0))
        {
            /* Set the ClassID to update the species */
            pObject->ClassID = ClassID;

            /* Get the size of the payload */
            PayloadSize = ReadInt16(&pTemp);
       
            /* Get the static struct size */
            Size = ReadInt16(&pTemp);

            TOUCH(PayloadSize);
            TOUCH(Size);

            /* Get the Error ClassID */
            pObject->ErrorClassID = ReadInt32(&pTemp);

            /* Deserialize the CErrSvr dynamic members */
            /* Get the DataSize */
            pObject->DataSize = ReadInt32(&pTemp);
            
            if (pObject->DataSize > 0)
            {
                pObject->Data = (uint8*)SYS_REALLOC(pObject->Data, pObject->DataSize);
                if (!pObject->Data)
                    return FALSE;

                /* Copy the data into Data buffer */
                memcpy(pObject->Data, pTemp, pObject->DataSize);
                pTemp += pObject->DataSize;
            }
            else
            {
                SYS_FREE(pObject->Data);
                pObject->Data = NULL;
            }

            /* Calculate the CRC */
            CRCSize = CErrSvr_SizeAll(pObject) - sizeof(uint16);
            CalcCRC = CalculateCRC(CRC_SEED, pBuf, CRCSize);

            /* Get the stored CRC */
            StoredCRC = ReadInt16(&pTemp);

            /* Check the CRC */
            if (CalcCRC == StoredCRC) 
                Result = TRUE;
        }
    }

    return Result;
}


