/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CLinkTable.h   :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CLINKTABLE_H)
    #define  CLINKTABLE_H

#include "Target.h"
#include "StdVarTyp.h"
#include "Common.h"

typedef struct ConnArgs
{
    /* User configurable */
    boolean     bSvrFlag;               /* TRUE - Host/Server, FALSE - Client */
    boolean     bRNDZDisabled;          /* Turns off RNDZ authentication if set to TRUE */
    uint8       EncryptionType;         /* From Common.h of type WACP_ENCRYPTION_TYPES enuermation */
    int32       Priority;               /* 0 for normal, Positive for higher, Negative for lower */
    uint32      Timeout;                /* RNDZ authentication timeout in millseconds */
    STACK_TYPE  StackType;              /* From Common.h of type STACK_TYPE enumeration */
    uint16      RNDZPort;               /* RNDZ Port identifier */
    uint16      WACPPort;               /* RNDZ Port identifier */
    char        SourceID[ADDR_BUF_SIZE];/* Name of connection string identifier */
    void*       pConfig;               /* pointer to a stack configuration structure */

    /* Stack modified values */
    uint8       ValidationState;
    boolean     bRNDZStatus;
    boolean     bConnected;
    uint8*      pKey;
    void*       pCommSvr;
    void*       pConnObj;
    void*       pGUIDStd;
}ConnArgs;

typedef struct CLinkTable
{
    uint16    Count;
    ConnArgs* Connection[MAX_CONNECTIONS];
}CLinkTable;

boolean CLinkTable_Init(CLinkTable* pObject);
boolean CLinkTable_Destroy(CLinkTable* pObject);
boolean CLinkTable_AddEntry(CLinkTable* pObject, ConnArgs* pConnArgs);
boolean CLinkTable_RemoveEntry(CLinkTable* pObject, const ConnArgs* pConnArgs);
int32 CLinkTable_GetEntryIndex(const CLinkTable* pObject, const char* pSourceID);
ConnArgs* CLinkTable_GetEntry(const CLinkTable* pObject, const char* pSourceID);
void CLinkTable_GetSourceList(const CLinkTable* pObject, char** pSourceList);

#endif /* #if !defined(CLINKTABLE_H) */

/*---------------------------------------------------*/
/* End of File */
