/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// WACP.h                                                            //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#if !defined(WACP_H)
    #define  WACP_H

#include "Target.h"
#include "Interchange.h"

#include <stdio.h>
#include "stdvartyp.h"
#include "DtSupport.h"
#include "DtData.h"
#if defined(TRACE_SERVER_ON)
#include "CTraceSvr.h"
#endif
#include "CErrEStd.h"
#include "CRNDZApp.h"
#include "CMsgSvr.h"
#include "CErrSvr.h"
#include "CConObjSvr.h"
#include "CTrapObjSvr.h"
#include "CRNDZDStd.h"
#include "Common.h"
#include "CMOMPCallBack.h"

/* Global references */
#if defined(TRACE_SERVER_ON)
extern CTraceSvr TraceServer;
#endif

extern CLinkTable LinkTable;

#if defined(WACPDLL)
    #define APIEntry    __declspec( dllexport )
#else
    #define APIEntry
#endif

#if defined(WACPDLL)
#include "Packet.h"
#endif

/*================================================================================
WACP_SendMsg: Main interface method for sending a WACP message.
================================================================================*/
APIEntry boolean WACP_SendMsg (const char* pSourceID, uint16 Family, uint8 Genus,
                               uint8 Species, const uint8* Buffer, uint32 Size,
                               boolean bEncryptionOn);

/*================================================================================
WACP_RecvMsg: Main interface method for receiving a WACP message.
================================================================================*/
APIEntry boolean WACP_RecvMsg (char* pSourceID, uint16 Family, uint8 Genus,
                               uint8 Species, uint8* Buffer);

/*================================================================================
WACP_Init: Main interface starting the WACP stack.
================================================================================*/
APIEntry boolean WACP_Init(int32 ThreadPriority);

/*================================================================================
WACP_Destroy: Main interface stopping the WACP stack.
================================================================================*/
APIEntry void WACP_Destroy(void);

/*================================================================================
WACP_Open: Main interface opening all communications channels.
================================================================================*/
APIEntry boolean WACP_Open(ConnArgs **pConnArgs, uint8 NumConnections);

/*================================================================================
WACP_Connect: Main interface starting a connection.
================================================================================*/
APIEntry boolean WACP_Connect(const char* pSourceID);

/*================================================================================
WACP_Close: Main interface closing a connection.
================================================================================*/
APIEntry boolean WACP_Close(const char* pSourceID);

/*================================================================================
WACP_SetConnOptions: Interface for setting the Trap server options
================================================================================*/
APIEntry boolean WACP_SetConnOptions(const char* pSourceID, uint16 Option, time_t Interval,
                                     time_t TimeOut);

/*================================================================================
WACP_GetConnOptions: Interface for getting the Trap server options
================================================================================*/
APIEntry boolean WACP_GetConnOptions(const char* pSourceID, uint16 *pOption, time_t *pInterval, 
                                     time_t *pTimeOut);

/*================================================================================
WACP_SendTrap: Interface Sending an application ACK/NACK message
================================================================================*/
APIEntry boolean WACP_SendTrap(const char* pSourceID, uint32 MsgClassID, uint32 ErrClassID, 
                               uint8 ErrorSpecies, uint16 ErrorCode, const char* pErrMsg, 
                               uint16 ErrBufSize, const uint8 *pData, uint16 DataSize);
/*================================================================================
WACP_SendStackTrap: Interface Sending a message processing NACK message
================================================================================*/
APIEntry boolean WACP_SendMsgProcTrap(const char* pSourceID, uint32 ClassID, uint16 ErrorCode);

/*================================================================================
WACP_GetLastError: Interface for retreiving the last error message
================================================================================*/
APIEntry boolean WACP_GetLastError(const char* pSourceID);

/*================================================================================
WACP_GetGUIDObj: Interface for retreiving the GUID Object for a connection
================================================================================*/
APIEntry CRNDZDStd* WACP_GetGUIDObj(const char* pSourceID);

/*================================================================================
WACP_GetGUID: Interface for retreiving the GUID of the connection
================================================================================*/
APIEntry uint8* WACP_GetGUID(const char* pSourceID);

#define SEND_COMMAND(pSourceID, Family, CommandID, DataBuffer, DataSize, bEncryptionOn) WACP_SendMsg(pSourceID, Family, GnCOMMAND, CommandID, DataBuffer, DataSize, bEncryptionOn);
#define SEND_REQUEST(pSourceID, Family, RequestID, DataBuffer, DataSize, bEncryptionOn) WACP_SendMsg(pSourceID, Family, GnREQUEST, RequestID, DataBuffer, DataSize, bEncryptionOn);
#define SEND_RESPONSE(pSourceID, Family, ResponseID, DataBuffer, DataSize, bEncryptionOn) WACP_SendMsg(pSourceID, Family, GnRESPONSE, ResponseID, DataBuffer, DataSize, bEncryptionOn);
#define SEND_STATUS(pSourceID, Family, StatusID, DataBuffer, DataSize, bEncryptionOn) WACP_SendMsg(pSourceID, Family, GnSTATUS, StatusID, DataBuffer, DataSize, bEncryptionOn);
#define SEND_EVENT(pSourceID, Family, EventMsgID, DataBuffer, DataSize, bEncryptionOn) WACP_SendMsg(pSourceID, Family, GnEVENT, EventMsgID, DataBuffer, DataSize, bEncryptionOn);
#define SEND_STREAM(pSourceID, Family, StreamMsgID, DataBuffer, DataSize, bEncryptionOn) WACP_SendMsg(pSourceID, Family, GnSTREAM, StreamMsgID, DataBuffer, DataSize, bEncryptionOn);
#define SEND_DATAOBJ(pSourceID, Family, Species, DataBuffer, DataSize) WACP_SendMsg(pSourceID, Family, GnDATA, Species, DataBuffer, DataSize, FALSE);
#define SEND_TRAP(pSourceID, MsgClassID, ErrorClassID, ErrorType, ErrorCode, pErrMsg, ErrMsgSize, pData, DataSize) WACP_SendTrap(pSourceID, MsgClassID, ErrorClassID, ErrorType, ErrorCode, pErrMsg, ErrMsgSize, pData, DataSize)
#define SEND_MSGPROC_TRAP(pSourceID, ClassID, ErrorCode)  WACP_SendMsgProcTrap(pSourceID, ClassID, ErrorCode)

#endif /* #if !defined(WACP_H) */

