/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CInterchange.cpp:                                                 //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CInterchange.h"

DLLStruct* CInterchange::ExcgTable[MAX_DLL_COUNT];
uint16    CInterchange::DLLCount;

/* Faked Termiante message */
const uint8 WacpTerminate[] = 
{   
    0x00,0x1E,0x01,0x00,0x00,
    0x00,0x00,0x07,0x00,0x00,
    0x00,0x00,0x00,0x61,0xAD
};

/* Faked CRC Error message */
const uint8 CRCError[] = 
{   
    0x00, 0x1c, 0x03, 0x04, /* CErrSvr ClassID */
    0x00, 0x35,             /* CErrSvr Payload Size */
    0x00, 0x04,             /* CErrSvr Static Struct Size */
    0x00, 0x00, 0x00, 0x00, /* CErrSvr Error ClassID */ 
    0x00, 0x00, 0x00, 0x29, /* CErrSvr Data Size */
          
        0x00, 0x72, 0x03, 0x01, /* CErrEStd ClassID */
        0x00, 0x23,             /* CErrEStd Payload Size */
        0x00, 0x64,             /* CErrEStd Version */
        0xc0,                   /* CErrEStd DataByte */
        0x00, 0x1a,             /* CErrEStd Static Size */
        0x00, 0x00, 0x00, 0x00, /* CErrEStd Error ClassID */
        0x00, 0x05,             /* CErrEStd Errorcode */
        0x43, 0x52, 0x43, 0x20, 0x4d, 0x69, 0x73, 0x6d, 0x61, 0x74, 
        0x63, 0x68, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
        0x00, 0x00,             /* CErrEStd Error Description */
        0x2b, 0x06,             /* CErrEStd CRC*/

    0x46, 0x29              /* CErrSvr CRC*/
};


/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
CInterchange Start up: 
*********************************************************************************/

CInterchange::CInterchange()
{
}

CInterchange::~CInterchange()
{
    MESSAGE       MsgStruct;

    Close();

    /* Cleanup any remaining messages from the WACPQMgr */
    while (WACPQMgr->get_num_used() > 0)
    {
        /* Get the first packet Stream in the queue */
        WACPQMgr->dequeue_array((uint8*)&MsgStruct, sizeof(MESSAGE));

        /* Free the DataBuffer allocated by EvaluateRecv()*/
        SYS_FREE(MsgStruct.pDataBuf);

    } /* if (WACPQMgr->get_num_elements() > 0) */     

    /* Cleanup any remaining messages from the INQMgr */
    while (INQMgr->get_num_used() > 0)
    {
        /* Get the first packet Stream in the queue */
        INQMgr->dequeue_array((uint8*)&MsgStruct, sizeof(MESSAGE));

        /* Free the DataBuffer allocated by RecvMsg()*/
        SYS_FREE(MsgStruct.pDataBuf);

    } /* if (INQMgr->get_num_elements() > 0) */     

    /* Destroy the queue manager */
    delete(WACPQMgr);
    delete(INQMgr);

    if (pCommSvr != NULL)
    {
        delete(pCommSvr);
        pCommSvr = NULL;
    }

    /* Free the Queue buffers */
    SYS_FREE(WACPQueueBuf);
    
    /* Free the Queue buffers */
    SYS_FREE(INQueueBuf);

#if defined(DEBUG_PRINT)
    printf("Removing %p from ExcgTable DLLCount = %d\n", ExcgTable[DLLCount], DLLCount);
#endif

    /* Decrement the DLLCount to track the number of CInterchange class instances */
    DLLCount--;

#if defined(DEBUG_PRINT)
    for (uint16 i=0; i < DLLCount; i++)
    {
        printf("ExcgTable[%d]->pExchange = %p\n", i, ExcgTable[i]->pExchange);
        printf("ExcgTable[%d]->pAppName = %s\n", i, ExcgTable[i]->pAppName);
    }
#endif
}

void CInterchange::Init(uint16 RNDZPortNum, uint16 WACPPortNum, int32 Priority, void* pCallBackObj)
{   
    /* Set the Port values for RNDZ and WACP */
    RNDZPort = RNDZPortNum;
    WACPPort = WACPPortNum;

    /* Initialize the WACP and RNDZ Thread Counts */
    WACPCount = 0;

    /* Allocate space for the IN/OUT and WACP queues */
    WACPQueueBuf = (uint8*)SYS_MALLOC(RCVBUFSIZE);
    INQueueBuf = (uint8*)SYS_MALLOC(RCVBUFSIZE);

    /* Create the queue managers for the IN/OUT and WACP queues */
    WACPQMgr = new uint8QManager(WACPQueueBuf, RCVBUFSIZE);
    INQMgr   = new uint8QManager(INQueueBuf, RCVBUFSIZE);

    /* Create the semaphores for signaling data in the IN queue */
    pRecvSemaphore = new CWACPSemaphore(NULL, 0, MAX_RECV_QUEUE_ITEMS);

    /* Open the named handle and signal the Semaphore */
    memset(AppSemName, 0, sizeof(AppSemName));
    sprintf(AppSemName, WACP_SEM_NAME, SYS_GETPID(), this);
    pDataSemaphore = new CWACPSemaphore(AppSemName, 0, MAX_SEM_COUNT);

    /* Set this to TRUE before starting the Recv Threads */
    ThreadStatus = TRUE;

    /* Store stack settings */
    Args.ThreadPriority = Priority;
    Args.pCallBackObj = pCallBackObj;

    /* Start the Receiving Thread */
    RecvThread.StartThread(this->ReceivingThread, this, Args.ThreadPriority);

#if defined(TRACE_SERVER_ON) || defined(TRACE_SERVER_ON_STATUS)
    /* Open the Trace file here so we can start writing to it */
    TraceServer.OpenFile(TRACE_EXCG_LOG, "Exchange");
    TraceServer.GenerateXSLFile();
    TraceServer.EnableLogging(TRUE, TRUE);
    ErrCount = 0;
#endif

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    MsgInCount = 0;
    MsgOutCount = 0;
#endif

    /* Setup the ExcgStruct structure with the information on this CInterchange instance */
    ExcgStruct.pAppName = AppSemName;
    ExcgStruct.pExchange = this;

    /* Initialize this to NULL until StartThreads fills it in */
    ExcgStruct.pSourceID = NULL;

#if defined(DEBUG_PRINT)
    printf("Adding %p from ExcgTable DLLCount = %d\n", this, DLLCount);
#endif

    /* Add this instance to the ExcgTable to track each CInterchange class instance */
    ExcgTable[DLLCount] = &ExcgStruct;

    /* Increment the DLLCount to track the number of CInterchange class instances */
    DLLCount++;

#if defined(DEBUG_PRINT)
    for (uint16 i=0; i < DLLCount; i++)
    {
        printf("ExcgTable[%d]->pExchange = %p\n", i, ExcgTable[i]->pExchange);
        printf("ExcgTable[%d]->pAppName = %s\n", i, ExcgTable[i]->pAppName);
    }
#endif

    /* Initialize the CommSvr pointer */
    pCommSvr = NULL;
}

/*================================================================================
STATIC CInterchange Receive Thread : Processes messages in the IN queue.
================================================================================*/
uint32 WACPAPI CInterchange::ReceivingThread(void* pData)
{
    /* Recreate the CInterchange class from the pointer sent in pData. */
    CInterchange* pExchange = (CInterchange*)pData;

    /* Lock the Mutex until the thread is terminated */
    pExchange->RecvThreadMutex.Lock(INFINITE);

    /* Wait until we get a notification of data in the Recv queue */
    pExchange->pRecvSemaphore->Wait(INFINITE);

    while(pExchange->GetThreadStatus() == (uint16)TRUE)
    {
        /* Check for new messages */
        pExchange->EvaluateReceive();        

        /* Wait until we get a notification of data in the Recv queue */
        pExchange->pRecvSemaphore->Wait(INFINITE);
    }

    /* Release the mutex to allow CInterchange to terminate */
    pExchange->RecvThreadMutex.Release();
    
    return TRUE;
}

/*================================================================================
STATIC CInterchange CreateWACPLayer Creates the WACP Presentation Layer
================================================================================*/
uint32 WACPAPI CInterchange::CreateWACPLayer(void* pData)
{     
    ThreadArgs*   pThreadArgs;
    CInterchange* pExchange;
    CWACPEvent*   pExitEvent;
    LinkStruct    LnkStruct;
    char          Buffer[EVENT_BUF_SIZE];
    MESSAGE       MsgStruct;

    /* Recreate the ThreadArgs structure from the pointer sent in pData. */
    pThreadArgs = (ThreadArgs*)pData;

    /* Recreate the CInterchange class from the pointer sent in pThreadArgs. */
    pExchange = (CInterchange*)(pThreadArgs->pExchange);

    /* Set the semaphore name used to retrieve data */
    LnkStruct.pSemName = pExchange->AppSemName;

    /* Update the bServer flag  */
    LnkStruct.bServer = FALSE;
    
    /* Set the ThreadPriority */
    LnkStruct.ThreadPriority = pExchange->Args.ThreadPriority;

    /* Set the DLL reference */
    LnkStruct.pCallBackObj = pExchange->Args.pCallBackObj;

    /* Set the CLinkTable instance */
    LnkStruct.pCLinkTable = (void*)&(pExchange->LinkTable);

    /* Increment the WACPCont to track the number of connections */
    pExchange->WACPCount++;

    /* Cleanup any remaining messages from a previous session */
    while (pExchange->WACPQMgr->get_num_used() > 0)
    {
        /* Get the first packet Stream in the queue */
        pExchange->WACPQMgr->dequeue_array((uint8*)&MsgStruct, sizeof(MESSAGE));

        /* Free the DataBuffer allocated by EvaluateRecv()*/
        SYS_FREE(MsgStruct.pDataBuf);

    } /* if (WACPQMgr->get_num_elements() > 0) */

    /* Create a new CCommSvr instance. This is a wrapper object for accessing 
     * each stack specific CommSvr 
     */
    pExchange->pCommSvr = new CCommSvr(pExchange->RecvMsgWrapper, pExchange->TerminateClientWrapper, pExchange->AppSemName);
    
    /* Create the application layer */  
    CWACPApp* pApp = new CWACPApp(pExchange->SendPacketWrapper,
                                  pExchange->RecvWACPPacketWrapper,
                                  pExchange->MsgFuncWrapper,
                                  &(pExchange->LinkTable),
                                  &LnkStruct);
    /* Start up the App */
    pApp->AppStart();

    /* Delete the WACP Application Layer. This deletes 
     * the WACP Presentation Layer as well.
     */
    delete(pApp);

    /* Delete the CCommSvr if it still exists */
    if (pExchange->pCommSvr)
    {
        delete(pExchange->pCommSvr);
        pExchange->pCommSvr =  NULL;
    }
    
    /* Raise an event to signify the end of the client session */
    memset(Buffer, 0, EVENT_BUF_SIZE);
    sprintf(Buffer,EXIT_EXCHANGE,SYS_GETPID());
    pExitEvent = new CWACPEvent(Buffer, FALSE);
    pExitEvent->Raise();

    /* Clean up the Event object */
    delete(pExitEvent);

    /* Decrement the WACPCont to track the number of connections */
    pExchange->WACPCount--;

    return TRUE;
}

/*================================================================================
STATIC SendPacketWrapper: Wrapper function for SendPacket().
================================================================================*/
boolean CInterchange::SendPacketWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 Size)
{
    boolean RetVal = FALSE;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp((char*)pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)ExcgTable[i]->pExchange;
    }

    if (pThis != NULL)
    {
        RetVal = pThis->SendPacket(pSourceID, pBuf, Size);
    }

    return (RetVal);
}

/*================================================================================
STATIC RecvWACPPacket: Wrapper function for RecvWACPPacket().
================================================================================*/
uint32 CInterchange::RecvWACPPacketWrapper(char* pSemName, char** pSourceID, uint8** pBuf)
{
    uint32 RetVal = 0;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp(pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)ExcgTable[i]->pExchange;
    }

    if (pThis != NULL)
    {
        RetVal = pThis->RecvWACPPacket(pSourceID, pBuf);
    }

    return (RetVal);
}

/*================================================================================
STATIC MsgFuncWrapper: Wrapper function for MsgFunc().
================================================================================*/
boolean CInterchange::MsgFuncWrapper(char* pSemName, uint16 Option, void* pData, uint16 DataVal)
{
    boolean RetVal = FALSE;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp(pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)ExcgTable[i]->pExchange;
    }

    if (pThis != NULL)
    {
        RetVal = pThis->MsgFunc(Option, pData, DataVal);
    }

    return (RetVal);
}

/*================================================================================
STATIC RecvMsgWrapper: Wrapper function for RecvMsg().pCommSvr and pNewConnection 
       are only used in Server configurations
================================================================================*/
boolean CInterchange::RecvMsgWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 MsgSize,
                                 void* pNewConnection)
{
    boolean Result = FALSE;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp(pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)(ExcgTable[i]->pExchange);
    }

    TOUCH(pNewConnection);

    if ((pThis != NULL) && (pBuf != NULL))
    {
        Result = pThis->RecvMsg(pSourceID, pBuf, MsgSize);
    }
    
    return (Result);
}

/*================================================================================
STATIC TerminateClientWrapper: Wrapper function for TerminateClient().
================================================================================*/
boolean CInterchange::TerminateClientWrapper(char* pSemName, char* pSourceID, uint16 Port)
{
    boolean Result = FALSE;
    CInterchange* pThis = NULL;
    uint16 i;

    for (i=0; i < DLLCount; i++)
    {
        if (strcmp((char*)pSemName, ExcgTable[i]->pAppName) == 0)
            pThis = (CInterchange*)(ExcgTable[i]->pExchange);
    }

    if (pThis != NULL) 
    {
        if (Port == pThis->WACPPort)
        {
            Result = pThis->TerminateClient(pSourceID);
        }
    }
    
    return (Result);
}

/*================================================================================
CInterchange Send Packet: Wraps the passed in Buffer into a PACKET structure and
    sends the message via the CommSvr.
================================================================================*/
boolean CInterchange::SendPacket(char* pSourceID, uint8* pBuf, uint32 Size)
{
    boolean RetVal = FALSE;
    uint8* OutBuffer;
    uint32 PacketLen;
    ConnArgs* pConnArgs = LinkTable.GetEntry(pSourceID);

    /* Set the Size of the entire packet */
    PacketLen = PACKET_HEADER_SIZE + Size + CRCSIZE;

    if ((pCommSvr != NULL) && (pConnArgs != NULL) && (pConnArgs->bConnected != FALSE))
    {
        /* Allocate room for the serialize packet Stream freed below */
        OutBuffer = GeneratePacket(&PacketLen, pCommSvr->GetPortNum(pConnArgs),0, 0, pBuf, Size);
        if (OutBuffer)
        {
            /* Send the message */
            RetVal = pCommSvr->SendMsg(pConnArgs, (uint8*)OutBuffer, PacketLen);

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
            {
                char tempString[] = {"SourceID %s\nMsgOutCount %d\nMessage Data:\n%A"};
                TraceServer.tprintf(TRACE_Type_SEND,
                    "CInterchange", "SendPacket", 0, 0, 0, 
                    tempString, pSourceID, ++MsgOutCount, OutBuffer, PacketLen);
            }
#endif
            /* Free the buffer */
            SYS_FREE(OutBuffer);

        }/* if (OutBuffer) */
#if defined(TRACE_SERVER_ON)
        else
        {
            TraceServer.tprintf(TRACE_Type_ERROR,
                "CInterchange", "SendPacket", 0, 0, 0, 
                "MEMORY ERROR Count %d!!! SendPacket: Unable to allocate memory\n",
                ++ErrCount);
        }
#endif

    }/* if (pCommSvr != NULL) */

    return(RetVal );
}

/*================================================================================
CInterchange RecvWACPPacket: Searches the messages in the STL list for the first
    message with the appropriate SourceID. The SourceID is found in the link 
    table via the passed in pSourceID.
================================================================================*/
uint32 CInterchange::RecvWACPPacket(char** pSourceID, uint8** pBuf)
{
    uint32 RetVal = 0;

    /* TODO: Change INFINITE to MAX_TIMEOUT */
    WACPRecvMutex.Lock(INFINITE);

    /* Check for valid pointers before continuing */
    if ((pSourceID != NULL) && (pBuf != NULL))
    {
        /* Make sure the list is not empty */
        if (WACPQMgr->get_num_used() > 0)
        {
            MESSAGE MsgStruct;

            /* Get the first packet Stream in the queue */
            WACPQMgr->dequeue_array((uint8*)&MsgStruct, sizeof(MESSAGE));

            /* Update the SourceID of the message */
            *pSourceID = MsgStruct.pSourceID;

            /* Update the data pointer */
            *pBuf = MsgStruct.pDataBuf;

            /* Set RetVal to return the number of Bytes received */
            RetVal = MsgStruct.DataSize;

        } /* if (WACPQMgr->get_num_elements() > 0) */

    }/* if ((pSourceID != NULL) && (pBuf != NULL)) */
#if defined(TRACE_SERVER_ON)
    else
    {
        TraceServer.tprintf(TRACE_Type_ERROR,
                "CInterchange", "RecvWACPPacket", 0, 0, 0, 
                "ERROR!!!RecvWACPPacket: Invalid pSourceID and/or Buffer passed in\n");
    }
#endif

    /* Release the mutex */
    WACPRecvMutex.Release();

    return(RetVal );
}

/*================================================================================
CInterchange MsgFunc: 
    WACP_CONNECT: Opens a set of connections based on the parameters passed in.
    WACP_STATUS: Updated the RNDZ status of the currrent connection.
    WACP_DISCONNECT: Closes a single connection based on the SourceID in pData
    WACP_CLOSE: Closes all connections in the LinkTable
================================================================================*/
boolean CInterchange::MsgFunc(uint16 Option, void* pData, uint16 DataVal)
{
    boolean RetVal = FALSE;

    if (ThreadStatus)
    {
        switch (Option)
        {
            case WACP_CONNECT:
                RetVal = Connect((ConnArgs**)pData, (uint8)DataVal);
                break;
            case WACP_STATUS:
                if (pCommSvr)
                {
                    ConnArgs* pConnArgs = LinkTable.GetEntry((char*)pData);
                    if (pConnArgs)
                    {
                        RetVal = pCommSvr->SetPortNum(pConnArgs);
                    }
                }
                break;
            case WACP_DISCONNECT:
                RetVal = Disconnect((char*)pData);
                break;
#if defined(TRACE_SERVER_ON)
            default:
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "SendPacket", 0, 0, 0, 
                    "ERROR Count %d!!! CInterchange MsgFunc: Error Invalid Option\n", ++ErrCount);
                break;
#endif
        }
    }

    return (RetVal);
}

/*================================================================================
CInterchange Evaluate Receive Thread 
================================================================================*/
void CInterchange::EvaluateReceive()
{
    PACKET      Pack;
    MESSAGE     Msg;
    ConnArgs*   pConnArgs = NULL;

    /* Process the message if the queue is not empty */
    if (INQMgr->get_num_used() > 0)
    {
        uint8*  pTemp;
        uint8*  pBuf = NULL;
        int     Port;
        uint32  DataLength;

        /* Wait for the INQMgr mutex */
        RecvMutex.Lock(INFINITE);

        /* Get the first packet Stream in the queue */
        INQMgr->dequeue_array((uint8*)&Msg, sizeof(MESSAGE));

        /* Release the RecvBuf Mutex */
        RecvMutex.Release();

        /* Set pBuf to the actual data buffer */
        pBuf = Msg.pDataBuf;
        
        /* Set the Temp Pointer beyond the PREAMBLE of the byte stream */
        pTemp = (uint8*)pBuf + PREAMBLE_SIZE;

        /* Read the Packet Length */
        Pack.Header.PacketLen = ReadInt32(pTemp);
        pTemp += sizeof(int32);

        /* Read the Port */
        Pack.Header.Port = ReadInt16(pTemp);
        pTemp += sizeof(int16);

        /* Check the last Preamble Character...
            * CTL_C is the full packetization support mode.
            * CTL_L is the light mode that does not contain the SequenceNum and UUID
            */
        if (pBuf[2] == CTL_C)
        {
            /* Read the SequenceNum */
            Pack.Header.SequenceNum = ReadInt16(pTemp);
            pTemp += sizeof(int16);

            /* Read the Packet_UUDID */
            Pack.Header.Packet_UUID = ReadInt32(pTemp);
            pTemp += sizeof(int32);
            
            DataLength = Pack.Header.PacketLen - PACKET_HEADER_SIZE  - 2;
        }
        else
        {
            DataLength = Pack.Header.PacketLen - PACKET_LITE_HEADER_SIZE  - 2;
        }

        /* Allocate room for the databuffer. Released in RecvWACPPacket() or this 
        * function if the CRCs do not match
        */
        Pack.pDataBuffer = (uint8*)SYS_MALLOC(DataLength);

        if (Pack.pDataBuffer != NULL)
        {
            /* Get the databuffer */
            memcpy(Pack.pDataBuffer, pTemp, DataLength);
            pTemp += DataLength;
        }

        /* Get the CRC */
        Pack.CRC = ReadInt16(pTemp);

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
        {
            char tempString[] = {"SourceID %s\nMsgInCount %d\nMessage Data:\n%A"};
            TraceServer.tprintf(TRACE_Type_RECEIVE,
                "CInterchange", "EvaluateReceive", 0, 0, 0, 
                tempString, Msg.pSourceID, ++MsgInCount, pBuf, Pack.Header.PacketLen);
        }
#endif

        /* Get the Port */
        Port = Pack.Header.Port;

        /* Check the SourceID in the LinkTable */
        pConnArgs = LinkTable.GetEntry(Msg.pSourceID);

        /* Exit the function and drop the message if the entry is not found in the LinkTable */
        if (!pConnArgs)
        {
#if defined(TRACE_SERVER_ON)
            char tempString[] = {"Unable to find LinkTable Entry. SourceID %s\nMessage Data:\n%A"};
            TraceServer.tprintf(TRACE_Type_ERROR,
                "CInterchange", "EvaluateReceive", 0, 0, 0, 
                tempString, Msg.pSourceID, pBuf, Pack.Header.PacketLen);
#endif
            SYS_FREE(pBuf);
            SYS_FREE(Pack.pDataBuffer);
            return;
        }

        /* Verify the CRC */
        if (Pack.CRC == CalculateCRC((uint8*)pBuf, Pack.Header.PacketLen - CRCSIZE))
        {
            boolean bRejectFlag = FALSE;

            /* Test that we have a valid Port number and, if RNDZ is enabled,
            * that RNDZ is complete before allowing any WACP messages.
            */
            if (pConnArgs->bRNDZDisabled == FALSE)
            {
                bRejectFlag = (!(Port == RNDZPort || Port == WACPPort) ||
                            (((Port == WACPPort) && (pConnArgs->bRNDZStatus == FALSE))));
            }

            if (bRejectFlag)
            {
#if defined(TRACE_SERVER_ON)
                char tempString[] = {"Unauthenticated Message. SourceID %s\nMessage Data:\n%A"};
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "EvaluateReceive", 0, 0, 0, 
                    tempString, Msg.pSourceID, pBuf, Pack.Header.PacketLen);
#endif
                SYS_FREE(pBuf);
                SYS_FREE(Pack.pDataBuffer);
                return;
            }

            /* Set the data length */
            Msg.DataSize = DataLength;

            /* Set the pDataBuf to the Pack DataBuffer */
            Msg.pDataBuf = Pack.pDataBuffer;

        }/* if (Pack.CRC == CalculateCRC(Pack)) */
        else
        {
            /* CRC mismatch! Pass message up to presentation layer for 
             * notification to the application
             */

            /* Set the data length */
            Msg.DataSize = sizeof(CRCError);

            Msg.pDataBuf = (uint8*)SYS_MALLOC(Msg.DataSize);
            if (Msg.pDataBuf != NULL)
            {
                /* Set the CRCError in the message data buffer*/
                memcpy(Msg.pDataBuf, CRCError, Msg.DataSize);
            } 

#if defined(TRACE_SERVER_ON)
            {
                char tempString[] = {"CRC failure Stored CRC %d Calculated CRC %d\nErrCount %d\nMessage Data:\n%A"};
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "EvaluateReceive", 0, 0, 0, 
                    tempString, Pack.CRC, 
                    CalculateCRC((uint8*)pBuf, Pack.Header.PacketLen - CRCSIZE),
                    ++ErrCount, pBuf, Pack.Header.PacketLen);
            }
#endif
            /* Free the memory created above since this is a corrupted message */
            SYS_FREE(Pack.pDataBuffer);
        }

        /* Wait for the WACPQMgr mutex */
        WACPRecvMutex.Lock(INFINITE);
        
        /* Add the message to the WACPQMgr*/
        WACPQMgr->enqueue_array((uint8*)&Msg, sizeof(MESSAGE));

        /* Release the WACPQMgr Mutex */
        WACPRecvMutex.Release();

        pDataSemaphore->Raise();     

        /* Free the message Buffer allocated in RecvMsg() */
        SYS_FREE(pBuf);

    }/*if (IN_queue.Size() > 0)*/
}

/*================================================================================
CInterchange StartThreads - unused for Client baselines.yy
================================================================================*/
boolean CInterchange::StartServer(ConnArgs** pConnList, uint8 NumConnections)
{       
    boolean Result = FALSE;

    TOUCH(pConnList);
    TOUCH(NumConnections);

    return (Result);
}
/*================================================================================
CInterchange StartThreads
================================================================================*/
boolean CInterchange::StartClient(void)
{       
    boolean Result = FALSE;

    /* Set the thread arguements */
    Args.pExchange = this;

    /* Create the WACP thread */
    WACPThread.StartThread(this->CreateWACPLayer, (void*)&Args, Args.ThreadPriority);

    /* Sleep to allow the WACPThread to start */
    SYS_SLEEP(WAIT_COUNT/4);

    return (Result);
}

/*================================================================================
CInterchange Thread Status
================================================================================*/
boolean CInterchange::GetThreadStatus()
{   
    return ThreadStatus;
}

/*================================================================================
CInterchange Close: Closes all Mutex handles and sets the ThreadStatus to FALSE
================================================================================*/
uint16 CInterchange::Close()
{
    ThreadStatus = FALSE;
    uint16 i;

    if (WACPCount)
    {
        char* pSourceList[MAX_CONNECTIONS];
        
        /* Get a list of all entries in Linktable and shut down all of them individually */
        LinkTable.GetSourceList(pSourceList);
        for (i=0; pSourceList[i] != NULL; i++)
        {
            TerminateClient(pSourceList[i]);
        }
    }

    /* Wait until all connections have terminated */
    while(WACPCount > 0)
        SYS_SLEEP(WAIT_COUNT);

    /* Raise the Semaphore to allow the recv thread to terminate */
    pRecvSemaphore->Raise();

    /* Wait for the Recv Thread to terminate */
    RecvThreadMutex.Lock(INFINITE);
    RecvThreadMutex.Release();

    if(pRecvSemaphore != NULL)
        delete(pRecvSemaphore);

    if(pDataSemaphore != NULL)
        delete(pDataSemaphore);

    return TRUE;
}

/*================================================================================
CInterchange RecvMsg: Adds a message to the INQMgr
================================================================================*/
boolean CInterchange::RecvMsg(char* pSourceID, uint8* pBuf, uint32 MsgSize)
{
    MESSAGE Msg;
    boolean Result = FALSE;

    /* Check for a valid Buffer and Size */
    if (pBuf != NULL)
    {
        /* Allocate room for the new message, this is released in EvaluateReceive() */
        Msg.pDataBuf = (uint8*)SYS_MALLOC(MsgSize);
        if (Msg.pDataBuf != NULL)
        {
            /* Create a copy of the message */
            memcpy(Msg.pDataBuf, pBuf, MsgSize);

            /* Set the Data size */
            Msg.DataSize = MsgSize;

            /* Set the SourceID */
            Msg.pSourceID = pSourceID;

            /* Wait for the Recv Mutex */
            RecvMutex.Lock(INFINITE);
    
            /* Queue up the MESSAGE */
            INQMgr->enqueue_array((uint8*)&Msg, sizeof(MESSAGE));
      
            /* Release the Mutex */
            RecvMutex.Release();

            /* Notify the Receiving thread to process the message */
            pRecvSemaphore->Raise();
        }
    }

    return (Result);
}

/*================================================================================
TerminateAllClients: Terminates all client connections
================================================================================*/
void CInterchange::TerminateAllClients(void)
{
    /* Empty method */
}
/*================================================================================
TerminateClient: Terminates the client connection
================================================================================*/
boolean CInterchange::TerminateClient(char* pSourceID)
{
    boolean Result = FALSE;
    MESSAGE Msg;
    CWACPSemaphore* pDataSemaphore;
  
    /* Open the named handle and signal the Semaphore */
    if (AppSemName != NULL)
    {
        /* Wait for the WACP_IN_list mutex */
        WACPRecvMutex.Lock(INFINITE);
        
        /* Cleanup any remaining messages from the WACPQMgr to process the 
            * termination message
            */
        while (WACPQMgr->get_num_used() > 0)
        {
            /* Get the first packet Stream in the queue */
            WACPQMgr->dequeue_array((uint8*)&Msg, sizeof(MESSAGE));

            /* Free the DataBuffer allocated by EvaluateRecv()*/
            SYS_FREE(Msg.pDataBuf);

        } /* if (WACPQMgr->get_num_elements() > 0) */     

        /* Set the data length */
        Msg.DataSize = sizeof(WacpTerminate);

        /* Set the SourceID */
        Msg.pSourceID = (char*)pSourceID;

        Msg.pDataBuf = (uint8*)SYS_MALLOC(Msg.DataSize);
        if (Msg.pDataBuf != NULL)
        {
            /* Set the pDataBuf to the Pack pDataBuf */
            memcpy(Msg.pDataBuf, WacpTerminate, Msg.DataSize);

            /* Add the message to the WACPQMgr*/
            WACPQMgr->enqueue_array((uint8*)&Msg, sizeof(MESSAGE));

            /* Release the WACP_IN_list Mutex */
            WACPRecvMutex.Release();

            /* Open a handle to the named Semaphore */
            pDataSemaphore = new CWACPSemaphore(AppSemName, 0, MAX_SEM_COUNT);
            
            /* Signal the Semaphore to notify the Presentation layer there is data available */
            pDataSemaphore->Raise();     
        
            /* Clean up the DataSemaphore Object */
            delete(pDataSemaphore);

            Result = TRUE;
        }/* if (Msg.pDataBuf != NULL) */
    }/* if (AppSemName != NULL) */

    TOUCH(pSourceID);

    return Result;
}

/* End of File */


/*================================================================================
GeneratePacket: Wraps the passed in Buffer into a PACKET structure and
    adds header information based on parameters. Returns a newly malloced 
    uint8 buffer filled with the new packet.
================================================================================*/
uint8* GeneratePacket(uint32* OutSize, int16 Port, int16 SequenceNum, int32 UUID, uint8* pBuf, uint32 Size)
{
    uint8* OutBuffer = NULL;
    boolean Packetized = FALSE;

    /* Set the Size of the entire packet */
    if (Packetized)
    {
        *OutSize = PACKET_HEADER_SIZE + Size + CRCSIZE;
    }
    else
    {
        *OutSize = PACKET_LITE_HEADER_SIZE + Size + CRCSIZE;
    }

    /* Allocate room for the serialize packet Stream freed in EvaluateSend*/
    OutBuffer = (uint8*)SYS_MALLOC(*OutSize);
    
    if (OutBuffer)
    {
        uint8* pTemp = OutBuffer; 

        /* Serialize the packet header struct */
        /* Set the PREAMBLE */    
        *pTemp++ = CTL_W;
        *pTemp++ = CTL_A;

        /* TODO: Check to see if this is a piece of a packetized message */
        if (Packetized)
            *pTemp++ = CTL_C;
        else
            *pTemp++ = CTL_L;

        /* Write the Packet Length */
        WriteInt32(pTemp, *OutSize);
        pTemp += sizeof(int32);

        /* Write the Port */
        WriteInt16(pTemp, Port);
        pTemp += sizeof(int16);

        /* TODO: Check to see if this is a piece of a packetized message */
        if (Packetized)
        {
            /* Write the SequenceNum */
            WriteInt16(pTemp, SequenceNum);
            pTemp += sizeof(int16);

            /* Write the Packet_UUDID */
            WriteInt32(pTemp, UUID);
            pTemp += sizeof(int32);
        }

        /* Copy the data Stream */
        memcpy((char*)pTemp, pBuf, Size);
        pTemp += Size;
        
        /* Write the CRC */
        WriteInt16(pTemp, CalculateCRC(OutBuffer, *OutSize - CRCSIZE));

    }/* if (OutBuffer) */

    return(OutBuffer);
}

/*================================================================================
Connect: Creates the connections specified in the ConnArgs array passed in. The
    status boolean bConnected is updated with each individual connection status.
    
    Returns TRUE if all connections are opened successfully
    Returns FALSE if any one connection fails to open.
================================================================================*/
boolean CInterchange::Connect(ConnArgs** pConfig, uint8 NumConnections)
{
    boolean Result = FALSE;
    uint16 i = 0;

    if (pCommSvr != NULL)
    {
        for (i=0; i < NumConnections; i++)
        {
            if (pConfig[i]->bConnected == FALSE)
            {
                /* Start the connection */
                Result = pCommSvr->Connect(pConfig[i]);
                if (Result)
                {
                    /* Update the connection status */
                    pConfig[i]->bConnected = TRUE;
                    SYS_SLEEP(WAIT_COUNT);
                }
#if defined(TRACE_SERVER_ON)
                else
                {
                    TraceServer.tprintf(TRACE_Type_ERROR,
                        "CInterchange", "Connect", 0, 0, 0,
                        "Failed to connect to %s\n", pConfig[i]->SourceID);
                }
#endif
            }
#if defined(TRACE_SERVER_ON)
            else
            {
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "Connect", 0, 0, 0,
                    "%s is already connected\n", pConfig[i]->SourceID);
            }
#endif
        }
    }

    return (Result);
}

/*================================================================================
Disconnect: Closes down a single connection based on the SourceID.

    Returns TRUE if the connection is successfully closed.
    Returns FALSE if the SourceID is not found or if the connection was not closed.
================================================================================*/
boolean CInterchange::Disconnect(char* pSourceID)
{
    boolean Result = FALSE;
    ConnArgs* pConnArgs = NULL;

    /* Find the entry in the linktable */
    pConnArgs = LinkTable.GetEntry(pSourceID);
    if ((pConnArgs) && (pConnArgs->bConnected == TRUE))
    {
        Result = pCommSvr->Disconnect(pConnArgs);
        if (Result == TRUE)
        {
            /* Reset the RNDZ authentication */
            pConnArgs->bRNDZStatus = FALSE;

            /* Reset the connection status */
            pConnArgs->bConnected = FALSE;

            /* Free up the memory allocated in connection sequence */
            SYS_FREE(pConnArgs->pKey);
            pConnArgs->pKey = NULL;

            if (pConnArgs->pGUIDStd)
            {
                delete(pConnArgs->pGUIDStd);
                pConnArgs->pGUIDStd = NULL;
            }
        }
#if defined(TRACE_SERVER_ON)
        else
        {
            TraceServer.tprintf(TRACE_Type_ERROR,
                "CInterchange", "Disconnect", 0, 0, 0,
                "Failed to disconnect %s\n", pSourceID);
        }
#endif
    }
    else
    {
        Result = FALSE;
#if defined(TRACE_SERVER_ON)
        TraceServer.tprintf(TRACE_Type_ERROR,
            "CInterchange", "Disconnect", 0, 0, 0,
            "Unable to find %s\n", pSourceID);
#endif
    }

    return (Result);
}

