/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CInterchang.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CINTERCHANGE_H)
    #define  CINTERCHANGE_H

#include "StdVarTyp.h"
#include "Common.h"
#include "CCommSvr.h"
#include "CWACPApp.h"
#include "CWACPMutex.h"
#include "CWACPEvent.h"
#include "CTrapObjSvr.h"
#include "CWACPThread.h"
#include "SysUtil.h"
#include "CWACPSemaphore.h"
#include "CTraceSvr.h"
#include "CLinkTable.h"


#define WACP_SEM_NAME "WACP_%d_%p"
#define EXIT_EXCHANGE "EXIT_EXCHANGE_%d"
#define CRCSIZE (2)

struct PACKET_HDR
{
    uint8   Preamble[ PREAMBLE_SIZE ];
    uint32  PacketLen;
    uint16  Port;
    uint16  SequenceNum;
    uint32  Packet_UUID;
};

/* PACKET_LITE_HEADER_SIZE = PREAMBLE + PacketLen + Port
 *      PREAMBLE  = sizeof(uint8) * PREAMBLE_SIZE
 *      PacketLen = sizeof(uint32)
 *      Port      = sizeof(uint16) 
 */
#define PACKET_LITE_HEADER_SIZE 9

/* PACKET_HEADER_SIZE = PREAMBLE + PacketLen + Port + SequenceNum + Packet_UUID
 *      PREAMBLE    = sizeof(uint8) * PREAMBLE_SIZE
 *      PacketLen   = sizeof(uint32)
 *      Port        = sizeof(uint16) 
 *      SequenceNum = sizeof(uint16) 
 *      Packet_UUID = sizeof(uint32)
 */
#define PACKET_HEADER_SIZE 15

struct PACKET
{
    PACKET_HDR  Header;
    uint8*      pDataBuffer;
    uint16      CRC;
};

struct MESSAGE
{
    char*  pSourceID;
    uint8* pDataBuf;
    uint32 DataSize;
};

struct ThreadArgs
{
    void* pExchange;
    int32 ThreadPriority;
    void* pCallBackObj;
};

struct DLLStruct
{
    void* pExchange;
    char* pSourceID;
    char* pAppName;
};

class CInterchange
{
public:


    CInterchange();
    ~CInterchange();

    void Init(uint16 RNDZPortNum, uint16 WACPPortNum, int32 Priority, void* pCallBackObj);

    /* Thread Functions */
    boolean StartServer(ConnArgs** pConnList, uint8 NumConnections);
    boolean StartClient(void);
    boolean CreateWACPThread(ThreadArgs* pThreadArgs);

    uint16 Close();

    boolean GetThreadStatus();

    boolean TerminateClient(char* pSourceID);
    void TerminateAllClients(void);

private:
    static DLLStruct*   ExcgTable[MAX_DLL_COUNT];
    static uint16       DLLCount;
    DLLStruct           ExcgStruct;

    /* Static Callback Function Wrappers */
    static boolean SendPacketWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 Size);
    static uint32 RecvWACPPacketWrapper(char* pSemName, char** pSourceID, uint8** pBuf);
    static boolean MsgFuncWrapper(char* pSemName, uint16 Option, void* pData, uint16 DataVal);
    static boolean RecvMsgWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 MsgSize, void* pNewConnection);
    static boolean TerminateClientWrapper(char* pSemName, char* pSourceID, uint16 Port);

    /* Callback Functions */
    boolean SendPacket(char* pSourceID, uint8* pBuf, uint32 Size);
    uint32 RecvWACPPacket(char** pSourceID, uint8** pBuf);
    boolean MsgFunc(uint16 Option, void* pData, uint16 DataVal);
    boolean RecvMsg(char* pSourceID, uint8* pBuf, uint32 MsgSize);
    boolean Connect(ConnArgs** pConnArgs, uint8 NumConnections);
    boolean Disconnect(char* pSourceID);

    /* Thread Functions */
    static uint32 WACPAPI ReceivingThread(void* pData);
    static uint32 WACPAPI CreateWACPLayer(void* pData);

    /* Receive Thread function */
    void EvaluateReceive();

    /* Mutex Pointers */
    CWACPMutex  WACPRecvMutex;             /* WACP Recv Queue mutex */
    CWACPMutex  RecvMutex;                 /* IN_queue mutex */
    CWACPMutex  RecvThreadMutex;           /* Exit RecvThread sync mutex */
    
    /* Thread Status var */
    boolean ThreadStatus;
    boolean bRNDZAuthentication;

    /* Threads */
    CWACPThread RecvThread;
    CWACPThread WACPThread;

    /* Queue Buffer */
    uint8* WACPQueueBuf;
    uint8* INQueueBuf;
    
    uint8QManager* WACPQMgr;
    uint8QManager* INQMgr;

    /* Thread arguements*/
    ThreadArgs Args;
    CLinkTable LinkTable;

    /* WACP and RNDZ Ports */
    uint16 RNDZPort;
    uint16 WACPPort;

    /* Pointer to the CCommSvr */
    CCommSvr* pCommSvr;

    /* Semaphores used for message notification */
    CWACPSemaphore* pRecvSemaphore;
    CWACPSemaphore* pDataSemaphore;
    char            AppSemName[EVENT_BUF_SIZE];
    
    /* WACP Thread Count */
    uint16 WACPCount;

#if defined(TRACE_SERVER_ON) || (TRACE_SERVER_ON_STATUS)
    /* TraceServer reference */
    CTraceSvr   TraceServer;
    uint32 ErrCount;
#endif

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    uint32 MsgInCount;
    uint32 MsgOutCount;
#endif

};

#endif /* #if !defined(CINTERCHANGE_H) */

/*---------------------------------------------------*/
/* End of File */





