/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Common .h      :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(COMMON_H)
    #define  COMMON_H

#include "StdVarTyp.h"

#if !defined (WIN32)  && !defined(_WIN32_WCE)
    #if !defined(INFINITE)
        #define INFINITE 0xFFFFFFFF
    #endif

    #if !defined(INVALID_HANDLE_VALUE)
        #define INVALID_HANDLE_VALUE       ( 0 )
    #endif

    typedef void* HANDLE;

#endif

#define WAIT_SUCCESS       ( 0 )
#define WAIT_FAILURE       ( 1 )

#define ADDR_BUF_SIZE   (32)
#define EVENT_BUF_SIZE  (32)

#define RNDZ_PORT_DEFAULT ( 281 )
#define WACP_PORT_DEFAULT ( 282 )

#define TRACE_WACP_LOG      "TraceWACPLog.xml"
#define TRACE_RNDZ_LOG      "TraceRNDZLog.xml"
#define TRACE_EXCG_LOG      "TraceEXCGLog.xml"

enum MSG_FUNCS{
    WACP_CONNECT = 0,
    WACP_STATUS,
    WACP_DISCONNECT,
    WACP_CLOSE
};

typedef enum STACK_TYPE{
    STACK_UNDEFINED = 0,
    SERIAL,
    ETHERNET,
    USB,
    BLUETOOTH,
    ZIGBEE
} STACK_TYPE;

#if !defined __CTSTYPOPTION
    #define  __CTSTYPOPTION
    typedef  enum CTStypOPTION{
        CTS_SpNONE = 0,
        CTS_SpACK,
        CTS_SpDESC,
        CTS_SpMAX
    } CTS_OPTION;
#endif

typedef  enum WACP_ENCRYPTION_TYPES {
    WACP_ENCRYPTION_NONE = 0,
    WACP_ENCRYPTION_AES,
    WACP_ENCRYPTION_MAX
}WACP_ENCRYPTION_TYPES;


typedef struct ConnArgs
{
    /* User configurable */
    boolean     bSvrFlag;                       /* TRUE - Host/Server, FALSE - Client */
    boolean     bByPass;                        /* TRUE - Serial Server bypass */
    boolean     bRNDZDisabled;                  /* Turns off RNDZ authentication if set to TRUE */
    uint8       EncryptionType;                 /* From Common.h of type WACP_ENCRYPTION_TYPES enuermation */
    int32       ThreadPriority;                 /* 0 for normal, Positive for higher, Negative for lower */
    uint32      Timeout;                        /* RNDZ authentication timeout in millseconds */
    STACK_TYPE  StackType;                      /* From Common.h of type STACK_TYPE enumeration */
    uint16      RNDZPort;                       /* RNDZ Port identifier */
    uint16      WACPPort;                       /* WACP Port identifier */
    void*       pConfig;                        /* pointer to a stack configuration structure */
    char        SourceID[ADDR_BUF_SIZE];        /* Name of connection string identifier */

    /* Stack modified values */
    uint8       ValidationState;
    boolean     bRNDZStatus;
    boolean     bConnected;
    uint8*      pKey;
    void*       pCommSvr;
    void*       pConnObj;
    void*       pGUIDStd;
    char*       pByPassSrc;
}ConnArgs;

typedef struct LinkStruct
{
    char*   pSemName;
    int32   ThreadPriority;
    void*   pCallBackObj;
    void*   pCLinkTable;
    boolean bServer;
} LinkStruct;

#ifndef MOMP_DLLAPI
    #if defined(_MOMP_EXPORT_DLLAPI_)
        #define MOMP_DLLAPI  __declspec(dllexport)
    #elif defined(_MOMP_IMPORT_DLLAPI_)
        #define MOMP_DLLAPI  __declspec(dllimport)
    #else
        #define MOMP_DLLAPI
    #endif
#endif

#endif /* #if !defined(COMMON_H) */

/*---------------------------------------------------*/
/* End of File */
