/*
NOTE:   This code is used directly to perform a DTIS protocol test. If any of 
        this code is modified, it will directly affect that protocol.
*/

#include "Middleware.h"

int main(void)
{
    CDMOIBDictionary        Dictionary(<DDS_NAME>::Dictionary());
    CDMOIBStringTable       Table(<DDS_NAME>::StringTable(), 
                                 <DDS_NAME>::LinkTable(),
								 &Dictionary);
    void*                   pLanguageTable;
    uint32                  TableSize = 0;

    GetDef(&Dictionary, &Table);

    pLanguageTable = Table.GetLanguageTable(English);
    TableSize = LanguageTableSize(pLanguageTable);
    WriteLanguage((const char*)pLanguageTable, "English", TableSize);

    /*Used to give user time to actualy read the output of the
    results of the write attempts*/
    //printf("Press any key to continue.\n");
   // getchar();

    //pLanguageTable = Table.GetLanguageTable(German);
   // TableSize = LanguageTableSize(pLanguageTable);
   // WriteLanguage((const char*)pLanguageTable, "German", TableSize);

    /*Used to give user time to actualy read the output of the
    results of the write attempts*/
    printf("\nPress any key to continue.\n");
    getchar();

    return 0;
}


//Extracts Definitons and Link Tables from a dictionary to be written to file//
void GetDef(CDMOIBDictionary* pDict, CDMOIBStringTable* pTable)
{
    char*           pDef = NULL;
    char*           pDefLink = NULL;
    uint16          DefSize = 0;
    uint32          LinkSize = 0;
    ansichar*       pClassName = NULL;

    uint16 NumDefinitions = pDict->GetDefinitionCount();
    
    for(uint16 i = 0; i < NumDefinitions; i++)
    {
        /*Extract definition bytestream and cast to generic buffer*/
        pDef = (char*)pDict->GetDefinitionAt(i);

        /*Get the size of the file buffer needed*/
        DefSize = DefinitionFullSize(pDict->GetDefinitionAt(i));

        /*pointer check*/
        if(pDef != NULL)
        {
            /*This is done to get the proper ClassName for each Definition
            and Link Table*/
            pClassName = (ansichar*)pTable->GetClassText(DefinitionClassID(pDict->GetDefinitionAt(i)));
            pDefLink = (char*)pTable->GetLinkTable(DefinitionClassID(pDict->GetDefinitionAt(i)));
            LinkSize = LinkTableSize(pDefLink);

            /*Check pointers*/
            if(pDefLink != NULL && pClassName != NULL && pTable != NULL)
            {
                /*Write Definition to file*/
                WriteDef(pDef, pClassName, DefSize);

                /*Write Link Table to file*/
                WriteLink(pDefLink, pClassName, LinkSize);

                /*Used to give user time to actualy read the output of the
                results of the write attempts*/
                //printf("Press any key to continue.\n");
                //getchar();
            }
        }
    }
}


//Writes the Definitions to files with the ClassID as the filename//
void WriteDef(const char* pDef, const ansichar* pClassName, const uint16 DefSize)
{
    FILE*       pFile;
    uint16      WriteSize = 0;
    ansichar    pFileName[256];
    
    strncpy(pFileName, pClassName, 256);
    strcat(pFileName, ".bdd");
    pFile = fopen(pFileName, "w+b");
    
    if(pFile != NULL)
    {
        WriteSize = (uint16)fwrite(pDef, sizeof(char), DefSize, pFile);
        if(WriteSize == DefSize)
        {
            printf("\nDefintion File: %s was  Successfully Written", pFileName);
        }
        else
        {
            printf("\nDefintion File: %s was NOT Successfully Written", pFileName);
        }
        fclose(pFile);
    }
    else
    {
        printf("\nERROR! UNABLE TO OPEN THE FILE: %s!\n", pFileName);
    }

    // Repeat the process fo create a corrupted definition.
    //strncpy(pFileName, "CORRUPT_", 256);
    //strcat(pFileName, pClassName);
    //strcat(pFileName, ".bdd");
    //pFile = fopen(pFileName, "wb");
    //
    //if(pFile != NULL)
    //{
    //    for(uint32 i = DefSize/4; i < (uint32)(DefSize * 3/4); i++)
    //    {
    //        // Flip the bits for the middle 1/2 of the definition to corrupt it.
    //        ((char*)pDef)[i] ^= 0xFF;
    //    }

    //    WriteSize = (uint16)fwrite(pDef, sizeof(char), DefSize, pFile);
    //    if(WriteSize == DefSize)
    //    {
    //        printf("\nDefintion File: %s was  Successfully Written", pFileName);
    //    }
    //    else
    //    {
    //        printf("\nDefintion File: %s was NOT Successfully Written", pFileName);
    //    }
    //    fclose(pFile);
    //}
    //else
    //{
    //    printf("\nERROR! UNABLE TO OPEN THE FILE: %s!\n", pFileName);
    //}
}




//Writes the Link Tables to files with the ClassID as the filename//
void WriteLink(const char* pDefLink, const ansichar* pClassName, const uint32 LinkSize)
{
    FILE*       pFile;
    uint16      WriteSize = 0;
    ansichar    pFileName[256];

    strncpy(pFileName, pClassName, 256);
    strcat(pFileName, ".blt");
    pFile = fopen(pFileName, "wb");

    if(pFile != NULL)
    {
        WriteSize = (uint16)fwrite(pDefLink, sizeof(char), LinkSize, pFile);
        if(WriteSize == LinkSize)
        {
            printf("\nLink File: %s was  Successfully Written", pFileName);
        }
        else
        {
            printf("\nLink File: %s was NOT Successfully Written", pFileName);
        }
        fclose(pFile);
    }
    else
    {
        printf("\nERROR! UNABLE TO OPEN THE FILE: %s!\n", pFileName);
    }

    // Repeat the process fo create a corrupted definition.
    //strncpy(pFileName, "CORRUPT_", 256);
    //strcat(pFileName, pClassName);
    //strcat(pFileName, ".blt");
    //pFile = fopen(pFileName, "wb");

    //if(pFile != NULL)
    //{
    //    for(uint32 i = LinkSize/4; i < LinkSize * 3/4; i++)
    //    {
    //        // Flip the bits for the middle 1/2 of the definition to corrupt it.
    //        ((char*)pDefLink)[i] ^= 0xFF;
    //    }

    //    WriteSize = (uint16)fwrite(pDefLink, sizeof(char), LinkSize, pFile);
    //    if(WriteSize == LinkSize)
    //    {
    //        printf("\nLink File: %s was  Successfully Written", pFileName);
    //    }
    //    else
    //    {
    //        printf("\nLink File: %s was NOT Successfully Written", pFileName);
    //    }
    //    fclose(pFile);
    //}
    //else
    //{
    //    printf("\nERROR! UNABLE TO OPEN THE FILE: %s!\n", pFileName);
    //}
}


//Writes the Language Tables to files with the Language as the filename//
void WriteLanguage(const char* pDefTable, const ansichar* pTableName, const uint32 TableSize)
{
    FILE*       pFile;
    uint16      WriteSize = 0;
    ansichar    pFileName[256];

    strncpy(pFileName, pTableName, 256);
    strcat(pFileName, ".bst");
    pFile = fopen(pFileName, "wb");

    if(pFile != NULL)
    {
        WriteSize = (uint16)fwrite(pDefTable, sizeof(char), TableSize, pFile);
        if(WriteSize == TableSize)
        {
            printf("\nString File: %s was  Successfully Written", pFileName);
        }
        else
        {
            printf("\nString File: %s was NOT Successfully Written", pFileName);
        }
        fclose(pFile);
    }
    else
    {
        printf("\nERROR! UNABLE TO OPEN THE FILE: %s!\n", pFileName);
    }

    // Repeat the process fo create a corrupted definition.
    //strncpy(pFileName, "CORRUPT_", 256);
    //strcat(pFileName, pTableName);
    //strcat(pFileName, ".bst");
    //pFile = fopen(pFileName, "wb");
    //
    //if(pFile != NULL)
    //{
    //    for(uint32 i = TableSize/4; i < TableSize * 3/4; i++)
    //    {
    //        // Flip the bits for the middle 1/2 of the definition to corrupt it.
    //        ((char*)pDefTable)[i] ^= 0xFF;
    //    }

    //    WriteSize = (uint16)fwrite(pDefTable, sizeof(char), TableSize, pFile);
    //    if(WriteSize == TableSize)
    //    {
    //        printf("\nString File: %s was  Successfully Written", pFileName);
    //    }
    //    else
    //    {
    //        printf("\nString File: %s was NOT Successfully Written", pFileName);
    //    }
    //    fclose(pFile);
    //}
    //else
    //{
    //    printf("\nERROR! UNABLE TO OPEN THE FILE: %s!\n", pFileName);
    //}
}
