/*-D*************************************************************************
 *   This program source file is confidential and proprietary property of
 *    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
 *                         Copyright (c) 2003 by
 *                              Welch Allyn Inc.
 *                            All rights reserved
 ****************************************************************************
 ****************************************************************************
 **
 **     A standard header file for use in all Welch Allyn Medical Division
 **     software development involving the C/C++ programming language.
 **
 ****************************************************************************
 **  ////////////////////////////////////////////////////////////////////////
 **  // Target.h    :                                                      //
 **  // version     :   0.0                                                //
 **  // Language    :   C/C++                                              //
 **  // Platform    :   Embedded and host/PC                               //
 **  // Development :                                                      //
 **  // Application :   DTP                                                //
 **  // Authors     :   Welch Allyn Inc. MedR&D                            //
 **  //                                                                    //
 **  // Sources     :                                                      //
 **  ////////////////////////////////////////////////////////////////////////
 ****************************************************************************
 **  
 **  
 ****************************************************************************
 ****************************************************************************
 ** MAINTENANCE HISTORY
 ** Date        Who  Vers    Description
 ** 2003-12-03  RJB  0.0     Initial Creation based on WAHDR.H
 *-E************************************************************************/

/*-D*************************************************************************
 *
 *  Target.H
 *
 *  A standard header file for use in all Welch Allyn Medical Division
 *  software development involving the C programming language.
 *
 *  This header file can be used with the following compilers with the
 *  associated symbol defined:
 *
 *    Compiler                  Version         Symbol
 *    --------                  -------         ------
 *    Microsoft Visual C++      6.0/7.x         _MSC_VER
 *
 *-E************************************************************************/

#if !defined(TARGET_H)
#define TARGET_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Simple macro to make compiler happy when function arguments are not used.
 */
#define TOUCH(X)    (X==X)

/*
 * Note: enums don't seem to work for #if conditional testing under VC++
 */

/*
 * Target Processor Types
 */
#define CPU_UNKNOWN     0   /* Unknown or generic CPU */
#define CPU_X86         1   /* Intel CPU Family */
#define CPU_6833X       2   /* Motorola 6833X CPU Family */
#define CPU_Z180        3   /* Zilog CPU Family */
#define CPU_68K         4   /* 68000 CPU Family */
#define CPU_H8X         5   /* Hitachi H8/H8s Family */
#define CPU_MXL         6   /* Motorola MC9328MXL */
#define CPU_C5000       7   /* Texas Instrument C5000 DSP */


/*
 * Stack Type
 */
 //#define STACK_ETHERNET  1 
 #define STACK_SERIAL  1   

/* Select a RS232 Connection type */
#if defined(STACK_SERIAL)
    /* Turn this on if the DTR line is supported. This allows the 
     * client to avoid sending out a connect message to
     * start the connection with a host 
     */
     //#define STACK_SERIAL_DTR    1  

     //#define STACK_SERIAL_HOST   1  

    /* Set the default baud rate */
    #define DEFAULT_BAUD_RATE   115200

#endif /* STACK_SERIAL */


/*
 * Processor endian Types
 */
#define LITTLE_ENDIAN   0  /* Intel, et. al. */
#define BIG_ENDIAN      1


/*
 * Endianness of data stream on the media.
 */
#define MEDIA_ENDIANNES         BIG_ENDIAN


/*
 * XML support
 */
#define XML_NONE        0
#define XML_SERIALIZE   1
#define XML_DESERIALIZE 2
#define XML_ALL         3

/*
 * DMOIB Specific
 */
/* Define this as 1 to include XML text in definitions, 0 to omit XML text. 
   If XML text is not included in a definition, then objects created from that
   definition CANNOT serialize/deserialize to/from XML, regardless of 
   TARGET_XML_SUPPORT's configuration. */
#define DMOIB_INCLUDE_XML_IN_DEFINITIONS    1

/* If this value is 0, then dynamic arrays reused previously allocated space 
   if possible, and only reallocate memory when they grow larger, minimizing 
   reallocations. If this value is 1, then dynamic arrays reallocate memory 
   every time they change size (smaller or larger), minimizing the memory 
   space used by them. */
#define DMOIB_FORCE_ARRAY_RESIZE            0

/******************************************/
/* Target Processor Dependent Definitions */
/******************************************/

#if defined(_MSC_VER)
/************************************************************************
  Microsoft Visual C++
*************************************************************************/
#define TARGET_CPU              CPU_X86
#define TARGET_ENDIANNESS       LITTLE_ENDIAN
#define TARGET_XML_SUPPORT      XML_SERIALIZE
#define MAX_DLL_COUNT           10

/* Calling convention */
#define WACPAPI                 WINAPI

/* Turn this on for AES Encryption */
//#define ENCRYPTION_ON          1

/*
 * DEBUG
 */
/* Turn this on to view detailed debug information throughtout the system */
//#define DEBUG_PRINT        1 

/* Turn this on to trace all errors in the link and presentation layers */
#define TRACE_SERVER_ON  1 

/* Turn this on to trace all messages received and sent from the link layer 
 * TRACE_SERVER_ON must be defined for this to be active
 */
/* #define TRACE_SERVER_ON_STATUS 1  */

/* Turn this on to view status messages such as the test stub prints */
#define DEBUG_PRINT_STATUS 1

/* Turn this on to print out the dynamic memory usage log */
/*#define MEMORY_DEBUG       1*/

/* Set this to the max message buffer size */
#define RCVBUFSIZE          262144

/*Used to define buffer size in Serialize_to_XMLbuf()*/
#define XML_BUFSIZE          4096

#define MAX_SEND_QUEUE_ITEMS 8192
#define MAX_RECV_QUEUE_ITEMS 8192
#define MAX_SEM_COUNT        8192

/* Connection Object Server defaults */
#define DEFAULT_CONN_SETTING    CONN_CFG_SpKEEPALIVEOFF

#define DEFAULT_TIMEOUT_INTERVAL    8    /* in seconds + 2 for the interval */
#define DEFAULT_KEEPALIVE_INTERVAL  2    /* in seconds */

/* Set the default interval in milliseconds between keep alive checks */
#define DEFAULT_KEEPALIVE_CHECK         (DEFAULT_KEEPALIVE_INTERVAL * 1000)

/* Set the default timeout for RNDZ connections in milliseconds */
#define DEFAULT_RNDZ_TIMEOUT            10000

#define WAIT_COUNT                  100  /* milliseconds */
#define CONNOBJSVR_POLLTIME         1000 /* milliseconds */

/* Set the Non-Overlapped IO polling time */
#define SERIAL_POLLTIME             100  /* milliseconds */

/* Set the wait time used by the server between checks for incoming connections */
#define SERIAL_SERVER_WAIT          2500 /* milliseconds */

/* Defines the total timeout for each asynchronous read request */
#define USB_RECEIVE_TIMEOUT         1000 /* milliseconds */

/* Defines the maximum number of bytes cached when receiving data from the USB Driver */
#define USB_CACHE_SIZE             65536

/* Defines the number of outstanding asynchronous read requests per endpoint that can be queued up */
#define USB_MAX_REQUEST_QUEUE           5

#endif      /* #if defined(_MSC_VER) */

#if defined( __cplusplus )
}
#endif

#endif  /* #if !defined (TARGET_H) */

/* End of File */


