#include "StdVarTyp.h"
#include "CErrEStd.h"
#include "CFamilyEnum.h"
#include "Wacp.h"
#include "CSerialCommSvr.h"
#include "CEthCommSvr.h"
#include "CDeviceDStd.h"
#include "CErrEStd.h"

extern CSerialCommSvr CommSvr;
//extern CEthCommSvr CommSvr;
extern boolean bConnected;

uint8 OutBuffer[MAX_PACKET_SIZE];

uint32 SendMsg(uint32 MsgClassID, uint8* pBuf, uint32 BufSize, uint32 ErrClassID)
{
    uint32 Result = 0;
    uint32 OutSize = 0;

    Result = GeneratePacket(OutBuffer, &OutSize, pBuf, BufSize, MsgClassID, ErrClassID);
    if (Result == WACP_SUCCESS)
    {
        CSerialCommSvr_SendMessage(&CommSvr, OutBuffer, OutSize);
        //CEthCommSvr_SendMessage(&CommSvr, OutBuffer, OutSize);
    }

    return Result;
}

/********************************************************************************************
RecvMessage: Main interface method for receiving a WACP message.
********************************************************************************************/
boolean RecvMessage(const char *pSourceID, uint32 ClassID, uint8 *pBuffer, uint32 BufSize)
{
    boolean bResult = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint16  Family  = (uint16)(ClassID >> 16);
    uint8   Genus = (uint8)(ClassID >> 8);
    uint8   Species = (uint8)(ClassID);

    TOUCH(pSourceID);
    TOUCH(BufSize);

    switch(Family)
    {
        case FmDEVICE:
        {
#if defined(WACP_DEBUG)
            SYS_PRINTF("RecvMessage: FmDEVICE\n");
#endif
            switch(Genus)
            {
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION:
                        {
                            CDeviceDStd NewObj;
                            ansichar GUIDBuf[GUID_STR_SIZE];
                            uint32 Result = 0;

#if defined(WACP_DEBUG)
                            SYS_PRINTF("RecvMessage: FmDEVICE - DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION\n");
#endif
                            CDeviceDStd_Init(&NewObj);
                            Result = CDeviceDStd_Deserialize(&NewObj, pBuffer);
                            if (Result)
                            {
                                SYS_PRINTF("sDateTime.day = %d\n"   , NewObj.m_DateTime.day);
                                SYS_PRINTF("sDateTime.format = %d\n", NewObj.m_DateTime.format);
                                SYS_PRINTF("sDateTime.hour = %d\n"  , NewObj.m_DateTime.hour);
                                SYS_PRINTF("sDateTime.minute = %d\n", NewObj.m_DateTime.minute);
                                SYS_PRINTF("sDateTime.month = %d\n" , NewObj.m_DateTime.month);
                                SYS_PRINTF("sDateTime.second = %d\n", NewObj.m_DateTime.second);
                                SYS_PRINTF("sDateTime.year = %d\n"  , NewObj.m_DateTime.year);
                                SYS_PRINTF("ModelName: %s\n", NewObj.m_ModelName);
                                SYS_PRINTF("SerialNumber: %s\n", NewObj.m_SerialNumber);
                                SYS_PRINTF("Device GUID: %s\n", GUIDToStr(NewObj.m_DeviceGUID, GUIDBuf, sizeof(GUIDBuf)));
                            }
                        }
                    }
                }
            }
            break;
        }
        case FmTEMP:
        {
#if defined(WACP_DEBUG)
            SYS_PRINTF("RecvMessage: FmDEVICE\n");
#endif
            break;
        }
        case FmRENDEZVOUS:
        {
#if defined(WACP_DEBUG_EXT)
            SYS_PRINTF("RecvMessage: FmRENDEZVOUS\n");
#endif
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    /* RNDZ_CCMD_SpSTART_HOST_PROC is sent from the client once RNDZ initialization is
                    * completed. 
                    */
                    if (Species == RNDZ_CCMD_SpSTART_HOST_PROC)
                    {
                        /* Send the HOST READY command and serialize the CRNDZDStd object to send */
                        SendMsg(MakeClassID(FmRENDEZVOUS, GnCOMMAND, RNDZ_CCMD_SpSEND_HOST_READY), NULL, 0, 0);
#if defined(WACP_DEBUG)
                        SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CCMD_SpSTART_HOST_PROC. RNDZ Complete\n");
#endif
                        bConnected = TRUE;
                    }
                    /* RNDZ_CCMD_SpSEND_HOST_READY is sent from the host indicating a successful
                     * RNDZ authentication. This signals the client to terminate the RNDZ client
                     * and to start up the WACP session.
                     */
                    else if (Species == RNDZ_CCMD_SpSEND_HOST_READY)
                    {
                        /* RNDZ authentication complete */
#if defined(WACP_DEBUG)
                        SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CCMD_SpSEND_HOST_READY. RNDZ Complete\n");
#endif
                        bConnected = TRUE;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    if (Species == RNDZ_CRST_SpSTART_RNDZ)
                    {
                        if(strstr((const char*)pBuffer, "RNDZCONNECT"))
                        {   
#if defined(WACP_DEBUG)
                            SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CRST_SpSTART_RNDZ.\n");
#endif

                            /* Send the RNDZ_CRSP_SpRNDZ_ACCEPT Response */
                            SendMsg(MakeClassID(FmRENDEZVOUS, GnRESPONSE, RNDZ_CRSP_SpRNDZ_ACCEPT), NULL, 0, 0);
                        }
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    /* RNDZ_CRSP_SpRNDZ_ACCEPT is sent from the client indicating that the request
                     * to start RNDZ has been accepted.
                     */
                    if (Species == RNDZ_CRSP_SpRNDZ_ACCEPT)
                    {
                        /* If CRNDZDStd is supported, then serialize the object that contains
                         * the GUID and send it out */
                        /* CRNDZDStd_Serialize(&GUIDStd, pObjBuffer); */

                        /* Send the START HOST PROC command */
                        SendMsg(MakeClassID(FmRENDEZVOUS, GnCOMMAND, RNDZ_CCMD_SpSTART_HOST_PROC), NULL, 0, 0);
                    }
                    break;
                }
            }
            break;
        }
        case FmCONNECTION:
        {
#if defined(WACP_DEBUG_EXT)
            SYS_PRINTF("RecvMessage: FmCONNECTION\n");
#endif
            if (Species == CONN_CMD_SpSHUTDOWN)
            {
#if defined(WACP_DEBUG)
                SYS_PRINTF("RecvMessage: FmCONNECTION - CONN_CMD_SpSHUTDOWN\n");
#endif
                /* Handle Shutdown sequence */
            }
            break;
        }
        default:
        {
            ErrorCode = CERR_PROC_UNIMPL;
            SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRMSGPROC), NULL, 0, ClassID);
            break;
        }
    }

    return bResult;
}

/********************************************************************************************
RecvError: Main interface method for receiving an Error message.
********************************************************************************************/
boolean RecvError(const char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8 *pBuffer, uint32 BufSize)
{
    boolean Result  = TRUE;
    uint16  Family  = (uint16)(ClassID >> 16);
    uint8   Genus   = (uint8)(ClassID >> 8);
    uint8   Species = (uint8)ClassID;

    TOUCH(pSourceID);
    TOUCH(BufSize);
    TOUCH(pBuffer);

    if(ErrorType == CERR_SpERRNONE)
    {
#if defined(WACP_DEBUG)
        SYS_PRINTF("RxError: ACK F:%d G:%d S:%d\n", Family, Genus, Species);
#endif
    }
    else 
    {
#if defined(WACP_DEBUG)
        /* Handle simple NACK notification */
        SYS_PRINTF("RxError: NACK F:%d G:%d S:%d\n", Family, Genus, Species);
#endif
    }

    return Result;
}