/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmDATATRANSFERStub.c                                             //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.7.4                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmDATATRANSFERStub.h"
#include "CDataTransferCDesc.h"
#include "CDataTransferDExtStd.h"
#include "CFileSysPSelector.h"
#include "CErrEStd.h"

#include "<DDS_NAME>.h"

#define BDD_MAX_PACKET_SIZE 64
#define BDD_LENGTH <DDS_NAME>_BDD_SIZE
#define bdd_data <DDS_NAME>_BDD

static CDataTransferCDesc DataTransferCDesc;
static CDataTransferDExtStd DataTransferDExtStd;
static uint8 datatransfer_state = 0;
static const ansichar FileName[] = "\\Device.bdds";
#define FILENAME_SIZE 12

/*================================================================================
RecvCommand : CommandID matches the Species of the command,and Buffer is
a generic byte Buffer holding data defined by the developer
================================================================================*/
boolean CFmDATATRANSFERStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint32  MsgClassID = MakeClassID(FmDATATRANSFER, Genus, Species);
	uint8* ObjBuffer = NULL;
	uint32  ObjSize = 0;
	uint32 sequence_number;
	uint8 ErrorSpecies = CERR_SpERRMSGPROC;
	CFileSysPSelector FileSysPSelector;
	uint8 i;

    TOUCH(Buffer);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case DATATRANSFER_CCMD_SpEXECUTE_DTT:
					/*	Data Transmission Terminate:
						Reset the persisted objects
						ACK;	
						reset datatransfer_state;
					*/ 
					CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
					
	                WACP_SendTrap(pSourceID,
	                              MsgClassID,
	                              MsgClassID,
	                              CERR_SpERRNONE,
	                              0,
	                              NULL,
	                              0,
	                              NULL, 
	                              0);
					ErrorCode = CERR_PROC_NONE;
					datatransfer_state = 0;
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case DATATRANSFER_CRST_SpGET_CONFIG:
					/*	Get Config:
						Deserialize;
						Validate data;
						Verify role;
						Verify Initial sequence number
						Set BDD length, packet size;
						Send response;
						Set datatransfer_state;
					*/
					CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
				
					if (CDataTransferCDesc_Deserialize(&DataTransferCDesc, Buffer))
					{
						if (DataTransferCDesc.m_Role == CDTACDSC_Role_DATA_RECIPIENT)
							DataTransferCDesc.m_Role = CDTACDSC_Role_DATA_SOURCE;
						else
						{
							ErrorCode = CERR_PROC_UNSUPPORTED;
							CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
							break;
						}
						
						/*Verify that the intial sequence number is valid*/
						if ((0 == DataTransferCDesc.m_InitSeqNumber) ||
						    (DataTransferCDesc.m_InitSeqNumber > BDD_LENGTH))
						{
						    ErrorCode = CERR_PROC_INVL_PARAM;
						    CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE); 
							break;
						}
						
						if ((NULL != DataTransferCDesc.m_Descriptor) && (DataTransferCDesc.m_DescriptorSize > 0))
						{
							CFileSysPSelector_Init(&FileSysPSelector);
							ObjBuffer = CDataTransferCDesc_GetDescriptorAt(&DataTransferCDesc, 0, &ObjSize);
							if (!CFileSysPSelector_Deserialize(&FileSysPSelector, ObjBuffer))
							{
								ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
								break;
							}
							ObjBuffer = NULL;
							ErrorCode = CERR_PROC_NONE;
							for (i = 0; i < FILENAME_SIZE; i++)
							{
								if (FileName[i] != (FileSysPSelector.m_PathName[i] & 0xFF))
								{
								    /*PATH_NOT_FOUND Error*/
									ErrorCode = CERR_ERROR_COMMON_PATH_NOT_FOUND; 
									
									/*The ErrorCode is from the Error Common species*/
									ErrorSpecies = CERR_SpERROR_COMMON;
									
									/*Set the AppInternal offset to indicate there is an Application error*/
									SET_APP_INTERNAL_ERROR_OFFSET(ErrorCode);
    								CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
									break;
								}
							}
							
							CFileSysPSelector_Reset(&FileSysPSelector, TRUE, TRUE);
							
							if (ErrorCode)
							{							    
								break;
							}				
						}
						else
						{
							ErrorCode = CERR_PROC_UNSUPPORTED;
							CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
							break;
						}					
												
						/*Set the Total Data Size of the requested transfer*/
						DataTransferCDesc.m_TotalDataSize = BDD_LENGTH;
						
						/*Set the DataPacket size*/
						if ((0 == DataTransferCDesc.m_DataPacketSize) ||
						    (DataTransferCDesc.m_DataPacketSize > BDD_MAX_PACKET_SIZE))
						{
						    DataTransferCDesc.m_DataPacketSize = BDD_MAX_PACKET_SIZE;
						}

	                    /* Get object size */
	                    ObjSize = CDataTransferCDesc_SizeAll(&DataTransferCDesc);

	                    /* Allocate memory */
	                    if((ObjBuffer = (uint8*)SYS_REALLOC(ObjBuffer,ObjSize)) != NULL)
	                    {
	                        /* Serialize object */
	                        if (CDataTransferCDesc_Serialize(&DataTransferCDesc,ObjBuffer))
	                        {
	                            /* Send serialized object */
	                            SEND_RESPONSE(pSourceID,
	                                          FmDATATRANSFER,
	                                          DATATRANSFER_CRSP_SpPUT_CONFIG,
	                                          ObjBuffer,
	                                          ObjSize,
	                                          FALSE);
								ErrorCode = CERR_PROC_NONE;
								datatransfer_state = 1;
	                        }
	                        else
	                        {
	                            ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
	                            CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
	                        }
	                        SYS_FREE(ObjBuffer);
	                    }
	                    else
	                    {
	                        ErrorCode = CERR_PROC_MEMORY;
	                        CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
	                    }
					}
					else
					{
						ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
					}
                    break;
                case DATATRANSFER_CRST_SpGET_DATA:
					/*	Get Data:
						Verify that DTR and Get config has happened (datatransfer_state = 2);
						Deserialize;
						Validate data;
						Verify Indentifier;
						Validate sequence number;
						Init Object;
						Set Sequence, Idenifier, Data Size and Data pointer;
						Send Response;
					*/
					if (datatransfer_state != 2)
					{
						ErrorCode = CERR_PROC_CONTEXT;
						break;
					}
					CDataTransferDExtStd_Init(&DataTransferDExtStd);
					if (CDataTransferDExtStd_Deserialize(&DataTransferDExtStd, Buffer))
					{
						if (DataTransferDExtStd.m_Identifier != DataTransferCDesc.m_Identifier)
						{
							ErrorCode = CERR_PROC_INVL_PARAM;
							CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
							break;
						}
						if (0 == DataTransferDExtStd.m_SeqNumber || DataTransferDExtStd.m_SeqNumber > BDD_LENGTH)
						{
							ErrorCode = CERR_PROC_INVL_PARAM;
							CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
							break;
						}
						sequence_number = DataTransferDExtStd.m_SeqNumber;
						
						CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
						
						DataTransferDExtStd.m_SeqNumber = sequence_number;
						DataTransferDExtStd.m_Identifier = DataTransferCDesc.m_Identifier;
						DataTransferDExtStd.m_DataSize = (sequence_number + DataTransferCDesc.m_DataPacketSize) > BDD_LENGTH ?
												BDD_LENGTH - sequence_number + 1 :
												DataTransferCDesc.m_DataPacketSize;
						DataTransferDExtStd.m_Data = &(bdd_data[sequence_number - 1]);
						
	                    /* Get object size */
	                    ObjSize = CDataTransferDExtStd_SizeAll(&DataTransferDExtStd);

	                    /* Allocate memory */
	                    if((ObjBuffer = (uint8*)SYS_REALLOC(ObjBuffer,ObjSize)) != NULL)
	                    {
	                        /* Serialize object */
	                        if (CDataTransferDExtStd_Serialize(&DataTransferDExtStd,ObjBuffer))
	                        {
	                            /* Send serialized object */
	                            SEND_RESPONSE(pSourceID,
	                                          FmDATATRANSFER,
	                                          DATATRANSFER_CRSP_SpPUT_DATA,
	                                          ObjBuffer,
	                                          ObjSize,
	                                          FALSE);
								ErrorCode = CERR_PROC_NONE;
	                        }
	                        else
	                        {
	                            ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
	                            CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
	                        }
	                        SYS_FREE(ObjBuffer);
	                    }
	                    else
	                    {
	                        ErrorCode = CERR_PROC_MEMORY;
	                        CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
	                    }
					}
					else
					{
					    ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;					    
					}
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case DATATRANSFER_CRSP_SpPUT_CONFIG:
                    break;
                case DATATRANSFER_CRSP_SpPUT_DATA:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnSTATUS:
        {
            switch(Species)
            {
                case DATATRANSFER_CSS_SpREPORT_DTR:
					/* 	Data Transmission Ready:
						Verify that Get config has happened (datatransfer_state = 1);
						Deserialize;
						Validate data;
						Verify that Transmission Identifier matches;
						ACK;
					*/
					if (datatransfer_state != 1)
					{
						ErrorCode = CERR_PROC_CONTEXT;
						break;
					}
					CDataTransferDExtStd_Init(&DataTransferDExtStd);
					if (CDataTransferDExtStd_Deserialize(&DataTransferDExtStd, Buffer))
					{
						if (DataTransferDExtStd.m_Identifier != DataTransferCDesc.m_Identifier)
						{
							ErrorCode = CERR_PROC_INVL_PARAM;
							CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
							break;
						}
		                WACP_SendTrap(pSourceID,
		                              MsgClassID,
		                              MsgClassID,
		                              CERR_SpERRNONE,
		                              0,
		                              NULL,
		                              0,
		                              NULL, 
		                              0);
						ErrorCode = CERR_PROC_NONE;
						datatransfer_state = 2;						
					}
					else
					{
					    ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
					}
					CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
                    break;
                case DATATRANSFER_CSS_SpREPORT_DTC:
					/* 	Data Transmission Complete:
					    Reset the persisted objects
						ACK;
						reset datatransfer_state;
					*/					
					CDataTransferCDesc_Reset(&DataTransferCDesc, TRUE, TRUE);
					
	                WACP_SendTrap(pSourceID,
	                              MsgClassID,
	                              MsgClassID,
	                              CERR_SpERRNONE,
	                              0,
	                              NULL,
	                              0,
	                              NULL, 
	                              0);
					ErrorCode = CERR_PROC_NONE;
					datatransfer_state = 0;
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

     /* Any Errors? */
    if (ErrorCode == CERR_PROC_NONE)
    {
        MsgResult = TRUE;
    }

    /* Send a TRAP message if we have an application error */
    if (APP_INTERNAL_ERROR(ErrorCode))
    {
        /* Clear the error offset */
        CLEAR_APP_INTERNAL_ERROR_OFFSET(ErrorCode);

        /* Send a trap with raw error code and appropriate Species */
        WACP_SendTrap(pSourceID,
                      MsgClassID,
                      MsgClassID,
                      ErrorSpecies,
                      ErrorCode,
                      NULL,
                      0,
                      NULL, 
                      0);
        MsgResult = TRUE;
    }

   /* Send a TRAP message if we have a unimplemented or unsupported error */
    if (!MsgResult)
    {
        SEND_MSGPROC_TRAP(pSourceID, MsgClassID, ErrorCode);
    }

    return MsgResult;
}
