<?xml version="1.0" encoding="UTF-8"?>
<!--The DDS_to_Table.xsl will parse a DDS and create html table for each object
that notes the member type, member name, and comments. -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text" version="1.0" encoding="UTF-8" indent="no" omit-xml-declaration="yes"/>
  <xsl:variable name="MemberID" select="1"/>

  <xsl:template match="/DDS">
    <xsl:text>&lt;html&gt;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>&lt;body&gt;</xsl:text>
    <xsl:text>&#10;</xsl:text>

    <!--Creates section header for the Appendix-->
    <xsl:text>&lt;h1&gt;</xsl:text>
    <xsl:text>Appendix</xsl:text>
    <xsl:text>&lt;/h1&gt;</xsl:text>

      <!--***************************************************************************-->
      <!--Start: Family parsing to create an appendix-->
      <!--***************************************************************************-->
      <xsl:for-each select="FAMILY">
        <xsl:text>&lt;h2&gt;</xsl:text>
        <xsl:text>Family: </xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>&lt;/h2&gt;</xsl:text>
        <xsl:for-each select="DEFINITION">

          <!--Print the Object Name in a header-->
          <xsl:text>&lt;p&gt;</xsl:text>
          <xsl:text>&lt;span style='font-size:12.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:text>&lt;b&gt;</xsl:text>
          <xsl:text>Object: </xsl:text>
          <xsl:value-of select="@class"/>
          <xsl:text>&lt;/b&gt;</xsl:text>
          <xsl:text>&lt;/span&gt;</xsl:text>
          <xsl:text>&lt;/p&gt;&#10;</xsl:text>

          <!--Create a table-->
          <xsl:text>&lt;table border="1" cellspacing="0" cellpadding="0"&gt;&#10;</xsl:text>

          <!--Create a row-->
          <xsl:text>&lt;tr&gt;&#10;</xsl:text>

          <!--Create the Member ID column-->
          <xsl:text>&lt;td width=50 valign=top &gt;&#10;</xsl:text>
          <xsl:text>&lt;p&gt;</xsl:text>
          <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:text>&lt;b&gt;</xsl:text>
          <xsl:text>Member ID:</xsl:text>
          <xsl:text>&lt;/b&gt;</xsl:text>
          <xsl:text>&lt;/span&gt;</xsl:text>
          <xsl:text>&lt;/p&gt;&#10;</xsl:text>
          <xsl:text>&lt;/td&gt;&#10;</xsl:text>

          <!--Create the Type column-->
          <xsl:text>&lt;td width=75 valign=top &gt;&#10;</xsl:text>
          <xsl:text>&lt;p&gt;</xsl:text>
          <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:text>&lt;b&gt;</xsl:text>
          <xsl:text>Type:</xsl:text>
          <xsl:text>&lt;/b&gt;</xsl:text>
          <xsl:text>&lt;/span&gt;</xsl:text>
          <xsl:text>&lt;/p&gt;&#10;</xsl:text>
          <xsl:text>&lt;/td&gt;&#10;</xsl:text>

          <!--Create the Size column-->
          <xsl:text>&lt;td width=100 valign=top &gt;&#10;</xsl:text>
          <xsl:text>&lt;p&gt;</xsl:text>
          <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:text>&lt;b&gt;</xsl:text>
          <xsl:text>Size:</xsl:text>
          <xsl:text>&lt;/b&gt;</xsl:text>
          <xsl:text>&lt;/span&gt;</xsl:text>
          <xsl:text>&lt;/p&gt;&#10;</xsl:text>
          <xsl:text>&lt;/td&gt;&#10;</xsl:text>

          <!--Create the Name column-->
          <xsl:text>&lt;td width=150 valign=top&gt;&#10;</xsl:text>
          <xsl:text>&lt;p&gt;</xsl:text>
          <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:text>&lt;b&gt;</xsl:text>
          <xsl:text>Name:</xsl:text>
          <xsl:text>&lt;/b&gt;</xsl:text>
          <xsl:text>&lt;/span&gt;</xsl:text>
          <xsl:text>&lt;/p&gt;&#10;</xsl:text>
          <xsl:text>&lt;/td&gt;&#10;</xsl:text>

          <!--Create the Description column-->
          <xsl:text>&lt;td width=250 valign=top&gt;&#10;</xsl:text>
          <xsl:text>&lt;p&gt;</xsl:text>
          <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:text>&lt;b&gt;</xsl:text>
          <xsl:text>Description:</xsl:text>
          <xsl:text>&lt;/b&gt;</xsl:text>
          <xsl:text>&lt;/span&gt;</xsl:text>
          <xsl:text>&lt;/p&gt;&#10;</xsl:text>
          <xsl:text>&lt;/td&gt;&#10;</xsl:text>

          <!--Terminate the first row-->
          <xsl:text>&lt;/tr&gt;&#10;</xsl:text>

          <xsl:text>&#10;</xsl:text>
          <xsl:for-each select="MEMBERS/MEMBER">

            <!--Create new rows for each member-->
            <xsl:text>&lt;tr&gt;&#10;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>

            <!--Print the type in the Member ID column-->
            <xsl:text>&lt;td&gt;&#10;</xsl:text>
            <xsl:variable name="Count">
              <xsl:number count="MEMBERS/MEMBER" />
            </xsl:variable>

            <xsl:value-of select="$Count - 1"/>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Print the type in the Type column-->
            <xsl:text>&lt;td&gt;&#10;</xsl:text>
            <xsl:value-of select="@type"/>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Print the size in the Size column-->
            <xsl:text>&lt;td&gt;&#10;</xsl:text>
            <xsl:choose>
              <xsl:when test="@size">
                <xsl:value-of select="@size"/>
              </xsl:when>
              <!--Print 1 if the size attribute is not available-->
              <xsl:otherwise>
                <xsl:text>1</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Print the name in the Name column-->
            <xsl:text>&lt;td&gt;&#10;</xsl:text>
            <xsl:value-of select="@name"/>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Print the comment in the Description column-->
            <xsl:text>&lt;td&gt;&#10;</xsl:text>
            <xsl:value-of select="@comment"/>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/tr&gt;&#10;</xsl:text>

          </xsl:for-each>

          <!--Close the table-->
          <xsl:text>&lt;/table&gt;&#10;</xsl:text>

          <!--Add a space between tables-->
          <xsl:text>&lt;p&gt;</xsl:text>
          <xsl:text>&lt;/p&gt;&#10;</xsl:text>

          <xsl:text>&#10;</xsl:text>

          <!--Create an enumeration table-->
          <xsl:for-each select="ENUMERATION">
            <!--Print the Object Name in a header-->
            <xsl:text>&lt;p&gt;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
            <xsl:text>&lt;b&gt;</xsl:text>
            <xsl:text>Enumeration: </xsl:text>
            <xsl:value-of select="@name"/>
            <xsl:text>&lt;/b&gt;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/p&gt;&#10;</xsl:text>

            <!--Create a table-->
            <xsl:text>&lt;table border="1" cellspacing="0" cellpadding="0"&gt;&#10;</xsl:text>

            <!--Create a row-->
            <xsl:text>&lt;tr&gt;&#10;</xsl:text>

            <!--Create the Name column-->
            <xsl:text>&lt;td width=100 valign=top &gt;&#10;</xsl:text>
            <xsl:text>&lt;p&gt;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
            <xsl:text>&lt;b&gt;</xsl:text>
            <xsl:text>Name:</xsl:text>
            <xsl:text>&lt;/b&gt;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/p&gt;&#10;</xsl:text>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Create the value column-->
            <xsl:text>&lt;td width=100 valign=top &gt;&#10;</xsl:text>
            <xsl:text>&lt;p&gt;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
            <xsl:text>&lt;b&gt;</xsl:text>
            <xsl:text>Value:</xsl:text>
            <xsl:text>&lt;/b&gt;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/p&gt;&#10;</xsl:text>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Create the Description column-->
            <xsl:text>&lt;td width=100 valign=top &gt;&#10;</xsl:text>
            <xsl:text>&lt;p&gt;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
            <xsl:text>&lt;b&gt;</xsl:text>
            <xsl:text>Description:</xsl:text>
            <xsl:text>&lt;/b&gt;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/p&gt;&#10;</xsl:text>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Terminate the first row-->
            <xsl:text>&lt;/tr&gt;&#10;</xsl:text>

            <xsl:for-each select="ENUM">
              <xsl:text>&lt;tr&gt;&#10;</xsl:text>
              <!--Print the name in the Name column-->
              <xsl:text>&lt;td&gt;&#10;</xsl:text>
              <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
              <xsl:value-of select="@name"/>
              <xsl:text>&lt;/span&gt;</xsl:text>
              <xsl:text>&lt;/td&gt;&#10;</xsl:text>

              <!--Print the name in the value column-->
              <xsl:text>&lt;td&gt;&#10;</xsl:text>
              <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
              <xsl:choose>
                <xsl:when test="@defaultvalue">
                  <xsl:value-of select="@defaultvalue"/>
                </xsl:when>
                <!--Print nothing if the defaultvalue attribute is not available-->
                <xsl:otherwise>
                  <xsl:text></xsl:text>
                </xsl:otherwise>
              </xsl:choose>
              <xsl:text>&lt;/span&gt;</xsl:text>
              <xsl:text>&lt;/td&gt;&#10;</xsl:text>

              <!--Print the comment in the Description column-->
              <xsl:text>&lt;td&gt;&#10;</xsl:text>
              <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
              <xsl:value-of select="@comment"/>
              <xsl:text>&lt;/span&gt;</xsl:text>
              <xsl:text>&lt;/td&gt;&#10;</xsl:text>

              <xsl:text>&lt;/tr&gt;&#10;</xsl:text>

            </xsl:for-each>

            <!--Close the table-->
            <xsl:text>&lt;/table&gt;&#10;</xsl:text>

          </xsl:for-each>

          <xsl:text>&#10;</xsl:text>

          <!--Create a Species table-->
          <xsl:for-each select="SPECIES_TABLE">
            <!--Print the Object Name in a header-->
            <xsl:text>&lt;p&gt;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
            <xsl:text>&lt;b&gt;</xsl:text>
            <xsl:text>Species: </xsl:text>
            <xsl:text>&lt;/b&gt;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/p&gt;&#10;</xsl:text>

            <!--Create a table-->
            <xsl:text>&lt;table border="1" cellspacing="0" cellpadding="0"&gt;&#10;</xsl:text>

            <!--Create a row-->
            <xsl:text>&lt;tr&gt;&#10;</xsl:text>

            <!--Create the Name column-->
            <xsl:text>&lt;td width=100 valign=top &gt;&#10;</xsl:text>
            <xsl:text>&lt;p&gt;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
            <xsl:text>&lt;b&gt;</xsl:text>
            <xsl:text>Name:</xsl:text>
            <xsl:text>&lt;/b&gt;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/p&gt;&#10;</xsl:text>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Create the value column-->
            <xsl:text>&lt;td width=100 valign=top &gt;&#10;</xsl:text>
            <xsl:text>&lt;p&gt;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
            <xsl:text>&lt;b&gt;</xsl:text>
            <xsl:text>Value:</xsl:text>
            <xsl:text>&lt;/b&gt;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/p&gt;&#10;</xsl:text>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Create the Description column-->
            <xsl:text>&lt;td width=100 valign=top &gt;&#10;</xsl:text>
            <xsl:text>&lt;p&gt;</xsl:text>
            <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
            <xsl:text>&lt;b&gt;</xsl:text>
            <xsl:text>Description:</xsl:text>
            <xsl:text>&lt;/b&gt;</xsl:text>
            <xsl:text>&lt;/span&gt;</xsl:text>
            <xsl:text>&lt;/p&gt;&#10;</xsl:text>
            <xsl:text>&lt;/td&gt;&#10;</xsl:text>

            <!--Terminate the Species row-->
            <xsl:text>&lt;/tr&gt;&#10;</xsl:text>

            <xsl:for-each select="SPECIES">
              <xsl:text>&lt;tr&gt;&#10;</xsl:text>

              <!--Print the name in the Name column-->
              <xsl:text>&lt;td&gt;&#10;</xsl:text>
              <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
              <xsl:value-of select="@name"/>
              <xsl:text>&lt;/span&gt;</xsl:text>
              <xsl:text>&lt;/td&gt;&#10;</xsl:text>

              <!--Print the default value in the value column-->
              <xsl:text>&lt;td&gt;&#10;</xsl:text>
              <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
              <xsl:choose>
                <xsl:when test="@defaultvalue">
                  <xsl:value-of select="@defaultvalue"/>
                </xsl:when>
                <!--Print nothing if the defaultvalue attribute is not available-->
                <xsl:otherwise>
                  <xsl:text></xsl:text>
                </xsl:otherwise>
              </xsl:choose>
              <xsl:text>&lt;/span&gt;</xsl:text>
              <xsl:text>&lt;/td&gt;&#10;</xsl:text>

              <!--Print the comment in the Description column-->
              <xsl:text>&lt;td&gt;&#10;</xsl:text>
              <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
              <xsl:value-of select="@comment"/>
              <xsl:text>&lt;/span&gt;</xsl:text>
              <xsl:text>&lt;/td&gt;&#10;</xsl:text>

              <!--Terminate the row-->
              <xsl:text>&lt;/tr&gt;&#10;</xsl:text>
            </xsl:for-each>

            <!--Close the table-->
            <xsl:text>&lt;/table&gt;&#10;</xsl:text>

          </xsl:for-each>

          <!--Add a space between tables-->
          <xsl:text>&lt;p&gt;</xsl:text>
          <xsl:text>&lt;/p&gt;&#10;</xsl:text>
        </xsl:for-each>

        <xsl:text>&lt;h3&gt;</xsl:text>
        <xsl:text>Messages</xsl:text>
        <xsl:text>&lt;/h3&gt;</xsl:text>

        <!--Create a messages  table-->
        <xsl:text>&lt;table border="1" cellspacing="0" cellpadding="0"&gt;&#10;</xsl:text>

        <!--Create a row-->
        <xsl:text>&lt;tr&gt;&#10;</xsl:text>
        <!--Create the Type column-->
        <xsl:text>&lt;td width=100 valign=top&gt;&#10;</xsl:text>
        <xsl:text>&lt;p&gt;</xsl:text>
        <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
        <xsl:text>&lt;b&gt;</xsl:text>
        <xsl:text>Type:</xsl:text>
        <xsl:text>&lt;/b&gt;</xsl:text>
        <xsl:text>&lt;/span&gt;</xsl:text>
        <xsl:text>&lt;/p&gt;&#10;</xsl:text>
        <xsl:text>&lt;/td&gt;&#10;</xsl:text>

        <!--Create the Name column-->
        <xsl:text>&lt;td width=100 valign=top&gt;&#10;</xsl:text>
        <xsl:text>&lt;p&gt;</xsl:text>
        <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
        <xsl:text>&lt;b&gt;</xsl:text>
        <xsl:text>Name:</xsl:text>
        <xsl:text>&lt;/b&gt;</xsl:text>
        <xsl:text>&lt;/span&gt;</xsl:text>
        <xsl:text>&lt;/p&gt;&#10;</xsl:text>
        <xsl:text>&lt;/td&gt;&#10;</xsl:text>

        <!--Create the Description column-->
        <xsl:text>&lt;td width=250 valign=top&gt;&#10;</xsl:text>
        <xsl:text>&lt;p&gt;</xsl:text>
        <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
        <xsl:text>&lt;b&gt;</xsl:text>
        <xsl:text>Description:</xsl:text>
        <xsl:text>&lt;/b&gt;</xsl:text>
        <xsl:text>&lt;/span&gt;</xsl:text>
        <xsl:text>&lt;/p&gt;&#10;</xsl:text>
        <xsl:text>&lt;/td&gt;&#10;</xsl:text>

        <xsl:text>&lt;/tr&gt;&#10;</xsl:text>



        <xsl:for-each select="MESSAGES/MSG">
          <xsl:text>&lt;tr&gt;&#10;</xsl:text>

          <!--Print the type column-->
          <xsl:text>&lt;td&gt;&#10;</xsl:text>
          <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:value-of select="@type"/>
          <xsl:text>&lt;/span&gt;</xsl:text>
          <xsl:text>&lt;/td&gt;&#10;</xsl:text>

          <!--Print the name  column-->
          <xsl:text>&lt;td&gt;&#10;</xsl:text>
          <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:value-of select="@name"/>
          <xsl:text>&lt;/span&gt;</xsl:text>
          <xsl:text>&lt;/td&gt;&#10;</xsl:text>

          <!--Print the comment in the Description column-->
          <xsl:text>&lt;td&gt;&#10;</xsl:text>
          <xsl:text>&lt;span style='font-size:10.0pt;font-family:"Times New Roman"'&gt;</xsl:text>
          <xsl:value-of select="@comment"/>
          <xsl:text>&lt;/td&gt;&#10;</xsl:text>
          <xsl:text>&lt;/span&gt;</xsl:text>


          <xsl:text>&lt;/tr&gt;&#10;</xsl:text>

        </xsl:for-each>
        <!--Close the table-->
        <xsl:text>&lt;/table&gt;&#10;</xsl:text>
      </xsl:for-each>

    <xsl:text>&#10;</xsl:text>
    <xsl:text>&lt;/body&gt;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>&lt;/html&gt;</xsl:text>

  </xsl:template>
</xsl:stylesheet>
