<?xml version="1.0" encoding="UTF-8"?>
<!--The DDS_to_Table.xsl will parse a DDS and create html table for each object
that notes the member type, member name, and comments. -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="1.0">
  <xsl:output method="html" version="1.0" encoding="ISO-8859-1" indent="no" omit-xml-declaration="yes"/>
  <xsl:variable name="MemberID" select="1"/>
  <xsl:variable name="Object_Name_Overhead" select="1"/>
  <xsl:variable name="MOIB_Header_Overhead" select="1"/>
  <xsl:variable name="CRC_Overhead" select="1"/>
  <xsl:variable name="Total_Byte_Size_Overhead" select="1"/>
  <xsl:variable name="Space_Overhead" select="1"/>
  <xsl:variable name="Payload_Overhead" select="1"/>
  <xsl:variable name="Static_Struct_Size" select="2"/>

  <xsl:template match="/DDS" name="DDS">
    <xsl:param name="newline" select="1"/>
    <html xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:x="urn:schemas-microsoft-com:office:excel" xmlns="http://www.w3.org/TR/REC-html40">
      <!--<xsl:text>&lt;xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:x="urn:schemas-microsoft-com:office:excel" xmlns="http://www.w3.org/TR/REC-html40"&gt;</xsl:text>-->

      <body>
        <!--<xsl:text>&lt;body&gt;</xsl:text>-->

        <xsl:for-each select="FAMILY">

          <h3>
            <xsl:text>Family: </xsl:text>
            <xsl:value-of select="@name"/>
          </h3>
          <xsl:for-each select="DEFINITION">

            <!--Create a table-->
            <table border="1" cellspacing="0" cellpadding="0">

              <!--Create a row-->
              <tr>
                <td width="150" valign="top">

                  <!--Print the Object Name in a header-->
                  <p>
                    <span style='font-size:12.0pt;font-family:"Times New Roman"'>
                      <h5>
                        <xsl:text>Object: </xsl:text>
                        <xsl:value-of select="@class"/>
                      </h5>
                    </span>
                  </p>
                </td>

                <!--Create the Type column-->
                <td width="175" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <xsl:text>Type:</xsl:text>
                      </b>
                    </span>
                  </p>
                </td>

                <!--Create the Name column-->
                <td width="250" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <xsl:text>Name:</xsl:text>
                      </b>
                    </span>
                  </p>
                </td>

                <!--Create the Number of Elements column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <xsl:text>Number of Elements:</xsl:text>
                      </b>
                    </span>
                  </p>
                </td>

                <!--Create the Byte Size column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <xsl:text>Byte Size:</xsl:text>
                      </b>
                    </span>
                  </p>
                </td>

                <!--Terminate the first row-->
              </tr>


              <!--Create the second row-->
              <tr>
                <td width="50" valign="top">

                  <!--Print the MOIB header-->
                  <p>
                    <span style='font-size:12.0pt;font-family:"Times New Roman"'>
                      <h5>
                        <xsl:text>MOIB Header</xsl:text>
                      </h5>
                    </span>
                  </p>
                </td>

                <!--Create the Type column-->
                <td width="75" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Create the Name column-->
                <td width="150" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Create the Number of Elements column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Create the Byte Size column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>

                        <!--Calculate the MOIB Header byte size. In the byte size calculation, the size value changes
                        based on the current family, genus, and species values. The calculatio is as follows:
                        ClassID - 4 bytes
                        Size - 2/4 bytes
                        Version - 2 bytes
                        Bit Field - 1 byte-->

                        <xsl:choose>

                          <xsl:when test="@family = 'FmSESSION' and @genus != 'GnPARAMETER'">
                            <xsl:text>11</xsl:text>
                          </xsl:when>

                          <xsl:when test="@family = 'FmDTCOLLECTION'">
                            <xsl:text>11</xsl:text>
                          </xsl:when>

                          <xsl:when test="@family = 'GnDATAEXT'">
                            <xsl:text>11</xsl:text>
                          </xsl:when>

                          <xsl:when test="@family = 'FmSPIRO' and @genus = 'GnDATA' and @species = 'SpSTANDARD'">
                            <xsl:text>11</xsl:text>
                          </xsl:when>

                          <xsl:when test="@family = 'FmECG' and @genus = 'GnDATA' and @species = 'SpPACER'">
                            <xsl:text>11</xsl:text>
                          </xsl:when>

                          <xsl:when test="@family = 'FmCALIBRATION' and @genus = 'GnDATA' and @species = 'SpSPIRO'">
                            <xsl:text>11</xsl:text>
                          </xsl:when>

                          <xsl:when test="@family = 'FmTEST' and @genus = 'GnDATA' and @species = 'SpSESSION'">
                            <xsl:text>11</xsl:text>
                          </xsl:when>

                          <xsl:otherwise>
                            <xsl:text>9</xsl:text>
                          </xsl:otherwise>

                        </xsl:choose>

                      </b>
                    </span>
                  </p>
                </td>

                <!--Terminate the second row-->
              </tr>


              <!-- The Member_Count variable contains the total number of members up to the current member location in the current object. -->
              <xsl:variable name="Member_Count" select="count(MEMBERS/MEMBER)"/>

              <!-- The Previous_Members variable contains the total number of previous members from the current member location. -->
              <xsl:variable name="Previous_Members" select="count(preceding::MEMBERS/MEMBER)"/>

              <!-- The Previous_Definitions variable contains the total number of previous definitions from the current definition location. -->
              <xsl:variable name="Previous_Definitions" select="count(preceding::DEFINITION)"/>

              <!-- The Previous_Familys variable contains the total number of previous Familys from the current Family location. The calculation has a +1 in order to start the count
              at 1, as opposed to 0.-->
              <xsl:variable name="Previous_Familys" select="count(preceding::FAMILY) + 1"/>

              <!-- The SubOject_Inherited_Count variable contains the total number of members that are Aggregate/Inherited objects -->
              <xsl:variable name="SubObject_Inherited_Count" select="count(MEMBERS/MEMBER[@type = 'OBJECT' and @class and not(@size)])"/>

              <!-- The Vector_Collection_Count variable contains the total number of members that are Record/Collection objects -->
              <xsl:variable name="Vector_Collection_Count" select="count(MEMBERS/MEMBER[@type = 'OBJECT' and @class and @size])"/>

              <!-- The Dynamic_Count variable contains the total number of members that are Dynamic data members -->
              <xsl:variable name="Dynamic_Count" select="count(MEMBERS/MEMBER[@type and (@size = 'UNBOUNDED' or @size = 'UNBOUNDED LINKED') and not(@class)])"/>

              <!--Iterate through each member-->
              <xsl:for-each select="MEMBERS/MEMBER">

                <!--Create new rows for each member-->
                <tr>
                  <span style='font-size:10.0pt;font-family:"Times New Roman"'>

                    <!--Print the type in the first column-->
                    <td>
                      <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                    </td>

                    <!--Print the type in the Type column-->
                    <td>
                      <xsl:value-of select="@type"/>
                      <xsl:choose>
                        <xsl:when test="@class">
                          <xsl:text> = </xsl:text>
                          <xsl:value-of select="@class"/>
                        </xsl:when>
                      </xsl:choose>
                    </td>

                    <!--Print the name in the Name column-->
                    <td>
                      <xsl:value-of select="@name"/>
                    </td>

                    <!--Print the size in the Number of Elements column-->
                    <td>
                      <xsl:choose>
                        <!--Check to see if size is unbounded/unbounded linked and make the size column blank -->

                        <!-- Record/Collection/Dynamic types -->
                        <xsl:when test="@size and ((@size = 'UNBOUNDED') or (@size = 'UNBOUNDED LINKED'))">
                          <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                        </xsl:when>
                        <!--Check to see if the size is valid, and the type is object, then make the column blank -->

                        <!-- Aggregate/Inherited types -->
                        <xsl:when test="@type = 'OBJECT'">
                          <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                        </xsl:when>

                        <!-- Static array types -->
                        <xsl:when test="@size">
                          <xsl:value-of select="@size"/>
                        </xsl:when>

                        <!-- Static types -->
                        <!--Print 1 if the size attribute is not available-->
                        <xsl:otherwise>
                          <xsl:text>1</xsl:text>
                        </xsl:otherwise>
                      </xsl:choose>
                    </td>

                    <!--Print the size of the Byte Size column-->
                    <td>

                      <xsl:variable name="Count">
                        <xsl:number count="MEMBERS/MEMBER" />
                      </xsl:variable>

                      <!--This choose is used to determine the math needed to properly calculate byte size per type. The math done here is to make sure that
                      the correct row is being used for the calcuation-->
                      <xsl:choose>

                        <xsl:when test="@type = 'uint8'">
                          <!--If type is unit8, then number of elements is multiplied by 1 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*1</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'uint16'">
                          <!--If type is unit16, then number of elements is multiplied by 2 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*2</xsl:text>

                        </xsl:when>


                        <xsl:when test="@type = 'uint32'">
                          <!--If type is unit32, then number of elements is multiplied by 4 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*4</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'int8'">
                          <!--If type is int8, then number of elements is multiplied by 1 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*1</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'int16'">
                          <!--If type is nit16, then number of elements is multiplied by 2 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*2</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'int32'">
                          <!--If type is unit32, then number of elements is multiplied by 4 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*4</xsl:text>

                        </xsl:when>


                        <xsl:when test="@type = 'float'">
                          <!--If type is float, then number of elements is multiplied by 4 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*4</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'double'">
                          <!--If type is double, then number of elements is multiplied by 8 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*8</xsl:text>

                        </xsl:when>


                        <xsl:when test="@type = 'ansichar'">
                          <!--If type is unit16, then number of elements is multiplied by 1 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*1</xsl:text>

                        </xsl:when>


                        <xsl:when test="@type = 'widechar'">
                          <!--If type is widechar, then number of elements is multiplied by 2 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*2</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'byte'">
                          <!--If type is byte, then number of elements is multiplied by 1 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*1</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'boolean'">
                          <!--If type is boolean, then number of elements is multiplied by 1 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*1</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'TIME'">
                          <!--If type is TIME, then number of elements is multiplied by 8 -->
                          <xsl:text>=D</xsl:text>
                          <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Count + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                          <xsl:text>*8</xsl:text>

                        </xsl:when>

                        <xsl:when test="@type = 'OBJECT'">
                          <!--If type is OBJECT, then the byte size is left blank -->
                          <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                        </xsl:when>

                        <xsl:otherwise>
                          <!--else case, byte size is left blank -->
                          <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                        </xsl:otherwise>
                      </xsl:choose>

                    </td>
                  </span>
                </tr>
              </xsl:for-each>

              <!--Create the first row after the last dynamic member (Payload Overhead)-->
              <tr>
                <td width="50" valign="top">

                  <!--Blank Column-->
                  <p>
                    <span style='font-size:12.0pt;font-family:"Times New Roman"'>
                      <h5>
                        <xsl:text>Payload Overhead</xsl:text>
                      </h5>
                    </span>
                  </p>
                </td>

                <!--Fill in the Type column-->
                <td width="75" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Name column-->
                <td width="150" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Number of Elements column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Byte Size column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <xsl:value-of select="($SubObject_Inherited_Count*4) + ($Vector_Collection_Count*2) + ($Dynamic_Count*2) + $Static_Struct_Size"/>
                      </b>
                    </span>
                  </p>
                </td>

                <!--Terminate the row-->
              </tr>


              <!--Create the CRC row-->
              <tr>
                <td width="50" valign="top">

                  <!--Create a blank row-->
                  <p>
                    <span style='font-size:12.0pt;font-family:"Times New Roman"'>
                      <h5>
                        <xsl:text>CRC</xsl:text>
                      </h5>
                    </span>
                  </p>
                </td>

                <!--Fill in the Type column-->
                <td width="75" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Name column-->
                <td width="150" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Number of Elements column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Byte Size column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <xsl:text>2</xsl:text>
                      </b>
                    </span>
                  </p>
                </td>

                <!--Terminate the CRC row-->
              </tr>


              <tr>
                <td width="50" valign="top">

                  <!--Fill in the Name column-->
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <xsl:text>Total Byte Size:</xsl:text>
                      </b>
                    </span>
                  </p>
                </td>
                <!--Fill in the type in the Type column-->
                <td width="75" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Name column-->
                <td width="150" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Number of Elements column-->
                <td width="100" valign="top">
                  <p>
                    <span style='font-size:10.0pt;font-family:"Times New Roman"'>
                      <b>
                        <!--<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>-->
                      </b>
                    </span>
                  </p>
                </td>

                <!--Fill in the Byte Size column-->
                <!--Calculation done for proper row alignment (for calculations) is done here:-->
                <!-- 
                Object_Name_Overhead: This is the overhead used by the Object field from the first object.
                MOIB_Header_Overhead: This is the overhead used by the MOIB Header field from the first object.
                Previous_Members: This is the total number of previous members at the current point of processing.
                Previous_Definitions*6: This is the total number of previous definitions at the current point of processing. It is multiplied by 
                                        6 because there is a six row offset per definition after the first definition (Object, MOIB Header, Payload Overhead,
                                        CRC, and Total Byte Size rows. There is also a blank row inbetween Object definitions.)
                Previous_Familys*2: This is the total number of previous family at the current point of processing. It is multiplied by 2
                                    because there is a two row offset per Family (Family: Fm[Family] + blank row after).
                Member_Count: This counts the current member of the current object (works as an incrementing counter).
                CRC_Overhead: This is the CRC overhead used by the CRC field from the first object.
                Payload_Overhead: This is the Payload overhead used by the Payload overhead field from the first object.
                -->

                <td>
                  <span>
                    <b>

                      <xsl:text>=SUM(E</xsl:text>
                      <xsl:value-of select="$Object_Name_Overhead + $MOIB_Header_Overhead + $Previous_Members + ($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                      <xsl:text>:E</xsl:text>
                      <xsl:value-of select="$Member_Count + $Previous_Members + $MOIB_Header_Overhead + $Space_Overhead + $CRC_Overhead + $Payload_Overhead +($Previous_Definitions*6) + ($Previous_Familys*2)"/>
                      <xsl:text>)</xsl:text>
                    </b>
                  </span>
                </td>

                <!--Terminate the Total Byte Size row-->
              </tr>

              <!--Close the table-->
            </table>

            <!--Add a space between tables-->
            <p>
            </p>



            <!--Add a space between tables-->
            <p>
            </p>

          </xsl:for-each>

        </xsl:for-each>


      </body>
      <!--<xsl:text>&lt;/body&gt;</xsl:text>-->

      <!--<xsl:text>&lt;/html&gt;</xsl:text>-->
    </html>
  </xsl:template>
</xsl:stylesheet>
