#include <windows.h>
#include <process.h>
#include <stdio.h>
#include "CSerialCommSvr.h"
#include "CEthCommSvr.h"
#include "CFamilyEnum.h"
#include "SysUtil.h"
#include "DtData.h"
#include "CErrEStd.h"

extern uint32 SendMsg(uint32 MsgClassID, uint8* pBuf, uint32 BufSize, uint32 ErrClassID);

/*'****************************************************************************
'** Disconnect 
'*****************************************************************************/
const uint8 DISCONNECT_MSG[] =
{   0x17, 0x01, 0x0c, //envelope preamble
	0x00, 0x00, 0x00, 0x1a, //envelope size
	0x01, 0x1a, //envelope port
		0x00, 0x1e, 0x01, 0x00,//message class ID 
		0x00, 0x00, 0x00, 0x07,//message package size
		0x00, //message encComp
		0x00, 0x00, 0x00, 0x00, //Message payload data size
		0x61, 0xad, //Message CRC
	0xb2, 0x78 //Envelope CRC
};

/*Thread proccess is using a C++ method. Need to type cast to WINAPI function*/
typedef unsigned (WINAPI* WACP_THREAD_PROC)
(    LPVOID lpThreadParameter);

/* Globals */
//CEthCommSvr     CommSvr;
CSerialCommSvr  CommSvr;
boolean         bConnected;


void StartSerial()
{
    boolean Result;

    CSerialCommSvr_Init(&CommSvr);
    Result = CSerialCommSvr_Connect(&CommSvr, (uint8*)"COM1", RNDZ_PORT_DEFAULT);

    /* Open up the communications channel */
    if (Result)
    {
		/* Launch the reader thread in the CommSvr */
        HANDLE hHandle = (HANDLE)_beginthreadex(NULL, 0,(WACP_THREAD_PROC)CSerialCommSvr_StartClient, &CommSvr, 0, NULL);
        if (hHandle)
        {
            Result = (boolean)SetThreadPriority(hHandle, THREAD_PRIORITY_HIGHEST);
            CloseHandle(hHandle);
        }
    }
}
#if 0
void StartEthernet()
{
    boolean Result;

    CEthCommSvr_Init(&CommSvr);
    Result = CEthCommSvr_Connect(&CommSvr, (uint8*)"127.1", RNDZ_PORT_DEFAULT);

    /* Open up the communications channel */
    if (Result)
    {
		/* Launch the reader thread in the CommSvr */
        HANDLE hHandle = (HANDLE)_beginthreadex(NULL, 0,(WACP_THREAD_PROC)CEthCommSvr_StartClient, &CommSvr, 0, NULL);
        if (hHandle)
        {
            Result = (boolean)SetThreadPriority(hHandle, THREAD_PRIORITY_HIGHEST);
            CloseHandle(hHandle);
        }
    }
}
#endif
void Menu()
{
    int     Data = 0;
    uint32  Result = 0;

	do
	{
		Data = 0;
		SYS_SLEEP(500);
		SYS_PRINTF("\n\n0. Disconnect\n");
		SYS_PRINTF("1. Get Device Description\n");
		SYS_PRINTF("2. Get Temperature\n");
                
		scanf("%d", &Data);
		
		switch(Data)
		{
			case 0: /* Disconnect */
			{
                //CEthCommSvr_SendMessage(&CommSvr, DISCONNECT_MSG, sizeof(DISCONNECT_MSG));
                CSerialCommSvr_SendMessage(&CommSvr, DISCONNECT_MSG, sizeof(DISCONNECT_MSG));
				SYS_SLEEP(1000);
				CSerialCommSvr_Destroy(&CommSvr);
				//CEthCommSvr_Destroy(&CommSvr);
				Data =0;
				break;
			}
			case 1: /* Get Device Description */
			{	
				Result = SendMsg(MakeClassID(FmDEVICE, GnREQUEST, DEVICE_CRST_SpGET_DEVICEDESCRIPTION), NULL, 0, 0);
				break;
			}
			case 2: /* Get Temp */
			{	
				Result = SendMsg(MakeClassID(FmTEMP, GnREQUEST, TEMP_CRST_SpGET_TEMP), NULL, 0, 0);
				break;
			}
			case 3: /* Test ACK */
			{	
				Result = SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRNONE), NULL, 0, MakeClassID(FmNIBP, GnSTATUS, 1));
				break;
			}
			case 4: /* Test NACK */
			{	
				Result = SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRAPPINTERNAL), NULL, 0, MakeClassID(FmNIBP, GnSTATUS, 1));
				break;
			}
			case 5: /* Test NACK with error object */
			{	
                CErrEStd TestError;

                uint8 TestBuf[128];
                uint32 TestSize;

                CErrEStd_Init(&TestError);

                TestError.m_ErrClassID = MakeClassID(FmDEVICE, GnDATA, 3);
                TestError.m_ErrorCode = 33;
                sprintf(TestError.m_Description, "Testing ErrObj");

                TestSize = CErrEStd_SizeAll(&TestError);
                CErrEStd_Serialize(&TestError, TestBuf);
				Result = SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRAPPINTERNAL), TestBuf, TestSize, MakeClassID(FmNIBP, GnSTATUS, 1));
				break;
			}
			default:
			{
						Data = -1;
						SYS_PRINTF("CLS");
						SYS_PRINTF("\n\n---UNSUPPORTED---\n");
			}

		}//End Switch

	}while (Data != 0); //End Do
}


int main()
{

    /* Start up transport */
    StartSerial();
    //StartEthernet();


    while(bConnected == FALSE)
    {
        SYS_SLEEP(200);
    }

    Menu();

}
