/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmDEVICEStub.c                                                   //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmDEVICEStub.h"

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
boolean CFmDEVICEStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint32  MsgClassID = MakeClassID(FmDEVICE, Genus, Species);

    TOUCH(Buffer);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case DEVICE_CCMD_SpWRITE_CONFIG:
                    break;
                case DEVICE_CCMD_SpWRITE_DEVICEDESCRIPTION:
                    break;
                case DEVICE_CCMD_SpENABLE_DISPLAY:
                    break;
                case DEVICE_CCMD_SpSTART_DISPLAY_TEST:
                    break;
                case DEVICE_CCMD_SpRESET:
                    break;
                case DEVICE_CCMD_SpPOWER:
                    break;
                case DEVICE_CCMD_SpBEEP:
                    break;
                case DEVICE_CCMD_SpSAVE_CONFIG:
                    break;
                case DEVICE_CCMD_SpRESTORE_CONFIG:
                    break;
                case DEVICE_CCMD_SpENABLE_AUTO_POWER_DOWN:
                    break;
                case DEVICE_CCMD_SpSET_DISPLAY_CONTRAST:
                    break;
                case DEVICE_CCMD_SpSET_BACKLIGHT:
                    break;
                case DEVICE_CCMD_SpSTART_DIGITAL_TEST:
                    break;
                case DEVICE_CCMD_SpSTOP_DISPLAY_TEST:
                    break;
                case DEVICE_CCMD_SpSET_DEVICE_MODE:
                    break;
                case DEVICE_CCMD_SpSTART_PRINTER_TEST:
                    break;
                case DEVICE_CCMD_SpSET_CALIBRATION:
                    break;
                case DEVICE_CCMD_SpSET_DATE_TIME:
                    break;
                case DEVICE_CCMD_SpNEXT_PATIENT:
                    break;
                case DEVICE_CCMD_SpSET_DEVICE_COMM_MODE:
                    break;
                case DEVICE_CCMD_SpSTART_HARDWARE_TEST:
                    break;
                case DEVICE_CCMD_SpSTOP_HARDWARE_TEST:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case DEVICE_CRST_SpGET_DEVICEDESCRIPTION:
					CDevPMsgArguments DevPMsgArgs;	// Request Object (may or may not exist)
					CDeviceDStd DeviceObj;			// Response Object
					CDeviceDExtStd DeviceDExtStd;	// Response Container (only used when responding to a request for SpALL)

					CDevPMsgArguments_Init(&DevPMsgArgs);  // Initialize the DevPMsgArgs struct

					uint8 ObjBuffer[256];	// Buffer to serialize response object into
					uint32 ObjSize = 0;			// Size of the response buffer
					boolean SpALL = FALSE;

					CDevicePMsgArguments_Init(&DevPMsgArgs);// Initialize the Argument Object
					CDeviceDExtStd_Init(&DeviceDExtStd);	// Initialize the Request Object
					CDeviceDStd_Init(&DeviceObj);			// Initialize the Response Object

					ObjSize = CDeviceDStd_SizeAll(&DeviceObj); // Get the size of the response object

					// If there is a Buffer included, that means there was an object included in the request message
					// Deserialize it into an object to find out what the host is asking for
					if(Buffer)
					{
						// All objects have their own deserialize method to call
						if (!CDevPMsgArguments_Deserialize(&DevPMsgArgs, Buffer)) 
							break;

						// If the host is requesting ALL, then we have some work to do
						if(CDEVDSTD_SpALL == DevPMsgArgs.m_SpeciesSelector)
							SpALL = TRUE;
						
						// Other Species Selectors can be checked here and handled accordingly (i.e. CDEVDSTD_SpSENSOR, CDEVDSTD_SpHOST, etc.)
					}

					// Set the Device Data
					memcpy(DeviceObj.m_ModelName, "New Sensor!", 11);
					memcpy(DeviceObj.m_ModelNumber, "123", 3);
					memcpy(DeviceObj.m_SerialNumber, "ABC123", 6);
					DeviceObj.m_DateTime.year = 2011;
					DeviceObj.m_DateTime.month = 7;
					DeviceObj.m_DateTime.day = 20;

					if(SpALL)
					{
						// The host is requesting SpALL
						// Init container to be filled with ALL CDeviceDStd objects (find out what objects to be created in the IDS)
						ErrorCode = CERR_PROC_CONTEXT; // set the error code, in case of failure
						if(!CDeviceDExtStd_InitDeviceContainer(&DeviceDExtStd, 1 * ObjSize)) // In this example we are inserting only 1 CDeviceDStd object
						{
							CDeviceDExtStd_Reset(&DeviceDExtStd, TRUE, TRUE);
							break;
						}

						// Allocate Buffer to serialize Device Data
						ErrorCode = CERR_PROC_MEMORY;
						if((ObjBuffer = (uint8*)SYS_REALLOC(ObjBuffer,ObjSize)) != NULL)
						{
							// Serialize Device Data
							ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
							if (!CDeviceDStd_Serialize(&DeviceObj,ObjBuffer))
							{
								CDeviceDExtStd_Reset(&DeviceDExtStd, TRUE, TRUE);
								SYS_FREE(ObjBuffer);
								break;
							}
							// Append Device Data
							ErrorCode = CERR_PROC_CONTEXT;
							if (!CDeviceDExtStd_AppendDeviceContainer(&DeviceDExtStd, ObjBuffer, ObjSize, 1))
							{
								CDeviceDExtStd_Reset(&DeviceDExtStd, TRUE, TRUE);
								SYS_FREE(ObjBuffer);
								break;
							}
							ErrorCode = OK;
						}

						/* Get object size */
						ObjSize = CDeviceDExtStd_SizeAll(&DeviceDExtStd);
	                    /* Allocate memory */
	                    if((ObjBuffer = (uint8*)SYS_REALLOC(ObjBuffer,ObjSize)) != NULL)
	                    {	
	                        /* Serialize object */
	                        if (!CDeviceDExtStd_Serialize(&DeviceDExtStd,ObjBuffer))
	                        {
	                            ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
	                        }
	                    }
	                    else
	                    {
	                        ErrorCode = CERR_PROC_MEMORY;
	                    }
						CDeviceDExtStd_Reset(&DeviceDExtStd, TRUE, TRUE);
						break;
					}
					else
					{
						// If the host didn't ask for SpALL then it asked for SpSENSOR or did not include an object, both requests will result in the same response
						CDeviceDStd_Serialize(&DeviceObj, ObjBuffer);  // Serialize the response object into a buffer

						ObjSize = CDeviceDStd_SizeAll(&DeviceObj);	// Retrieve the size of the response object
					}

					if(ErrorCode == OK)
					{
						/* Send the response to the request for device information */
						SendMsg(MakeClassID(FmDEVICE, GnRESPONSE, DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION), ObjBuffer, ObjSize, 0);

						// Mark that the message was handled properly (i.e. MsgResult = TRUE)
						MsgResult = TRUE;
					}
					else
					{
						// Fall through and send a NAK
					}

                    break;
                case DEVICE_CRST_SpGET_CONFIG:
                    break;
                case DEVICE_CRST_SpGET_POST:
                    break;
                case DEVICE_CRST_SpGET_DIGITAL_STATE:
                    break;
                case DEVICE_CRST_SpGET_DEVICE_MODE:
                    break;
                case DEVICE_CRST_SpGET_DEVICE_INVENTORY:
                    break;
                case DEVICE_CRST_SpGET_CALIBRATION:
                    break;
                case DEVICE_CRST_SpGET_DATE_TIME:
                    break;
                case DEVICE_CRST_SpGET_DEVICE_COMM_MODE:
                    break;
                case DEVICE_CRST_SpGET_HARDWARE_TEST_STATE:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION:
                    break;
                case DEVICE_CRSP_SpPUT_CONFIG:
                    break;
                case DEVICE_CRSP_SpPUT_POST:
                    break;
                case DEVICE_CRSP_SpPUT_DIGITAL_STATE:
                    break;
                case DEVICE_CRSP_SpPUT_DEVICE_MODE:
                    break;
                case DEVICE_CRSP_SpPUT_DEVICE_INVENTORY:
                    break;
                case DEVICE_CRSP_SpPUT_CALIBRATION:
                    break;
                case DEVICE_CRSP_SpPUT_DATE_TIME:
                    break;
                case DEVICE_CRSP_SpPUT_DEVICE_COMM_MODE:
                    break;
                case DEVICE_CRSP_SpPUT_HARDWARE_TEST_STATE:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnSTATUS:
        {
            switch(Species)
            {
                case DEVICE_CSS_SpREPORT_HARDWARE_TEST_STATE:
                    break;
                case DEVICE_CSS_SpREPORT_DOCK:
                    break;
                case DEVICE_CSS_SpREPORT_UNDOCK:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnDATA:
        {
            switch(Species)
            {
                case CDEVDSTD_SpSTANDARD:
                    break;
                case CDEVDSTD_SpHOST:
                    break;
                case CDEVDSTD_SpSENSOR:
                    break;
                case CDEVDSTD_SpALL:
                    break;
                case CDEVDPST_SpPOST:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

    /* Send a TRAP message if we have a unimplemented or unsupported error */
    if (!MsgResult)
    {
        SendTrap(CERR_SpERRMSGPROC, MsgClassID, ErrorCode);
    }

    return MsgResult;
}

