/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmDATATRANSFERStub.c                                             //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmDATATRANSFERStub.h"
#include "CDataTransferCDesc.h"
#include "CDataTransferDExtStd.h"
#include "CFileSysPSelector.h"
#include "DDS.h"

#define BDD_MAX_PACKET_SIZE 64
#define BDD_LENGTH DDS_BDD_SIZE
#define bdd_data DDS_BDD

static CDataTransferCDesc onGoingTransfer; //these are strategetically initialized, DO NOT initialize
static CDataTransferDExtStd DataTransferDExtStd; //these are strategetically initialized, DO NOT initialize
static uint8 datatransfer_state = 0;
static const ansichar BDDFileName[] = "\\Device.bdds";
#define BDD_FILENAME_SIZE 12

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
boolean CFmDATATRANSFERStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint32  MsgClassID = MakeClassID(FmDATATRANSFER, Genus, Species);
	uint8 ObjBuffer[MAX_PACKET_SIZE];
	uint32  ObjSize = 0;
	uint32 sequence_number;
	CFileSysPSelector FileSysPSelector;
	uint8 i;

    TOUCH(Buffer);

	CFileSysPSelector_Init(&FileSysPSelector);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case DATATRANSFER_CCMD_SpEXECUTE_DTT:
					/*	Data Transmission Terminate:
						ACK;
						reset datatransfer_state;
					*/
					SendTrap(CERR_SpERRNONE, MsgClassID, 0);
					ErrorCode = CERR_PROC_NONE;
					datatransfer_state = 0;
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case DATATRANSFER_CRST_SpGET_CONFIG:
                {

                	CDataTransferCDesc request;
                	CDataTransferCDesc_Init(&request);
                	CDataTransferCDesc_Init(&onGoingTransfer);

					if (!CDataTransferCDesc_Deserialize(&request, Buffer))
						break;
					if (!request.m_Role == CDTACDSC_Role_DATA_RECIPIENT)
					{
						ErrorCode = CERR_PROC_UNSUPPORTED;
						break;
					}

					if ((NULL != request.m_Descriptor) && (request.m_DescriptorSize > 0))
					{
						uint8* data;
						data = CDataTransferCDesc_GetDescriptorAt(&request, 0, &ObjSize);
						if (!CFileSysPSelector_Deserialize(&FileSysPSelector, data))
						{
							ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
							break;
						}
						ErrorCode = CERR_PROC_NONE;
						for (i = 0; i < BDD_FILENAME_SIZE; i++)
						{
							if (BDDFileName[i] != (FileSysPSelector.m_PathName[i] & 0xFF))
							{
								ErrorCode = CERR_PROC_INVL_PARAM;
								break;
							}
						}
						if (ErrorCode)
							break;
					}
					else
					{
						ErrorCode = CERR_PROC_UNSUPPORTED;
						break;
					}

					//initialize the response object
					onGoingTransfer.m_InitSeqNumber = 1;
					onGoingTransfer.m_TotalDataSize = BDD_LENGTH;
					if ((0 == request.m_DataPacketSize) ||
							(request.m_DataPacketSize > BDD_MAX_PACKET_SIZE))
						onGoingTransfer.m_DataPacketSize = BDD_MAX_PACKET_SIZE;
					else
						onGoingTransfer.m_DataPacketSize=request.m_DataPacketSize;
					request.m_DataPacketSize=onGoingTransfer.m_DataPacketSize;
					onGoingTransfer.m_Role = CDTACDSC_Role_DATA_SOURCE;
					onGoingTransfer.m_Identifier=request.m_Identifier;
					onGoingTransfer.m_WindowSize=1;

					//Initialize CFileSysPselector object
					widechar file[FileSysPSelector.m_PathNameSize];
					CFileSysPSelector selector;
					CFileSysPSelector_Init(&selector);
					selector.m_PathNameSize=FileSysPSelector.m_PathNameSize;
					memcpy(file,FileSysPSelector.m_PathName,FileSysPSelector.m_PathNameSize);
					selector.m_PathName=file;

					//add selector to response
					uint32 size = CFileSysPSelector_SizeAll(&selector);
					CFileSysPSelector_Serialize(&selector,ObjBuffer);
					CDataTransferCDesc_InitDescriptor(&onGoingTransfer,size);
					CDataTransferCDesc_AppendDescriptor(&onGoingTransfer,ObjBuffer, size, 1);

					//serialize response object
					uint8 outBuf[MAX_PACKET_SIZE];
					CDataTransferCDesc_Serialize(&onGoingTransfer,outBuf);
					ObjSize=CDataTransferCDesc_SizeAll(&onGoingTransfer);
					SendMsg(MakeClassID(FmDATATRANSFER,GnRESPONSE,DATATRANSFER_CRSP_SpPUT_CONFIG),outBuf,ObjSize,0);
					ErrorCode = CERR_PROC_NONE;
					datatransfer_state = 1; //set data transfer state
					break;
                }
                case DATATRANSFER_CRST_SpGET_DATA:
                {
					/*	Get Data:
						Verify that DTR and Get config has happened (datatransfer_state = 2);
						Deserialize;
						Validate data;
						Verify Indentifier;
						Validate sequence number;
						Init Object;
						Set Sequence, Idenifier, Data Size and Data pointer;
						Send Response;
					*/
					if (datatransfer_state != 2)
					{
						ErrorCode = CERR_PROC_CONTEXT;
						break;
					}
					CDataTransferDExtStd_Reset(&DataTransferDExtStd,TRUE,TRUE);
					if (CDataTransferDExtStd_Deserialize(&DataTransferDExtStd, Buffer))
					{
						if (DataTransferDExtStd.m_Identifier != onGoingTransfer.m_Identifier)
						{
							ErrorCode = CERR_PROC_CONTEXT;
							break;
						}
						if (0 == DataTransferDExtStd.m_SeqNumber || DataTransferDExtStd.m_SeqNumber > BDD_LENGTH)
						{
							ErrorCode = CERR_PROC_CONTEXT;
							//CDataDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
							break;
						}
						sequence_number = DataTransferDExtStd.m_SeqNumber;
						CDataTransferDExtStd_Reset(&DataTransferDExtStd, TRUE, TRUE);
						DataTransferDExtStd.m_SeqNumber = sequence_number;
						DataTransferDExtStd.m_Identifier = onGoingTransfer.m_Identifier;
						DataTransferDExtStd.m_DataSize = (sequence_number + onGoingTransfer.m_DataPacketSize) > BDD_LENGTH ?
												BDD_LENGTH - sequence_number + 1 :
												onGoingTransfer.m_DataPacketSize;
						DataTransferDExtStd.m_Data = (uint8*)&(bdd_data[sequence_number - 1]);
						/* Get object size */
						ObjSize = CDataTransferDExtStd_SizeAll(&DataTransferDExtStd);

						/* Serialize object */
						if (CDataTransferDExtStd_Serialize(&DataTransferDExtStd,ObjBuffer))
						{
							/* Send serialized object */
							SendMsg(MakeClassID(FmDATATRANSFER,
									GnRESPONSE,
									DATATRANSFER_CRSP_SpPUT_DATA),
									ObjBuffer,									// Serialized object
									ObjSize,									// Size of Serialized object
									0);									     	// ErrorClsID: 0
							ErrorCode = CERR_PROC_NONE;
						}
						else
						{
							ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
						}
						SYS_FREE(ObjBuffer);
					}
					break;
                }
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case DATATRANSFER_CRSP_SpPUT_CONFIG:
                    break;
                case DATATRANSFER_CRSP_SpPUT_DATA:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnSTATUS:
        {
            switch(Species)
            {
                case DATATRANSFER_CSS_SpREPORT_DTR:
					/* 	Data Transmission Ready:
						Verify that Get config has happened (datatransfer_state = 1);
						Deserialize;
						Validate data;
						Verify that Transmission Identifier matches;
						ACK;
					*/
					if (datatransfer_state != 1)
					{
						ErrorCode = CERR_PROC_CONTEXT;
						break;
					}
					CDataTransferDExtStd_Reset(&DataTransferDExtStd,TRUE,TRUE);
					if (CDataTransferDExtStd_Deserialize(&DataTransferDExtStd, Buffer))
					{
						if (DataTransferDExtStd.m_Identifier != onGoingTransfer.m_Identifier)
						{
							ErrorCode = CERR_PROC_CONTEXT;
							break;
						}
						SendTrap(CERR_SpERRNONE, MsgClassID, 0);
						ErrorCode = CERR_PROC_NONE;
						datatransfer_state = 2;
					}
                    break;
                case DATATRANSFER_CSS_SpREPORT_DTC:
					/* 	Data Transmission Complete:
						ACK;
						reset datatransfer_state;
					*/
					SendTrap(CERR_SpERRNONE, MsgClassID, 0);
					ErrorCode = CERR_PROC_NONE;
					datatransfer_state = 0;
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

     /* Any Errors? */
    if (ErrorCode == CERR_PROC_NONE)
    {
        MsgResult = TRUE;
    }

   /* Send a TRAP message if we have a unimplemented or unsupported error */
	if (!MsgResult)
	{
		SendTrap(CERR_SpERRMSGPROC, MsgClassID, ErrorCode);
	}

    return MsgResult;
}
