/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CCommSvr.h     :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined (CCOMMSVR_H)
    #define   CCOMMSVR_H

#include "Common.h"
#include "CWACPThread.h"

#define CTL_W 0x17
#define CTL_A 0x01
#define CTL_C 0x03
#define CTL_L 0x0C

#define PREAMBLE_SIZE (3)

#define MSG_HEADER (sizeof(uint8) * PREAMBLE_SIZE + sizeof(uint32))

const uint8 WACP_PREAMBLE[PREAMBLE_SIZE] = {CTL_W, CTL_A, CTL_L}; //Control codes for WACP header

typedef boolean (*SetRecvBufPtr)(char*, char*, uint8*, uint32);
typedef char* (*DevArrPtr)(char*, char*, ConnArgs*);
typedef boolean (*DevRemPtr)(char*, char*);

uint8* GeneratePacket(uint32 *OutSize, 
                      uint16 Port, 
                      int16 SequenceNum, 
                      int32 UUID, 
                      uint8* pBuf, 
                      uint32 Size);


class CCommSvr
{
    private:
        SetRecvBufPtr   pRecvMsg;
        DevArrPtr       pDeviceArrival;
        DevRemPtr       pDeviceRemoval;
        CWACPThread     RecvThread;
        char*           pSemName;


    public:
        CCommSvr(SetRecvBufPtr RecvMsgFunc, 
                 DevArrPtr DeviceArrivalFunc, 
                 DevRemPtr DeviceRemovalFunc, 
                 char* pAppName);
        CCommSvr();
        ~CCommSvr();

        /* Returns the SourceID of the CCommSvr. The SourceID is the unique
         * identifier for the communications session. In Ethernet the SourceID 
         * would hold the destination IP Address of the connection peer. In 
         * RS232 this would hold the Name of the com Port through which */
        char* GetSourceID(ConnArgs* pConnArgs);

        /* Returns the Port which represents either a RNDZ or WACP session */
        uint16 GetPortNum(ConnArgs* pConnArgs);

        /* Sets the Port which represents either a RNDZ or WACP session */
        boolean SetPortNum(ConnArgs* pConnArgs);

        /* Sends a binary Buffer */
        boolean SendMsg(ConnArgs* pConnArgs, uint8* pBuf, uint32 MsgSize);
        
        /* Creates a connection */
        boolean Connect(ConnArgs* pConnArgs);

        /* Disconnects a connection */
        boolean Disconnect(ConnArgs* pConnArgs);

        /* Starts the listener thread to wait and accept new connections from 
         * client 
         */
        boolean StartServer(ConnArgs* *pConnList, uint8 NumConnections);
};

#endif /* #if !defined (CCOMMSVR_H) */


/* End of File */





