/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CConDStd.h                                                        //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'*-E************************************************************************/

#if !defined(CCONDSTD_H)
    #define  CCONDSTD_H

/*Windows CE doesn't support streams through standard libraries, so
  streams are no longer used.
  #include <stdiostr.h>*/

#include "DtData.h"

#if defined(_WIN32_WCE)
#include "windows.h"
#endif

/*The following definition is the version.*/
#define CCNDSTD_OBJECT_VERSION 100

/*---------------------------------
Add Defines for string representation
of enumerations in the next section.
---------------------------------*/
    typedef  enum CCNDSTDtypSPECIES {
        CCNDSTD_SpSTANDARD = 0, 
        CCNDSTD_SpMAX
    } CCNDSTD_SPECIES;

#define CCNDSTD_SPECIES_STRING \
    (GetMsgString((MsgEntry*)TypSpeciesTbl, \
                 CCNDSTD_TYPSPECIESTBL_SIZE, \
                 species()))

    /*CConDStd Member's Version Enumerations*/
    typedef  enum CCNDSTDtypVERSIONS {
        CCNDSTD_DeviceGUID,
        CCNDSTD_SourceID,
        CCNDSTD_Reserved,
        CCNDSTD_MEMBER_MAX
    } CCNDSTD_VERSIONS;

#define CCNDSTD_VERSION_STATUS(MemberEnum) \
    (GetVersionStatus((VersionEntry*)TypVersionsTbl, \
                      CCNDSTD_MEMBER_MAX, \
                      MemberEnum))

/*---------------------------------
End...
---------------------------------*/
#ifndef CCONDSTD_DLLAPI
    #if defined(_MOIB_EXPORT_DLLAPI_)
        #define CCONDSTD_DLLAPI  __declspec(dllexport)
    #elif defined(_MOIB_IMPORT_DLLAPI_)
        #define CCONDSTD_DLLAPI  __declspec(dllimport)
    #else
        #define CCONDSTD_DLLAPI
    #endif
#endif

class CCONDSTD_DLLAPI CConDStd : public DtData
{
public:
    CConDStd();
    ~CConDStd();

    /*---------------------------------
    Functions:
    Add function prototypes here
    ---------------------------------*/
    uint8* GetDeviceGUID(uint16* pSize);
    void SetDeviceGUID(uint8* pBuf, uint16 Length);

    ansichar* GetSourceID(uint16* pSize);
    void SetSourceID(ansichar* pBuf, uint16 NumElements);

    uint8* GetReserved(uint16* pSize);
    void SetReserved(uint8* pBuf, uint16 NumElements);

    void Reset(boolean bResetAll);

    void SetObjectProperties(CConDStd *MyObjectSrc);
    /*--------------------------------*/

    /*---------------------------------
       Inherited/Overidden functions:
       Functions derived from DtData
    ---------------------------------*/
    uint16 getVersion (void);

#if((TARGET_XML_SUPPORT == XML_SERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
    uint32 serialize_to_XMLbuf(char* pBuf);
#endif
    /*---------------------------------*/

/*---------------------------------------------------
 Use the following struct for classes that will hold
     internal data.
---------------------------------------------------*/
    typedef struct ClassStruct_s{
        /*Static Data*/
            uint8         m_DeviceGUID[16]; /* Contains the GUID of the Connection */
    }ClassStruct_t;

#define CCNDSTD_STATIC_STRUCT_SIZE\
               (0 * MOIB_TIME_SIZE        + 0 *(sizeof(int8))\
                   + 0 *(sizeof(int16))   + 0 *(sizeof(int32))\
                   + 16 *(sizeof(uint8))   + 0 *(sizeof(uint16))\
                   + 0 *(sizeof(uint32))  + 0 *(sizeof(byte))\
                   + 0 *(sizeof(word))    + 0 *(sizeof(boolean))\
                   + 0 *(sizeof(char))    + 0 *(sizeof(ansichar))\
                   + 0 *(sizeof(widechar))+ 0 *(sizeof(float))\
                   + 0 *(sizeof(double)))


    /*Dynamic Data*/
    ansichar          *m_SourceID; /* Contains the SourceID of the Connection */
    uint16            m_SourceIDSize;
    uint8             *m_Reserved; /* Reserved for future expansion. */
    uint16            m_ReservedSize;
protected:

    /*---------------------------------
    Enumeration related tables.
    ---------------------------------*/
    static const MsgEntry     TypSpeciesTbl[];
    static const VersionEntry TypVersionsTbl[];
/*---------------------------------------------------*/
    /*---------------------------------
     Inherited/Overidden functions:
     Functions derived from DtData
    ---------------------------------*/
    uint32 get_payload_size(void);
    boolean serialize_payload(uint8QManager * QMgr);
    boolean deserialize_payload(uint8QManager * QMgr, uint32 Sz);

private:

#if((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
    void    parse_StartElement(void *UserData, const XML_Char *Name, const XML_Char **Atts);
    void    parse_CharacterData(void *UserData, const XML_Char *sTag, int Len);
#endif

    void deserialize_static_data(uint8QManager *QMgr, uint16 StructSize);
    void serialize_static_data(uint8QManager * QMgr);

    /*---------------------------------*/

    /*--------------------------------------------------
     The following data members relate to the struct
     defined above - if used
    --------------------------------------------------*/

    ClassStruct_t        m_structData;
};
#endif /* #if !defined(CCONDSTD_H) */

/*---------------------------------------------------*/
/* End of File */
