/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CErrSvr.cpp    :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CErrSvr.h"

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
*********************************************************************************/
CErrSvr::CErrSvr() : DtData(MakeClassID(FmTRAP, GnDATA, CES_SpNONE))
{
    /* Intialize the structure */
    memset(&m_structData, 0, sizeof(m_structData));
    Data = NULL;
    DataSize = 0;
}
/*********************************************************************************
  Default destructor
*********************************************************************************/
CErrSvr::~CErrSvr()
{
    memset(&m_structData, 0, sizeof(m_structData));
    
    SYS_FREE(Data);
    Data = NULL;
}

/*---------------------------------
 Static Members
---------------------------------*/

void CErrSvr::SetErrorClassID(uint32 NewID)
{
    m_structData.ErrorClassID = NewID;
}

uint32 CErrSvr::GetErrorClassID()
{
    return (m_structData.ErrorClassID);
}

/*********************************************************************************

---------------------------------
 Dynamic Members
---------------------------------
*********************************************************************************
 Set function for dynamic data member Data
*********************************************************************************/
void CErrSvr::SetData(uint8* NewData, uint32 Size)
{
    /*Allocate memory for the new Buffer*/
    if (NewData != NULL)
    {
        Data = (uint8*)SYS_REALLOC(Data, Size);

        if (Data != NULL)
        {
            /*Make a copy of the data*/
            memcpy(Data, NewData, Size);

            /* Set the DataSize Var */
            DataSize = Size;

        }
    }
    else
    {
        SYS_FREE(Data);
        Data = NULL;
    }

}
/*********************************************************************************
 Get function for dynamic data member Data
*********************************************************************************/
uint8* CErrSvr::GetData(uint32* pSize)
{
    if (pSize)
    {
        *pSize = DataSize;
    }
    return (uint8*)Data;
}

/*================================================================================
    INHERITED / OVERIDDEN
================================================================================*/
/*********************************************************************************
Helper Function to return TRUE if a serialized object has a uint32 size identifier
*********************************************************************************/
boolean CErrSvr::is_Extended(uint8* pBuffer)
{   
    TOUCH(pBuffer);

    return FALSE;
}
/*********************************************************************************
Helper Function to return TRUE if a serialized object has a uint32 size identifier
*********************************************************************************/
boolean CErrSvr::is_Extended(uint32 ClassID)
{
    TOUCH(ClassID);

    return FALSE;
}
/*********************************************************************************
Returns the number of Bytes that would be used by the serialize_payload and
deserialize_payload methods.
*********************************************************************************/
uint32 CErrSvr::get_payload_size(void)
{   
    uint32 Size = 0;
    Size = sizeof(uint16)                   /*Size of the struct*/
            + (uint16)CES_STATIC_STRUCT_SIZE;   /*struct data Size*/

    /* Get the Size of each dynamic member */
    if (Data != NULL)
    {
        Size += sizeof(uint32) + DataSize;
    }
    else
    {
        Size += sizeof(uint32);
    }

    Size += sizeof(uint16);             /* CRC Bytes */

    return Size;            
}

/*********************************************************************************
Serialize the static data defined by m_structData.
*********************************************************************************/
void CErrSvr::serialize_static_data(uint8QManager* QMgr)
{
    /* Enqueue only the  members of the Struct.*/
    QMgr->put(m_structData.ErrorClassID);
}

/*********************************************************************************
Deserialize the static data defined by m_structData.
*********************************************************************************/
void CErrSvr::deserialize_static_data(uint8QManager* QMgr)
{
    /* Dequeue only the  members of the Struct.*/
    QMgr->get(&m_structData.ErrorClassID);
}


/*********************************************************************************
Used to place the data elements of the object into a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CErrSvr::serialize_payload(uint8QManager* QMgr)
{
    uint32      CRCLen;

    /* Save the start of the byte stream for calculating the CRC */
    byte* TailPtr = (byte*)QMgr->GetTailPtr();


    /* Enqueue the Static Structure Size */
    QMgr->put((uint16)CES_STATIC_STRUCT_SIZE);

    /* Enqueue only the static members of the Struct */
    serialize_static_data(QMgr);

    /* Enqueue variable sized members and there sizes */
    if (Data != NULL)
    {
        QMgr->put(DataSize);
        QMgr->enqueue_array((byte*)Data, DataSize);
    }
    else
    {
        QMgr->put((uint32)(0) );
    }
    /* Enqueue the CRC */
    CRCLen = QMgr->get_num_elements() - sizeof(uint16);
    QMgr->put(uint16( CalculateCRC(TailPtr,CRCLen)));

    return TRUE; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CErrSvr::deserialize_payload(uint8QManager* QMgr, uint32 Sz)
{
    //These data pieces will need to be dequeued
    
    uint16      Size; 
    uint32      CRCSize;
    uint16      CalcCRC;
    uint16      StoredCRC;
        
    /* Save the start of the byte stream for calculating the CRC */
    byte* HeadPtr = (byte*)QMgr->GetHeadPtr();

    // Calculate the CRC on all but the 2 Bytes of the CRC
    CRCSize = QMgr->get_num_elements() - sizeof(uint16);
    CalcCRC = CalculateCRC(HeadPtr, CRCSize);

    /* Dequeue the static member struct Size */
    QMgr->get(&Size);

    /* Dequeue only the static members of the Struct */
    if ( Size <= (uint16)CES_STATIC_STRUCT_SIZE )
    {   
        deserialize_static_data(QMgr);
        #if defined(DEBUG_PRINT)
        if ( Size < (uint16)CES_STATIC_STRUCT_SIZE )
        {   SYS_PRINTF("\nThe source object is smaller than the destination. Some source data will be lost.\n\n");}
        #endif
    }
    else
    { 
        uint8   *Buffer;

        Buffer = (uint8*)SYS_MALLOC(Sz); 
        if (Buffer != NULL)
        {
            deserialize_static_data(QMgr);
#if defined(DEBUG_PRINT)
            SYS_PRINTF("\nThe Source object is larger than the destination. Some destination data will not be intialized.\n\n");
#endif
            /* Dequeue the remaining extra Bytes of the static struct */
            QMgr->dequeue_array( Buffer, Size - CES_STATIC_STRUCT_SIZE);

            /* TODO: Create a mechanism for the object to store this unknown data  */
            /*       to prevent data loss and to create a way to forward this data */
            /*       to a host/device that supports the correct version.           */

            SYS_FREE(Buffer);
        }   
    }

    /* Dequeue the variable data size*/
    QMgr->get(&DataSize);

    if (DataSize > 0)
    {
        Data = (uint8*)SYS_REALLOC(Data, DataSize);

        if (Data != NULL)
        {
            QMgr->dequeue_array((byte*)Data, DataSize); 
        }
    }
    else
    {
        SYS_FREE(Data);
        Data = NULL;
    }
    
    /* Dequeue the CRC */
    QMgr->get(&StoredCRC);
    if ( CalcCRC != StoredCRC)
    {
        #if defined(DEBUG_PRINT)
        SYS_PRINTF("CRC Mismatch Error: Calculated=0x%d, Stored=0x%d ", CalcCRC, StoredCRC);
        #endif
        return FALSE;
    }

    return TRUE; 
}


/* End of File */





