/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWACPSemaphore.cpp:                                               //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CWACPSemaphore.h"
#include "Common.h"

/*********************************************************************************
  Default constructor
*********************************************************************************/
CWACPSemaphore::CWACPSemaphore()
{
    hHandle = CreateSemaphore(NULL,0,1,NULL);
}

/*********************************************************************************
CWACPSemaphore::CWACPSemaphore - Constructor creates a new counting Semaphore 
    Handle with the name pointed to by pName.

Parameters:
    pName - Pointer to string that contains the name of the Semaphore used to open 
            the same Semaphore handle across threads. This can be NULL for
            non named Semaphores.

    InitCount - uint16 value for initial count of the semaphore. 

    MaxCount - uint16 value for the max count of the semaphore.

*********************************************************************************/
CWACPSemaphore::CWACPSemaphore(TCHAR* pName, uint16 InitCount, uint16 MaxCount)
{
    hHandle = CreateSemaphore(NULL,InitCount,MaxCount,pName);
}

/*********************************************************************************
Default Destructor. Closes and resets the Semaphore handle.
*********************************************************************************/
CWACPSemaphore::~CWACPSemaphore()
{
    if(hHandle)
    {
        CloseHandle(hHandle);
        hHandle = NULL;
    }
}

/*********************************************************************************
CWACPSemaphore::Raise - Raises the Semaphore.
*********************************************************************************/
void CWACPSemaphore::Raise()
{
    if (hHandle)
    {
        /* Only increment the semaphore count by 1 */
        ReleaseSemaphore(hHandle, 1, NULL);
    }
}

/*********************************************************************************
CWACPSemaphore::Wait - Waits for a semaphore to be released.

Parameters:
    Timeout - Amount of time in milliseconds to wait before returning. This can
              be set to INFINITE, which will stay blocked until the Semaphore is
              raised

Returns: 
    WAIT_SUCCESS when the Semaphore is raised
    WAIT_FAILURE, if the timeout occurs or the handle is invalid.
*********************************************************************************/
uint32 CWACPSemaphore::Wait(uint32 Timeout)
{
    uint32 Result = WAIT_FAILURE;

    if (hHandle)
    {
        Result = WaitForSingleObject(hHandle, Timeout);
        if (Result != WAIT_SUCCESS)
            Result = WAIT_FAILURE;
    }
    return(Result);
}


