/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWACPThread.cpp:                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CWACPThread.h"

typedef unsigned (WINAPI* WACP_THREAD_PROC)(
    LPVOID lpThreadParameter
);

/*********************************************************************************
  Default constructor
*********************************************************************************/
CWACPThread::CWACPThread()
{
    hHandle = NULL;
}

/*********************************************************************************
Default Destructor. 
*********************************************************************************/
CWACPThread::~CWACPThread()
{
    hHandle = NULL;
}

/*********************************************************************************
CWACPThread::StartThread - Starts the thread and sets the Thread Priority.

Parameters:
    StartAddress - Address of the thread process function. WACPAPI is specified
                   in Target.h

    ArgList - Pointer to data that is to be passed into the thread process 
              function as a parameter. 

    ThreadPriority - Priority value of the thread based on OS specific settings.
                     For Win32 THREAD_PRIORITY_NORMAL is 0 with higher priority
                     settings having positive values and lower priority having
                     negative values. 
    pName - Pointer to the thread name. Primarily used for identifying threads 
            for debug. 

Returns: 
    TRUE when the Thread is created and priority is set
    FALSE, if the an error occurred in creating the thread or setting the 
        priority. 
*********************************************************************************/
boolean CWACPThread::StartThread(uint32 (WACPAPI* StartAddress)(void*), void* ArgList, 
                                 int32 ThreadPriority, char* pName)
{
    boolean Result = FALSE;

    TOUCH(ThreadPriority);
    TOUCH(pName);

    hHandle = (HANDLE)_beginthreadex(NULL, 0, (WACP_THREAD_PROC)StartAddress, ArgList, 0, NULL);
    if (hHandle)
    {
        Result = (boolean)SetThreadPriority(hHandle, ThreadPriority);
        CloseHandle(hHandle);
        hHandle = NULL;
    }

    return (Result);
}

/* End of File */






