/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFamilyStub.cpp                                                   //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include <TCHAR.h>
#include "CWACPEvent.h"
#include "CWACPApp.h"
#include "CFamilyStub.h"



CFamilyStub::CFamilyStub()
{
    pCBMIDStd = new CBMIDStd();
    pCDeviceDStd = new CDeviceDStd();
    pCDevDPOST = new CDevDPOST();
    pCHeightDStd = new CHeightDStd();
    pCModuleDStd = new CModuleDStd();
    pCNumDStd = new CNumDStd();
    pCRecordDService = new CRecordDService();
    pCWeightDStd = new CWeightDStd();

    LoadObjects();

    ConnCount = 0;
    ErrCount  = 0;
    AckCount  = 0;
    MaxCount  = 0;
    NumConnections = 0;
    bConnected = FALSE;
}

CFamilyStub::~CFamilyStub()
{
    /* Clean up */
    for (uint16 i=0; i < ConnCount != NULL; i++)
    {
        if (ConnDArray[i])
        {
            delete(ConnDArray[i]);
        }
    }
    delete pCBMIDStd;
    delete pCDeviceDStd;
    delete pCDevDPOST;
    delete pCHeightDStd;
    delete pCModuleDStd;
    delete pCNumDStd;
    delete pCRecordDService;
    delete pCWeightDStd;
}

/*================================================================================
LoadObjects :
================================================================================*/
void CFamilyStub::LoadObjects()
{
}

/*================================================================================
AppInit : Virtual Function called by CMOMPBaseClass::AppInitInt
================================================================================*/
CMOMPBaseClass* CFamilyStub::AppInit()
{
    return (CMOMPBaseClass*)this;
}

/*================================================================================
AppEnd : Virtual Function called by CMOMPBaseClass::AppEndInt
================================================================================*/
uint32 CFamilyStub::AppEnd()
{
    return(0);
}

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
uint32 CFamilyStub::RecvMessage(char *pSourceID, uint32 ClassID, uint8 *pBuffer)
{
    uint32  Result     = 0;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint16  Family     = GetFamily(ClassID);
    uint8   Genus      = GetGenus(ClassID);
    uint8   Species    = GetSpecies(ClassID);
    uint32  MsgResult  = 0;

    TOUCH(pSourceID);
    TOUCH(pBuffer);

	SYS_PRINTF("CFamilyStub::RecvMessage() F: %d, G:%d, S:%d\n", Family, Genus, Species);

    switch(Family)
    {
        case FmBODYMASSINDEX:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case BODYMASSINDEX_CCMD_SpWRITE_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case BODYMASSINDEX_CRST_SpGET_BMI:
                            break;
                        case BODYMASSINDEX_CRST_SpGET_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case BODYMASSINDEX_CRSP_SpPUT_BMI:
                            break;
                        case BODYMASSINDEX_CRSP_SpPUT_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case BODYMASSINDEX_CSS_SpREPORT_BMI:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CBMIDSTD_SpSTANDARD:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmDATATRANSFER:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case DATATRANSFER_CCMD_SpEXECUTE_DTT:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case DATATRANSFER_CRST_SpGET_CONFIG:
                            break;
                        case DATATRANSFER_CRST_SpGET_DATA:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case DATATRANSFER_CRSP_SpPUT_CONFIG:
                            break;
                        case DATATRANSFER_CRSP_SpPUT_DATA:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case DATATRANSFER_CSS_SpREPORT_DTR:
                            break;
                        case DATATRANSFER_CSS_SpREPORT_DTC:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmDEVICE:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case DEVICE_CCMD_SpWRITE_CONFIG:
                            break;
                        case DEVICE_CCMD_SpWRITE_DEVICEDESCRIPTION:
                            break;
                        case DEVICE_CCMD_SpENABLE_DISPLAY:
                            break;
                        case DEVICE_CCMD_SpSTART_DISPLAY_TEST:
                            break;
                        case DEVICE_CCMD_SpRESET:
                            break;
                        case DEVICE_CCMD_SpPOWER:
                            break;
                        case DEVICE_CCMD_SpBEEP:
                            break;
                        case DEVICE_CCMD_SpSAVE_CONFIG:
                            break;
                        case DEVICE_CCMD_SpRESTORE_CONFIG:
                            break;
                        case DEVICE_CCMD_SpENABLE_AUTO_POWER_DOWN:
                            break;
                        case DEVICE_CCMD_SpSET_DISPLAY_CONTRAST:
                            break;
                        case DEVICE_CCMD_SpSET_BACKLIGHT:
                            break;
                        case DEVICE_CCMD_SpSTART_DIGITAL_TEST:
                            break;
                        case DEVICE_CCMD_SpSTOP_DISPLAY_TEST:
                            break;
                        case DEVICE_CCMD_SpSET_DEVICE_MODE:
                            break;
                        case DEVICE_CCMD_SpSTART_PRINTER_TEST:
                            break;
                        case DEVICE_CCMD_SpSET_CALIBRATION:
                            break;
                        case DEVICE_CCMD_SpSET_DATE_TIME:
                            break;
                        case DEVICE_CCMD_SpNEXT_PATIENT:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case DEVICE_CRST_SpGET_DEVICEDESCRIPTION:
						{
							CDeviceDStd DeviceObj;
							uint8 ObjBuffer[128];

							DeviceObj.SetModelName("WACP_PMP_TOOL", strlen("WACP_PMP_TOOL"));

							DeviceObj.serialize_all(ObjBuffer);

							SendMsg(pSourceID, FmDEVICE, GnRESPONSE, DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION, ObjBuffer, DeviceObj.size_all());

							MsgResult = TRUE;
                            break;
						}
                        case DEVICE_CRST_SpGET_CONFIG:
                            break;
                        case DEVICE_CRST_SpGET_POST:
                            break;
                        case DEVICE_CRST_SpGET_DIGITAL_STATE:
                            break;
                        case DEVICE_CRST_SpGET_DEVICE_MODE:
                            break;
                        case DEVICE_CRST_SpGET_DEVICE_INVENTORY:
                            break;
                        case DEVICE_CRST_SpGET_CALIBRATION:
                            break;
                        case DEVICE_CRST_SpGET_DATE_TIME:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION:
                            break;
                        case DEVICE_CRSP_SpPUT_CONFIG:
                            break;
                        case DEVICE_CRSP_SpPUT_POST:
                            break;
                        case DEVICE_CRSP_SpPUT_DIGITAL_STATE:
                            break;
                        case DEVICE_CRSP_SpPUT_DEVICE_MODE:
                            break;
                        case DEVICE_CRSP_SpPUT_DEVICE_INVENTORY:
                            break;
                        case DEVICE_CRSP_SpPUT_CALIBRATION:
                            break;
                        case DEVICE_CRSP_SpPUT_DATE_TIME:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CDEVDSTD_SpSTANDARD:
                            break;
                        case CDEVDSTD_SpHOST:
                            break;
                        case CDEVDSTD_SpSENSOR:
                            break;
                        case CDEVDSTD_SpALL:
                            break;
                        case CDEVDPST_SpPOST:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmERROR:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case ERROR_CCMD_SpCLEAR_ERROR:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case ERROR_CRST_SpGET_ERROR:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case ERROR_CRSP_SpPUT_ERROR:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case ERROR_CSS_SpREPORT_ERROR:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmFILESYS:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case FILESYS_CCMD_SpWRITE_FILE:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case FILESYS_CRST_SpGET_FILE:
                            break;
                        case FILESYS_CRST_SpGET_FILE_COUNT:
                            break;
                        case FILESYS_CRST_SpGET_FILE_LIST:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case FILESYS_CRSP_SpPUT_FILE:
                            break;
                        case FILESYS_CRSP_SpPUT_FILE_COUNT:
                            break;
                        case FILESYS_CRSP_SpPUT_FILE_LIST:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmFLASH:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case FLASH_CCMD_SpWRITE_SEGMENT:
                            break;
                        case FLASH_CCMD_SpERASE_SECTOR:
                            break;
                        case FLASH_CCMD_SpENABLE_FLASH:
                            break;
                        case FLASH_CCMD_SpDISABLE_FLASH:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case FLASH_CRST_SpGET_CRC:
                            break;
                        case FLASH_CRST_SpGET_SEGMENT:
                            break;
                        case FLASH_CRST_SpGT_FLASH_CONFIG:
                            break;
                        case FLASH_CRST_SpGET_SECTOR_CONFIG:
                            break;
                        case FLASH_CRST_SpGET_UPGRADE:
                            break;
                        case FLASH_CRST_SpGET_FLASH_PREPARATION:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case FLASH_CRSP_SpPUT_CRC:
                            break;
                        case FLASH_CRSP_SpPUT_SEGMENT:
                            break;
                        case FLASH_CRSP_SpPUT_FLASH_CONFIG:
                            break;
                        case FLASH_CRSP_SpPUT_SECTOR_CONFIG:
                            break;
                        case FLASH_CRSP_SpPUT_UPGRADE:
                            break;
                        case FLASH_CRSP_SpPUT_FLASH_PREPARATION:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case FLASH_CSS_SpFLASH_READY:
                            break;
                        case FLASH_CSS_SpFLASH_COMPLETE:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmHEIGHT:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case HEIGHT_CCMD_SpWRITE_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case HEIGHT_CRST_SpGET_HEIGHT:
                            break;
                        case HEIGHT_CRST_SpGET_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case HEIGHT_CRSP_SpPUT_HEIGHT:
                            break;
                        case HEIGHT_CRSP_SpPUT_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case HEIGHT_CSS_SpREPORT_HEIGHT:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CHGTDSTD_SpSTANDARD:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmMODULE:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case MODULE_CCMD_SpWRITE_CONFIG:
                            break;
                        case MODULE_CCMD_SpWRITE_MODULE_DESC:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case MODULE_CRST_SpGET_MODULE_DESC:
                            break;
                        case MODULE_CRST_SpGET_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case MODULE_CRSP_SpPUT_MODULE_DESC:
                            break;
                        case MODULE_CRSP_SpPUT_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnEVENT:
                {
                    switch(Species)
                    {
                        case MODULE_CEVNT_SpHW_MOD_CONNECT:
                            break;
                        case MODULE_CEVNT_SpHW_MOD_DISCONNECT:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CMODDSTD_SpSTANDARD:
                            break;
                        case CMODDSTD_SpHOST:
                            break;
                        case CMODDSTD_SpHARDWARE:
                            break;
                        case CMODDSTD_SpBOOTLOADER:
                            break;
                        case CMODDSTD_SpBOOTRECOVERY:
                            break;
                        case CMODDSTD_SpINITBOOTLOADER:
                            break;
                        case CMODDSTD_SpOPERATINGSYSTEM:
                            break;
                        case CMODDSTD_SpFONT:
                            break;
                        case CMODDSTD_SpAUXILIARYHOST:
                            break;
                        case CMODDSTD_SpAUXILIARYHARDWARE:
                            break;
                        case CMODDSTD_SpAUXILIARYBOOTLOADER:
                            break;
                        case CMODDSTD_SpAUXILIARYOPERATINGSYSTEM:
                            break;
                        case CMODDSTD_SpNIBPMODULE:
                            break;
                        case CMODDSTD_SpNIBPSAFETYALG:
                            break;
                        case CMODDSTD_SpNIBPSTEPALG:
                            break;
                        case CMODDSTD_SpNIBPFASTALG:
                            break;
                        case CMODDSTD_SpNIBPNEONATSTEPALG:
                            break;
                        case CMODDSTD_SpNIBPPOEMALG:
                            break;
                        case CMODDSTD_SpNIBPSTEPAOALG:
                            break;
                        case CMODDSTD_SpTEMPMODULE:
                            break;
                        case CMODDSTD_SpSURETEMPALG:
                            break;
                        case CMODDSTD_SpSPO2MODULE:
                            break;
                        case CMODDSTD_SpECGMODULE:
                            break;
                        case CMODDSTD_SpMEANS:
                            break;
                        case CMODDSTD_SpSPIROMETRY:
                            break;
                        case CMODDSTD_SpCO2MODULE:
                            break;
                        case CMODDSTD_SpFRONTEND:
                            break;
                        case CMODDSTD_SpPPP:
                            break;
                        case CMODDSTD_SpSECUREDIGITALCARD:
                            break;
                        case CMODDSTD_SpPRINTER:
                            break;
                        case CMODDSTD_SpCOMMSMODULE:
                            break;
                        case CMODDSTD_SpRADIO:
                            break;
                        case CMODDSTD_SpBARCODE:
                            break;
                        case CMODDSTD_SpSCALE:
                            break;
                        case CMODDSTD_SpBATTERY:
                            break;
                        case CMODDSTD_SpALL:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmNUMERIC:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case NUMERIC_CCMD_SpCMD_START_ACQUIRE:
                            break;
                        case NUMERIC_CCMD_SpCMD_STOP_ACQUIRE:
                            break;
                        case NUMERIC_CCMD_SpWRITE_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTREAM:
                {
                    switch(Species)
                    {
                        case NUMERIC_CSTR_SpSTREAM_NUMERIC:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CNUMDSTD_SpSTANDARD:
                            break;
                        case CNUMDSTD_SpECGHR:
                            break;
                        case CNUMDSTD_SpHRPR:
                            break;
                        case CNUMDSTD_SpNIBPSYS:
                            break;
                        case CNUMDSTD_SpNIBPDIA:
                            break;
                        case CNUMDSTD_SpNIBPMAP:
                            break;
                        case CNUMDSTD_SpNIBPPR:
                            break;
                        case CNUMDSTD_SpIBP1SYS:
                            break;
                        case CNUMDSTD_SpIBP1DIA:
                            break;
                        case CNUMDSTD_SpIBP1MAP:
                            break;
                        case CNUMDSTD_SpIBP1PR:
                            break;
                        case CNUMDSTD_SpIBP2SYS:
                            break;
                        case CNUMDSTD_SpIBP2DIA:
                            break;
                        case CNUMDSTD_SpIBP2MAP:
                            break;
                        case CNUMDSTD_SpIBP2PR:
                            break;
                        case CNUMDSTD_SpIBP3SYS:
                            break;
                        case CNUMDSTD_SpIBP3DIA:
                            break;
                        case CNUMDSTD_SpIBP3MAP:
                            break;
                        case CNUMDSTD_SpIBP3PR:
                            break;
                        case CNUMDSTD_SpSPO2PR:
                            break;
                        case CNUMDSTD_SpSPO2SAT:
                            break;
                        case CNUMDSTD_SpSPO2PI:
                            break;
                        case CNUMDSTD_SpRESPRATE:
                            break;
                        case CNUMDSTD_SpRRBR:
                            break;
                        case CNUMDSTD_SpCO2ENDT:
                            break;
                        case CNUMDSTD_SpCO2INSP:
                            break;
                        case CNUMDSTD_SpCO2BR:
                            break;
                        case CNUMDSTD_SpTEMP1:
                            break;
                        case CNUMDSTD_SpTEMP2:
                            break;
                        case CNUMDSTD_SpDELTAT:
                            break;
                        case CNUMDSTD_SpALL:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmRECORD:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case RECORD_CCMD_SpWRITE_CONFIG:
                            break;
                        case RECORD_CCMD_SpWRITE_RECORD_COUNT:
                            break;
                        case RECORD_CCMD_SpRESET_COUNTS:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case RECORD_CRST_SpGET_RECORD_COUNT:
                            break;
                        case RECORD_CRST_SpGET_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case RECORD_CRSP_SpPUT_RECORD_COUNT:
                            break;
                        case RECORD_CRSP_SpPUT_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case RECORD_CSS_SpREPORT_RECORD_COUNT:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CRCDDSVC_SpSERVICE:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmWEIGHT:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case WEIGHT_CCMD_SpWRITE_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case WEIGHT_CRST_SpGET_WEIGHT:
                            break;
                        case WEIGHT_CRST_SpGET_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case WEIGHT_CRSP_SpPUT_WEIGHT:
						{
							/*Received a weight object response*/
							if(pBuffer != NULL)
							{
								/*Deserialize the object in the buffer*/
								CWeightDStd WeightObj;

								WeightObj.deserialize_all(pBuffer);

								/*Display the weight*/
								SYS_PRINTF("\nWeight: %d\n", WeightObj.GetWeight());
							}
							MsgResult = TRUE;
                            break;
						}
                        case WEIGHT_CRSP_SpPUT_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case WEIGHT_CSS_SpREPORT_WEIGHT:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CWGHDSTD_SpSTANDARD:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmCONNECTION:
        {
            MsgResult = TRUE;
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case CONN_CMD_SpSHUTDOWN:
                        {
                            SYS_PRINTF("\nRecvMessage: %s CONN_CMD_SpSHUTDOWN The session has been terminated\n", pSourceID);
                            RemoveDevice(pSourceID);
                            Close(pSourceID);
                            break;
                        }
                        case CONN_CMD_SpSTARTBYPASS:
                        {
                            CConDStd ConDStd;

                            if (pBuffer)
                            {
                                Result = ConDStd.deserialize_all(pBuffer);
                                if (Result)
                                {
                                    SYS_PRINTF("\nRecvMessage: %s CONN_CMD_SpSTARTBYPASS Received ByPass Start Command to %s\n",
                                                 pSourceID, ConDStd.GetSourceID(NULL));

                                    Result = SetByPass(pSourceID, ConDStd.GetSourceID(NULL));
                                    if (Result)
                                    {
                                        SendTrap(pSourceID, ClassID, ClassID, CERR_SpERRNONE, 0, NULL, 0);
                                    }
                                    else
                                    {
                                       SendMsgProcTrap(pSourceID, ClassID, CERR_PROC_INVL_PARAM);
                                    }
                                }
                            }

                            break;
                        }
                        case CONN_CMD_SpENDBYPASS:
                        {
                            SYS_PRINTF("\nRecvMessage: %s CONN_CMD_SpENDBYPASS Received ByPass End Command\n", pSourceID);
                            Result = SetByPass(pSourceID, NULL);
                            if (Result)
                            {
                                SendTrap(pSourceID, ClassID, ClassID, CERR_SpERRNONE, 0, NULL, 0);
                            }
                            break;
                        }
                        case CONN_CMD_SpBYPASSTERMINATED:
                        {
                            SYS_PRINTF("\nRecvMessage: %s CONN_CMD_SpBYPASSTERMINATED ByPass Terminated\n", pSourceID);
                            break;
                        }
                        default:
                        {
                            MsgResult = FALSE;
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            SYS_PRINTF("\nRecvMessage: %s Unknown Connection Command Type %d\n", pSourceID, Species);
                            break;
                        }
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case CONN_CRSP_SpPUTCONNLIST:
                        {
                            uint8*      pConDStd;
                            uint32      ObjSize = 0;

                            /* Reset the Record container */
                            ConnList.InitConnectionList(0);

                            /* Clear out old array list */
                            for (uint16 i=0; i < ConnCount != NULL; i++)
                            {
                                if (ConnDArray[i])
                                {
                                    delete(ConnDArray[i]);
                                    ConnDArray[i] = NULL;
                                }
                            }

                            /* Reset the count */
                            ConnCount = 0;

                            SYS_PRINTF("\nRecvMessage: %s CONN_CRSP_SpPUTCONNLIST\n", pSourceID);

                            Result = ConnList.deserialize_all(pBuffer);
                            if (Result != FALSE)
                            {
                                ConnCount = ConnList.m_ConnectionListElements;
                                for (uint16 i=0; i < ConnCount != NULL; i++)
                                {
                                    pConDStd = ConnList.GetConnectionListAt(i, &ObjSize) ;
                                    if (pConDStd)
                                    {
                                        ConnDArray[i] = new CConDStd();
                                        if (ConnDArray[i])
                                        {
                                            ConnDArray[i]->deserialize_all(pConDStd);
                                        }
                                    }
                                }

                                if (ConnCount == 0)
                                {
                                    SYS_PRINTF("\nRecvMessage: %s CONN_CRSP_SpPUTCONNLIST No connections were found\n", pSourceID);
                                }

                                CWACPEvent* pWaitEvent = new CWACPEvent(_T("ByPass"), FALSE);
                                pWaitEvent->Raise();
                                delete (pWaitEvent);
                                MsgResult = TRUE;
                            }
                            else
                                SYS_PRINTF("\nRecvMessage: %s Error: CONN_CRSP_SpPUTCONNLIST Unable to deserialize CConDExtLst buffer\n", pSourceID);

                            break;
                        }
                        default:
                        {
                            MsgResult = FALSE;
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            SYS_PRINTF("\nRecvMessage: %s Unknown Connection Response Type %d\n", pSourceID, Species);
                            break;
                        }
                    }
                    break;
                }
                case GnEVENT:
                {
                    switch(Species)
                    {
                        case CONN_CEVNT_SpCONNECTED:
                        {
                            bConnected = TRUE;

                            this->pSourceID = pSourceID;

                            SYS_PRINTF("\nRecvMessage: %s CONN_CEVNT_SpCONNECTED\n", pSourceID);

                            AddDevice(pSourceID);

                            ShowDevices();

                            break;
                        }
                        default: 
                        {
                            MsgResult = FALSE;
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            SYS_PRINTF("\nRecvMessage: %s Unknown Connection Event Type %d\n", pSourceID, Species);
                            break;
                        }
                    }
                    break;
                }
                default:
                {
                    MsgResult = FALSE;
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
   }

   /* Send a TRAP message if we have a unimplemented or unsupported error */
   if (!MsgResult)
   {
        SendMsgProcTrap(pSourceID, ClassID, ErrorCode);
   }

   return(Result);
}

/*================================================================================
RecvError : Handles Error messages. 
================================================================================*/
uint32 CFamilyStub::RecvError(char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8 *pBuffer)
{
    uint32 RetVal  = 0;
    uint16 Family  = GetFamily(ClassID);
    uint8  Genus   = GetGenus(ClassID);
    uint8  Species = GetSpecies(ClassID);

    if(ErrorType == CERR_SpERRNONE)
    {
        /* Handle simple ACK notification */
        SYS_PRINTF("\nRxError: %s ACK #%d F:%d G:%d S:%d\n", pSourceID, ++AckCount, Family, Genus, Species);
    }
    else
    {
        /* Handle simple NAK notification */
        SYS_PRINTF("\nRxError: %s NAK #%d F:%d G:%d S:%d\n", pSourceID, ++ErrCount, Family, Genus, Species);

        if(pBuffer != NULL)
        {
            CErrEStd ErrorObj;

            /* Deserialize the Error object and strip off the Family, Genus, and Species */
            RetVal = ErrorObj.deserialize_all(pBuffer);
            if(RetVal)
            {
                /* Handle NAKs */
                switch(ErrorType)
                {
                    case CERR_SpERRWACPSTACK:
                    case CERR_SpERRMSGPROC:
                    {
                        uint32  XMLSize;
                        char    *Temp;

                        if (ErrorType == CERR_SpERRWACPSTACK)
                            SYS_PRINTF("\nRxError: %s CERR_SpERRWACPSTACK error\n", pSourceID);
                        else
                            SYS_PRINTF("\nRxError: %s CERR_SpERRMSGPROC error\n", pSourceID);

                        XMLSize = ErrorObj.serialize_to_XMLbuf(NULL);
                        Temp = (char*)SYS_MALLOC(XMLSize);
                        if (Temp != NULL)
                        {
                            ErrorObj.serialize_to_XMLbuf(Temp);
                            SYS_PRINTF("%s\n", Temp);
                            SYS_FREE(Temp);
                            break;
                        }
                    }
                    case CERR_SpERRAPPINTERNAL:
                    {
                        uint16 DescBufSize = 0;
                        char *pTempBuf = ErrorObj.GetDescription(&DescBufSize);

                        if (pTempBuf)
                        {
                            uint32  XMLSize;
                            char    *Temp;

                            SYS_PRINTF("\nRxError: %s CERR_SpERRAPPINTERNAL error\n", pSourceID);

                            XMLSize = ErrorObj.serialize_to_XMLbuf(NULL);
                            Temp = (char*)SYS_MALLOC(XMLSize);
                            if (Temp != NULL)
                            {
                                ErrorObj.serialize_to_XMLbuf(Temp);
                                SYS_PRINTF("%s\n", Temp);
                                SYS_FREE(Temp);
                                break;
                            }
                        }
                        break;
                    }
                }/* switch(ErrorType) */
            }/* if(RetVal) */
        }/* if(Buffer != NULL) */
    }
    return RetVal;
}

/*================================================================================
AddDevice : Adds a device to the connection list
================================================================================*/
void CFamilyStub::AddDevice(char* pSourceID)
{
    if (pSourceID)
    {
        ConnStruct* pConnection = new ConnStruct;

        memset(pConnection->SourceID, 0, sizeof(pConnection->SourceID));
        memcpy(pConnection->SourceID, pSourceID, strlen(pSourceID));

        /* Add the new connection into the STL list */
        SourceList.push_back(pConnection);

        SYS_PRINTF("\nCFamilyStub::AddDevice %s Count = %d\n", pSourceID, SourceList.size());
    }
}

/*================================================================================
RemoveDevice : Removes a device from the connection list
================================================================================*/
void CFamilyStub::RemoveDevice(char* pSourceID)
{
    boolean Result = FALSE;

    if (pSourceID)
    {
        ConnStruct* pConnection;

        /* Find the queue for this message */
        CONNECTION_LIST::iterator pList;

        /* Search for the node that matches the pWACPAppRef */
        for (pList = SourceList.begin(); pList != SourceList.end(); pList++)
        {
            pConnection = *pList;
            if (strcmp(pSourceID, pConnection->SourceID)==0)
            {
                SYS_PRINTF("\nCFamilyStub::RemoveDevice %s Count = %d\n", pSourceID, SourceList.size() - 1);
                delete(pConnection);

                /* Erase the entry from the list */
                SourceList.erase(pList);
                Result = TRUE;
                break;
            }
        }
    }
}

/*================================================================================
ShowDevices : Shows all connections within the connection list
================================================================================*/
void CFamilyStub::ShowDevices()
{
    CONNECTION_LIST::iterator pList;
    uint32 i = 0;

    SYS_PRINTF("\n\nCurrent Connections:\n");
    /* Print the list of connections */
    for (pList = SourceList.begin(), i=0; pList != SourceList.end(); pList++, i++)
    {
        SYS_PRINTF("\t%d. %s\n", i+1, (*pList)->SourceID);
    }
    SYS_PRINTF("\n\n");
}

/*================================================================================
GetNumConnections : Gets the total number of connections within the connection list
================================================================================*/
uint16 CFamilyStub::GetNumConnections()
{
    return (uint16(SourceList.size()));
}

/*================================================================================
GetConnCount : Terminates an active client based on an index (1 based)
================================================================================*/
boolean CFamilyStub::TerminateConn(uint32 Index)
{
    CONNECTION_LIST::iterator pList;
    boolean Result = FALSE;
    uint32  i;

    /* Print the list of connections */
    for (pList = SourceList.begin(), i=1; pList != SourceList.end(); pList++, i++)
    {
        if (i==Index)
        {
            Close((*pList)->SourceID);
            RemoveDevice((*pList)->SourceID);
            Result = TRUE;
            break;
        }
    }

    return Result;
}

/*================================================================================
GetConnCount : Terminates all active connections
================================================================================*/
boolean CFamilyStub::TerminateAll()
{
    boolean Result = FALSE;
    uint32  Index;
    for (Index=(uint32)SourceList.size(); Index > 0; Index--)
    {
        Result = TerminateConn(Index);
    }

    return Result;
}

