<?xml version='1.0'?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html"/>

<xsl:template name="replaceChar">
	<xsl:param name="string" />
	<xsl:param name="character" />
	<xsl:param name="replaceString" />
	<xsl:choose>
		<xsl:when test="contains($string, $character)">
			<xsl:value-of select="substring-before($string, $character)" />
			<xsl:value-of select="$replaceString" />
			<xsl:call-template name="replaceChar">
				<xsl:with-param name="string" select="substring-after($string, $character)" />
				<xsl:with-param name="character" select="$character" />
				<xsl:with-param name="replaceString" select="$replaceString" />
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$string" />
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="/">
<html>
<head>
<title>Trace Log</title>
<style>
	.noBorders{	
		FONT-Size: 12px; 
		COLOR: #012d96; 
		FONT-FAMILY: arial;
		border-top-width:0;
		border-left-width:0;
		border-right-width:0;
		border-bottom-width:0
	}
	.button{
		border-style: solid; 
		border-color: #000000;
		border-width: 1;
		cursor: default;
		background-color:#c6cfe7;
	}
</style>
</head>
<SCRIPT>
	function toggleRows(check, type)
	{
		var entryCount = document.getElementById('entryTable').children[0].children.length;
		for(var i = 2; i &lt; entryCount; i++)
		{
			if(document.getElementById('entryTable').children[0].children[i].children[2].style.color == check)
			{
				if(document.getElementById('entryTable').children[0].children[i].style.display == 'none')
				{
					document.getElementById('entryTable').children[0].children[i].style.display = 'block';
				}
				else
				{
					document.getElementById('entryTable').children[0].children[i].style.display = 'none';
				}
			}
		}
		if(document.getElementById(type).innerText == 'Hide')
		{
			document.getElementById(type).innerText = 'Show';
		}
		else
		{
			document.getElementById(type).innerText = 'Hide';
		}
	}
</SCRIPT>
<body bgcolor="FFFFF0">
<table id="entryTable" class="noBorders" cellpadding="3" border="1" borderColor="#000000" width="100%">
	<tr>
	<td colspan="6" bgcolor="#d6dff7">
		<table class="noBorders" cellpadding="3" width="100%">
			<tr>
			<td width="180">
			<b>Unique Identifier for this Log:</b>
			</td>
			<td>
			"<xsl:value-of select="TRACE_LOG/UNIQUE_ID"/>"
			</td>
			<td width="160" align="center" class="button"
				onmouseover="this.style.backgroundColor='#FFFFFF';"
				onmouseout="this.style.backgroundColor='#c6cfe7';"
				onclick="toggleRows('#ff0000','txtErrors');">
			<span id="txtErrors">Hide</span> ERROR Entries
			</td>
			</tr>
			<tr>
			<td width="180" rowspan="3" valign="top">
			<b>Number of logged entries:</b>
			</td>
			<td rowspan="3" valign="top">
			<xsl:value-of select="count(TRACE_LOG/ENTRY)"/>
			</td>
			<td width="160" align="center" class="button"
				onmouseover="this.style.backgroundColor='#FFFFFF';"
				onmouseout="this.style.backgroundColor='#c6cfe7';"
				onclick="toggleRows('#009900','txtStatus');">
			<span id="txtStatus">Hide</span> STATUS Entries
			</td>
			</tr>
			<tr>
			<td width="160" align="center" class="button"
				onmouseover="this.style.backgroundColor='#FFFFFF';"
				onmouseout="this.style.backgroundColor='#c6cfe7';"
				onclick="toggleRows('#ee7700','txtRecv');">
			<span id="txtRecv">Hide</span> RECEIVE Entries
			</td>
			</tr>
			<tr>
			<td width="160" align="center" class="button"
				onmouseover="this.style.backgroundColor='#FFFFFF';"
				onmouseout="this.style.backgroundColor='#c6cfe7';"
				onclick="toggleRows('#0000aa','txtSend');">
			<span id="txtSend">Hide</span> SEND Entries
			</td>
			</tr>
		</table> 
	</td>
	</tr>
	<tr>
	<td align="center" bgcolor="#d6dff7" colspan="3" width="20%">Entry Type</td>
	<td align="center" bgcolor="#d6dff7" width="20%">Entry Origin</td>
	<td align="center" bgcolor="#d6dff7" width="10%">Class ID</td>
	<td align="center" bgcolor="#d6dff7" width="50%">Description</td>
	</tr>
<xsl:for-each select="TRACE_LOG/ENTRY">
	<tr>
		<xsl:attribute name="style">
			<xsl:if test="position() mod 2 &gt; 0">
				background-color:#DDDDD0;
			</xsl:if>
		</xsl:attribute>
	<td class="noBorders" align="center" valign="top" width="3%">
		<xsl:value-of select="position()"/>
	</td>
	<td class="noBorders" align="center" valign="top" width="7%">
		<xsl:value-of select="TIME"/>
	</td>
	<td class="noBorders" align="center" valign="top" width="10%">
		<xsl:attribute name="style">
			<xsl:choose>
				<xsl:when test="TYPE = 'ERROR'">
					color:#ff0000;
				</xsl:when>
				<xsl:when test="TYPE = 'STATUS'">
					color:#009900;
				</xsl:when>
				<xsl:when test="TYPE = 'RECEIVE'">
					color:#ee7700;
				</xsl:when>
				<xsl:when test="TYPE = 'SEND'">
					color:#0000aa;
				</xsl:when>
				<xsl:otherwise>
					color:#000000;
				</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>
		<xsl:value-of select="TYPE"/>
	</td>
	<td class="noBorders" valign="top" width="20%">
		<xsl:value-of select="CLASS_NAME"/>::
		<xsl:value-of select="FUNCTION_NAME"/>
	</td>
	<td class="noBorders" valign="top" width="10%">
		<xsl:value-of select="FAMILY"/> 
		: <xsl:value-of select="GENUS"/> 
		: <xsl:value-of select="SPECIES"/>
	</td>
	<td class="noBorders" valign="top" width="50%">
		<xsl:variable name="outputDesc">
			<xsl:call-template name="replaceChar">
				<xsl:with-param name="string">
					<xsl:call-template name="replaceChar">
						<xsl:with-param name="string">
							<xsl:call-template name="replaceChar">
								<xsl:with-param name="string" select="DESCRIPTION" />
								<xsl:with-param name="character" select="'&#x0a;'" />
								<xsl:with-param name="replaceString" select="'&lt;br/&gt;'" />
							</xsl:call-template>
						</xsl:with-param>
						<xsl:with-param name="character" select="'&#xae;'" />
						<xsl:with-param name="replaceString" select="'&lt;span style=&#x22;FONT-FAMILY: courier new, courier, terminal;&#x22;&gt;'" />
					</xsl:call-template>
				</xsl:with-param>
				<xsl:with-param name="character" select="'&#xaf;'" />
				<xsl:with-param name="replaceString" select="'&lt;/span&gt;'" />
			</xsl:call-template>
		</xsl:variable>
		<xsl:value-of select="$outputDesc" disable-output-escaping="yes" />
	</td>
	</tr>
</xsl:for-each>
</table>
</body>
</html>
</xsl:template>
</xsl:stylesheet>