/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmDEVICEStub.c                                                   //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CFmDEVICEStub.h"

char SourceID[32];

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
boolean CFmDEVICEStub_RecvMsg(const char *pSourceID, uint8 Genus, uint8 Species, uint8* Buffer)
{
    boolean MsgResult  = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
	uint16  ErrorType  = CERR_PROC_UNIMPL;
    uint32  MsgClassID = MakeClassID(FmDEVICE, Genus, Species);
	uint8   ObjBuffer[128];


    TOUCH(Buffer);

    switch(Genus)
    {
        case GnCOMMAND:
        {
            switch(Species)
            {
                case DEVICE_CCMD_SpWRITE_CONFIG:
                    break;
                case DEVICE_CCMD_SpWRITE_DEVICEDESCRIPTION:
                    break;
                case DEVICE_CCMD_SpENABLE_DISPLAY:
                    break;
                case DEVICE_CCMD_SpSTART_DISPLAY_TEST:
                    break;
                case DEVICE_CCMD_SpRESET:
                    break;
                case DEVICE_CCMD_SpPOWER:
                    break;
                case DEVICE_CCMD_SpBEEP:
                    break;
                case DEVICE_CCMD_SpSAVE_CONFIG:
                    break;
                case DEVICE_CCMD_SpRESTORE_CONFIG:
                    break;
                case DEVICE_CCMD_SpENABLE_AUTO_POWER_DOWN:
                    break;
                case DEVICE_CCMD_SpSET_DISPLAY_CONTRAST:
                    break;
                case DEVICE_CCMD_SpSET_BACKLIGHT:
                    break;
                case DEVICE_CCMD_SpSTART_DIGITAL_TEST:
                    break;
                case DEVICE_CCMD_SpSTOP_DISPLAY_TEST:
                    break;
                case DEVICE_CCMD_SpSET_DEVICE_MODE:
                    break;
                case DEVICE_CCMD_SpSTART_PRINTER_TEST:
                    break;
                case DEVICE_CCMD_SpSET_CALIBRATION:
                    break;
                case DEVICE_CCMD_SpSET_DATE_TIME:
                    break;
                case DEVICE_CCMD_SpNEXT_PATIENT:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnREQUEST:
        {
            switch(Species)
            {
                case DEVICE_CRST_SpGET_DEVICEDESCRIPTION:
				{				
					CDeviceDStd DeviceDStd;
					CDevPMsgArguments SpeciesSelector;
					MsgResult = TRUE;
					ErrorCode = CERR_NONE;

					if(Buffer != NULL)
					{
						CDevPMsgArguments_Init(&SpeciesSelector);

						if(CDevPMsgArguments_Deserialize(&SpeciesSelector, Buffer))
						{
							CDeviceDStd_Init(&DeviceDStd);

							switch (SpeciesSelector.m_SpeciesSelector)
							{
								case CDEVDSTD_SpSENSOR:
								{
									/*Create object*/
									GetCDeviceDStd(&DeviceDStd);
									break;
								}
								default:
								{
									/*Invalid Paramter*/
									ErrorCode = CERR_PROC_INVL_PARAM;
									ErrorType = CERR_SpERRMSGPROC;
									break;
								}
							}							
						}
						else
						{
							/*Deserialize Failure*/
							ErrorCode = CERR_PROC_OBJECT_DESERIALIZE;
							ErrorType = CERR_SpERRMSGPROC;
						}
					}
					else
					{
						/*If no payload is used, continue creating the Device object*/
						GetCDeviceDStd(&DeviceDStd);
					}

					/*Send object if there is no error code*/
					if(ErrorCode == CERR_NONE)
					{
						 /* Serialize object */
						if (CDeviceDStd_Serialize(&DeviceDStd, ObjBuffer))
						{
							/*Send Message*/
							SendMsg(MakeClassID(FmDEVICE,						/*Family*/
										 GnRESPONSE,							/*Genus*/
										 DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION),	/*Species*/
										 ObjBuffer,								/*Serialized Object*/
										 CDeviceDStd_SizeAll(&DeviceDStd),   	/*Size of object*/
										 0);           							/*Not used*/

						}
						else
						{
							/*Serialization Failure*/
							ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
							ErrorType = CERR_SpERRMSGPROC;
						}
					}

					/*Send a NAK if there is an ErrorCode*/
					else
					{
						 SendTrap(ErrorType,		/*Trap ErrorType*/
                                  MsgClassID,		/*ClassID of the message that is NAK'd*/
								  ErrorCode);		/*Error Code: Not used*/
					}
                    break;
				}
                    break;
                case DEVICE_CRST_SpGET_CONFIG:
                    break;
                case DEVICE_CRST_SpGET_POST:
				{				
					CDevDPOST DevDPOST;
					MsgResult = TRUE;
					ErrorCode = CERR_NONE;

					GetCDevDPOST(&DevDPOST);

					/* Serialize object */
					if (CDevDPOST_Serialize(&DevDPOST, ObjBuffer))
					{
						/*Send Message*/
						SendMsg(MakeClassID(FmDEVICE,						/*Family*/
									 GnRESPONSE,							/*Genus*/
									 DEVICE_CRSP_SpPUT_POST),				/*Species*/
									 ObjBuffer,								/*Serialized Object*/
									 CDevDPOST_SizeAll(&DevDPOST),   	    /*Size of object*/
									 0);           							/*Not used*/

					}
					else
					{
						/*Serialization Failure*/
						ErrorCode = CERR_PROC_OBJECT_SERIALIZE;
						ErrorType = CERR_SpERRMSGPROC;
					}
					

					/*Send a NAK if there is an ErrorCode*/
					if(ErrorCode != CERR_NONE)
					{
						 SendTrap(ErrorType,		/*Trap ErrorType*/
                                  MsgClassID,		/*ClassID of the message that is NAK'd*/
								  ErrorCode);		/*Error Code: Not used*/
					}
                    break;
				}
                case DEVICE_CRST_SpGET_DIGITAL_STATE:
                    break;
                case DEVICE_CRST_SpGET_DEVICE_MODE:
                    break;
                case DEVICE_CRST_SpGET_DEVICE_INVENTORY:
                    break;
                case DEVICE_CRST_SpGET_CALIBRATION:
                    break;
                case DEVICE_CRST_SpGET_DATE_TIME:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnRESPONSE:
        {
            switch(Species)
            {
                case DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION:
                {
                    SYS_PRINTF("Device Description Received\n");
                    MsgResult = TRUE;
                
                    break;
                }
                case DEVICE_CRSP_SpPUT_CONFIG:
                    break;
                case DEVICE_CRSP_SpPUT_POST:
                    break;
                case DEVICE_CRSP_SpPUT_DIGITAL_STATE:
                    break;
                case DEVICE_CRSP_SpPUT_DEVICE_MODE:
                    break;
                case DEVICE_CRSP_SpPUT_DEVICE_INVENTORY:
                    break;
                case DEVICE_CRSP_SpPUT_CALIBRATION:
                    break;
                case DEVICE_CRSP_SpPUT_DATE_TIME:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        case GnDATA:
        {
            switch(Species)
            {
                case CDEVDSTD_SpSTANDARD:
                    break;
                case CDEVDSTD_SpHOST:
                    break;
                case CDEVDSTD_SpSENSOR:
                    break;
                case CDEVDSTD_SpALL:
                    break;
                case CDEVDPST_SpPOST:
                    break;
                default:
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

    /* Send a TRAP message if we have a unimplemented or unsupported error */
    if (!MsgResult)
    {
		SendTrap(CERR_SpERRMSGPROC, MsgClassID, 0);
    }

    return MsgResult;
}

boolean GetCDeviceDStd(CDeviceDStd* pObject)
{
	boolean bResult = FALSE;

	if(pObject)
	{
		bResult = TRUE;

		/*DeviceGUID does not need to be set here because CDeviceDStd_Init() does it*/

		//pObject->m_DateTime - To Do: Contains the manufacture date time, not sure if this will be provided. 
		memcpy(pObject->m_ModelName, "ToDo", strlen("ToDo"));
		memcpy(pObject->m_SerialNumber, "ToDo", strlen("ToDo"));
		memcpy(pObject->m_ModelNumber, "ToDo", strlen("ToDo"));

		pObject->m_RunTime = 0;

	}

	return bResult;
}

boolean GetCDevDPOST(CDevDPOST* pObject)
{
	boolean bResult = FALSE;
	uint8   ObjBuffer[128];

	if(pObject)
	{
		CErrEStd Error;
		bResult = TRUE;
		CDevDPOST_Init(pObject);

		/*Set the GUID ID*/
		memcpy(pObject->m_DeviceGUID, DeviceGUID, sizeof(DeviceGUID));

		/*Set source ID*/
		memset(SourceID, 0, sizeof(SourceID));
		memcpy(SourceID, "ToDo", strlen("ToDo"));//To do; Implemenet source id.
		pObject->m_SourceID = &SourceID;
		pObject->m_SourceIDSize = strlen("ToDo")+1;

		/*Check the POST state for failure*/
		if(PostState == CDEVDPST_POSTStatus_FAILURE)
		{
			/*Populate error object*/
			Error.m_ErrorCode = CERR_ERROR_COMMON_NONE; //To do; Implement ErrorCode for POST failures.
			memcpy(Error.m_Description, "Error", strlen("Error"));

			/*Set the post failure status in the CDevDPOST object*/
			pObject->m_POSTStatus = CDEVDPST_POSTStatus_FAILURE;

			/*Serialize the error object*/
			CErrEStd_Serialize(&Error, ObjBuffer);

			/*Initiate the error structure inside the CDevDPOST object and append the error object*/
			CDevDPOST_InitErrorList(pObject->m_ErrorList, 0);
			CDevDPOST_AppendErrorList(&pObject, ObjBuffer, CErrEStd_SizeAll(&Error), 1);
		}
		else if(PostState == CDEVDPST_POSTStatus_IN_PROGRESS)
		{
			pObject->m_POSTStatus = CDEVDPST_POSTStatus_IN_PROGRESS; 
		}
		else if(PostState == CDEVDPST_POSTStatus_COMPLETE)
		{
			pObject->m_POSTStatus = CDEVDPST_POSTStatus_COMPLETE; 
		}
		else
		{
			pObject->m_POSTStatus = CDEVDPST_POSTStatus_NONE; 
		}

	}

	return bResult;
}

