/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CMOMPCallBack.c                                                   //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.8.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/
#include "StdVarTyp.h"
#include "CErrEStd.h"
#include "CFamilyEnum.h"
#include "Wacp.h"
//#include "CSerialCommSvr.h"
//#include "CEthCommSvr.h"

#include "CMOMPCallBack.h"
#include "CFmTRAPStub.h"
#include "CFmCONNECTIONStub.h"
#include "CFmDEVICEStub.h"
#include "CFmERRORStub.h"
#include "CFmMODULEStub.h"
#include "CFmWAVESAMPLEStub.h"
#include "CFmRNDZStub.h"
#include "CFmDATATRANSFERStub.h"
#include "CFmFILESYSStub.h"
#include "CFmFLASHStub.h"
#include "CFmCO2Stub.h"
#include "CFmNUMERICStub.h"
#include "CFmSPO2Stub.h"
#include "CFmRECORDStub.h"


#if defined STACK_SERIAL
extern CSerialCommSvr CommSvr;
#elif defined STACK_ETHERNET
extern CEthCommSvr CommSvr;
#endif
uint8 OutBuffer[MAX_PACKET_SIZE];

/*================================================================================
SendMsg: Encapuslates the optional serialized object into the a message
envelope and sends out of the transport.

Parameters:
    MsgClassID:    32 Bit value that contains the Family, Genus, and Species of 
                   the outgoing message. 
    pBuf:          Pointer to the binary MOIB object stream. This may be NULL.
    BufSize:       Size of the buffer pointed to by pBuf. This size may be 0.
    ErrClassID:    ClassID of the message that is being ACK'd or NAK'd. This
                   may be 0 for non-trap messages
Returns: 

    TRUE - If the message was successfully sent, otherwise FALSE.
================================================================================*/
extern void WriteSerialData (uint8 const* pBuf, uint32 BufSize);

uint32 SendMsg(uint32 MsgClassID, uint8* pBuf, uint32 BufSize, uint32 ErrClassID)
{
    uint32 Result = 0;
    uint32 OutSize = 0;

    Result = GeneratePacket(OutBuffer, &OutSize, pBuf, BufSize, MsgClassID, ErrClassID);

	if (Result == WACP_SUCCESS)
    {
		WriteSerialData (OutBuffer, OutSize);
/*		
#if defined STACK_SERIAL
        CSerialCommSvr_SendMessage(&CommSvr, OutBuffer, OutSize);
#elif STACK_ETHERNET
        CEthCommSvr_SendMessage(&CommSvr, OutBuffer, OutSize);
#endif
*/
    }

    return Result;
}
/*================================================================================
SendTrap: Encapuslates an optional CErrEStd serialized object for NAK error types into the a message
envelope and sends out of the transport.

Parameters:
    ErrType:       The Trap type. Also known as the error species
				   (ACK = CERR_SpERRNONE (0), NAK is a value greater than zero. See CErrEStd)    
    ErrClassID:    ClassID of the message that is being ACK'd or NAK'd. 
	ErrCode:	   The specific Error code for the NAK type.

Returns: 

    TRUE - If the message was successfully sent, otherwise FALSE.
================================================================================*/
uint32 SendTrap(uint8 ErrType, uint32 ErrClassID, uint8 ErrorCode)
{
    uint32 Result = 0;
//    uint32 OutSize = 0;
	CErrEStd ErrorObj;

    uint8 Buf[128];
	uint8* pErrBuf = NULL;
    uint32 ErrSize = 0;
	uint32 MsgClassID = MakeClassID(FmTRAP, GnERROR, ErrType);

	CErrEStd_Init(&ErrorObj);

	ErrorObj.ClassID = MakeClassID(FmERROR, GnERROR, ErrType);

	pErrBuf = Buf;

	/*Set the ClassID of the message/object that caused the error*/
	ErrorObj.m_ErrClassID = ErrClassID;

	/*Set the ErrorCode*/
	ErrorObj.m_ErrorCode = ErrorCode;

	/*Retrieve the size of the serialized object*/
	ErrSize = CErrEStd_SizeAll(&ErrorObj);

	/*Serialize the object into the buffer*/
	CErrEStd_Serialize(&ErrorObj, pErrBuf);
	
	Result = SendMsg(MsgClassID,			 /*Trap MsgClassID.*/
                     pErrBuf,                /*Serialized Error object*/
                     ErrSize,                /*Size of serialized  Error object*/
                     ErrClassID);            /*ClassID of the message that is ACK/NAK*/
    
    return Result;
}
/*================================================================================
RecvMessage: Main callback method for receiving a WACP Data or connection message.
    
Parameters:
    pSourceID:     SourceID of the incoming message.
    ClassID:       32 Bit value that contains the Family, Genus, and Species of 
                   the incoming message. 
    pBuffer:       Pointer to the binary MOIB object stream. This may be NULL.
    BufSize        Size of the buffer pointed to by pBuffer. This size may be 0.

Returns: 

    TRUE - If the message was successfully processed, otherwise FALSE.

================================================================================*/
boolean RecvMessage(const char *pSourceID, uint32 ClassID, uint8 *pBuffer, uint32 BufSize)
{
    boolean bResult = FALSE;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint16  Family  = (uint16)(ClassID >> 16);
    uint8   Genus = (uint8)(ClassID >> 8);
    uint8   Species = (uint8)(ClassID);

    TOUCH(BufSize);
    TOUCH(pBuffer);

    switch(Family)
    {
        case FmDEVICE:
        {
            bResult = CFmDEVICEStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmERROR:
        {
            bResult = CFmERRORStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmMODULE:
        {
            bResult = CFmMODULEStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmWAVESAMPLE:
        {
            bResult = CFmWAVESAMPLEStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmDATATRANSFER:
        {
            bResult = CFmDATATRANSFERStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmFILESYS:
        {
            bResult = CFmFILESYSStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmFLASH:
        {
            bResult = CFmFLASHStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmCO2:
        {
            bResult = CFmCO2Stub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmNUMERIC:
        {
            bResult = CFmNUMERICStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmSPO2:
        {
            bResult = CFmSPO2Stub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmRECORD:
        {
            bResult = CFmRECORDStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
        case FmCONNECTION:
        {
            bResult = CFmCONNECTIONStub_RecvMsg(pSourceID, Genus, Species, pBuffer);
            break;
        }
		case FmRENDEZVOUS:
		{
            bResult = CFmRNDZStub_RecvMsg(pSourceID, Genus, Species, pBuffer);            
            break;

		}
        default:
        {
            ErrorCode = CERR_PROC_UNIMPL;
            SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRMSGPROC), NULL, 0, ClassID);
            break;
        }
    }

    return bResult;
}

/*================================================================================
RecvError: Main interface method for receiving an Error message.
================================================================================*/
boolean RecvError(const char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8 *pBuffer, uint32 BufSize)
{
	TOUCH(BufSize);
    boolean bResult = CFmTRAPStub_RecvMsg(pSourceID, ErrorType, ClassID, pBuffer);
    return bResult;
}

